(function() {
  var $, CDN, Options, Promise, hasStyle, loader, _;

  Promise = require('es6-promise').Promise;

  _ = require('underscore');

  $ = require('jquery');

  Options = require('./options');

  CDN = {
    server: 'http://cdn.intermine.org',
    imtables: '/js/intermine/im-tables/2.0.0-beta/',
    tests: {
      fontawesome: /font-awesome/,
      glyphicons: /bootstrap-icons/
    },
    resources: {
      prettify: ['/js/google-code-prettify/latest/prettify.js'],
      d3: '/js/d3/3.0.6/d3.v3.min.js',
      glyphicons: "/css/bootstrap/2.3.2/css/bootstrap-icons.css",
      fontawesome: "/css/font-awesome/4.x/css/font-awesome.min.css",
      filesaver: '/js/filesaver.js/FileSaver.min.js'
    }
  };

  Options.set('CDN', CDN);

  hasStyle = function(pattern) {
    var links;
    if (pattern == null) {
      return false;
    }
    links = _.asArray(document.querySelectorAll('link[rel="stylesheet"]'));
    return _.any(links, function(link) {
      return pattern.test(link.href);
    });
  };

  loader = function(server) {
    return function(resource, resourceRegex) {
      var fetch, link, resolution;
      resolution = new Promise(function(resolve) {
        return _.delay(resolve, 50, true);
      });
      if (/\.css$/.test(resource)) {
        if (hasStyle(resourceRegex)) {
          return resolution;
        }
        link = $('<link type="text/css" rel="stylesheet">');
        link.appendTo('head').attr({
          href: server + resource
        });
        return resolution;
      } else {
        fetch = $.ajax({
          url: server + resource,
          cache: true,
          dataType: 'script'
        });
        return fetch.then(function() {
          return resolution;
        });
      }
    };
  };

  exports.load = function(ident) {
    var c, conf, load, server, test;
    server = Options.get('CDN.server');
    conf = Options.get(['CDN', 'resources', ident]);
    test = Options.get(['CDN', 'tests', ident]);
    load = loader(server);
    if (!conf) {
      return Promise.reject("No resource is configured for " + ident);
    } else if (_.isArray(conf)) {
      return Promise.all((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = conf.length; _i < _len; _i++) {
          c = conf[_i];
          _results.push(load(c));
        }
        return _results;
      })());
    } else {
      return load(conf, test);
    }
  };

}).call(this);
