(function() {
  var Model, NestedModel, isPlainObject, mergeOldAndNew, setSection, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __slice = [].slice;

  _ = require('underscore');

  Model = require('../core-model');

  mergeOldAndNew = function(oldValue, newValue) {
    return _.extend({}, (_.isObject(oldValue) ? oldValue : {}), newValue);
  };

  setSection = function(m, k) {
    var v;
    v = m[k];
    if (!_.isObject(v)) {
      return m[k] = {};
    } else {
      return v;
    }
  };

  isPlainObject = function(value) {
    return (_.isObject(value)) && (!_.isFunction(value)) && (!_.isArray(value));
  };

  module.exports = NestedModel = (function(_super) {
    __extends(NestedModel, _super);

    function NestedModel() {
      return NestedModel.__super__.constructor.apply(this, arguments);
    }

    NestedModel.prototype._triggerChangeRecursively = function(ns, obj) {
      var k, thisKey, v, _results;
      _results = [];
      for (k in obj) {
        v = obj[k];
        thisKey = "" + ns + "." + k;
        if (_.isObject(v)) {
          _results.push(this._triggerChangeRecursively(thisKey, v));
        } else {
          _results.push(this.trigger("change:" + thisKey, this, this.get(thisKey)));
        }
      }
      return _results;
    };

    NestedModel.prototype.get = function(key) {
      var head, tail;
      if (_.isArray(key)) {
        head = key[0], tail = 2 <= key.length ? __slice.call(key, 1) : [];
        return tail.reduce((function(m, k) {
          return m && m[k];
        }), NestedModel.__super__.get.call(this, head));
      } else if (/\w+\.\w+/.test(key)) {
        return this.get(key.split(/\./));
      } else {
        return NestedModel.__super__.get.call(this, key);
      }
    };

    NestedModel.prototype.pick = function() {
      var attrs;
      attrs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return _.object(_.flatten(attrs).map((function(_this) {
        return function(a) {
          return [a, _this.get(a)];
        };
      })(this)));
    };

    NestedModel.prototype._triggerPathChange = function(key, value) {
      var path, section, _i, _len, _results;
      path = [];
      this.trigger("change", this, key.join('.'), value);
      _results = [];
      for (_i = 0, _len = key.length; _i < _len; _i++) {
        section = key[_i];
        path.push(section);
        _results.push(this.trigger("change:" + (path.join('.')), this, this.get(path)));
      }
      return _results;
    };

    NestedModel.prototype._triggerUnsetPath = function(path, prev) {
      var k, v, _results;
      if (_.isObject(prev)) {
        _results = [];
        for (k in prev) {
          v = prev[k];
          _results.push(this._triggerUnsetPath(path.concat([k]), v));
        }
        return _results;
      } else {
        return this.trigger("change:" + (path.join('.')));
      }
    };

    NestedModel.prototype.set = function(key, value) {
      var currentValue, end, head, headVal, k, mid, prev, root, v, _i, _results;
      if (key == null) {
        throw new Error("No key");
      }
      if (_.isArray(key)) {
        if (isPlainObject(value)) {
          for (k in value) {
            v = value[k];
            this.set(key.concat([k]), v);
          }
          return;
        }
        head = key[0], mid = 3 <= key.length ? __slice.call(key, 1, _i = key.length - 1) : (_i = 1, []), end = key[_i++];
        headVal = this.get(head);
        if (headVal && (_.isFunction(headVal) || (!_.isObject(headVal)))) {
          this.unset(head);
        }
        root = headVal != null ? headVal : {};
        currentValue = mid.reduce(setSection, root);
        prev = currentValue[end];
        currentValue[end] = value;
        NestedModel.__super__.set.call(this, head, root);
        this._triggerPathChange(key, value);
        if ((prev != null) && (value == null)) {
          return this._triggerUnsetPath(key, prev);
        }
      } else if (_.isString(key)) {
        if (/\w+\.\w+/.test(key)) {
          return this.set(key.split(/\./), value);
        } else if (_.isObject(value) && !_.isArray(value)) {
          return this.set([key], value);
        } else {
          return NestedModel.__super__.set.apply(this, arguments);
        }
      } else {
        _results = [];
        for (k in key) {
          v = key[k];
          _results.push(this.set(k, v));
        }
        return _results;
      }
    };

    return NestedModel;

  })(Model);

}).call(this);
