(function() {
  var ArrayOf, Backbone, Callable, CoreCollection, CoreModel, DataModel, InstanceOfAssertion, InterMineService, IsArray, StringType, StructuralTypeAssertion, _,
    __slice = [].slice;

  _ = require('underscore');

  Backbone = require('backbone');

  CoreModel = require('../core-model');

  CoreCollection = require('./collection');

  exports.assertMatch = function(assertion, value, paramName) {
    var isOk;
    if (paramName == null) {
      paramName = 'value';
    }
    isOk = assertion.test(value);
    if (!isOk) {
      throw new Error(assertion.message(paramName));
    }
  };

  InstanceOfAssertion = (function() {
    function InstanceOfAssertion(type, name) {
      this.type = type;
      this.name = name;
      if (!(this instanceof InstanceOfAssertion)) {
        return new InstanceOfAssertion(this.type, this.name);
      }
    }

    InstanceOfAssertion.prototype.test = function(m) {
      return m instanceof this.type;
    };

    InstanceOfAssertion.prototype.message = function(p) {
      return "" + p + " is not an instance of " + this.name;
    };

    return InstanceOfAssertion;

  })();

  StructuralTypeAssertion = (function() {
    function StructuralTypeAssertion(name, structure) {
      this.name = name;
      this.structure = structure;
      this._msg = null;
    }

    StructuralTypeAssertion.prototype.failValidation = function(msg) {
      this._msg = msg;
      return false;
    };

    StructuralTypeAssertion.prototype.test = function(v) {
      var prop, propIsOk, propName, subtest, _ref;
      this._msg = null;
      if (v == null) {
        return this.failValidation('it is null');
      }
      if (v == null) {
        return false;
      }
      _ref = this.structure;
      for (propName in _ref) {
        subtest = _ref[propName];
        prop = v[propName];
        propIsOk = subtest.test(prop);
        if (!propIsOk) {
          return this.failValidation(subtest.message("." + propName));
        }
      }
      return true;
    };

    StructuralTypeAssertion.prototype.message = function(p) {
      return "" + p + " failed " + this.name + " validation, because " + this._msg;
    };

    return StructuralTypeAssertion;

  })();

  exports.InstanceOf = InstanceOfAssertion;

  exports.Structure = StructuralTypeAssertion;

  exports.Model = new InstanceOfAssertion(Backbone.Model, 'Backbone.Model');

  exports.CoreModel = new InstanceOfAssertion(CoreModel, 'core-model');

  exports.CoreCollection = new InstanceOfAssertion(CoreCollection, 'core/collection');

  exports.Array = IsArray = {
    name: 'array',
    test: _.isArray,
    message: function(p) {
      return "" + p + " is not an array";
    }
  };

  exports.ArrayOf = ArrayOf = function(elem) {
    return {
      name: "array of " + elem.name,
      test: function(v) {
        return this.passed = (IsArray.test(v)) && (_.all(v, elem.test));
      },
      message: function(p) {
        var _ref;
        return "" + p + " is not an array of " + ((_ref = elem.name) != null ? _ref : 'the correct type');
      }
    };
  };

  exports.Function = {
    name: 'function',
    test: _.isFunction,
    message: function(p) {
      return "" + p + " is not a function";
    }
  };

  exports.Number = {
    name: 'number',
    test: _.isNumber,
    message: function(p) {
      return "" + p + " is not a number";
    }
  };

  exports.String = StringType = {
    name: 'string',
    test: _.isString,
    message: function(p) {
      return "" + p + " is not a string";
    }
  };

  exports.Maybe = function(assertion) {
    return {
      name: "maybe " + assertion.name,
      test: function(v) {
        return (v == null) || (assertion.test(v));
      },
      message: function(p) {
        return assertion.message(p);
      }
    };
  };

  exports.Callable = Callable = {
    name: 'callable',
    test: function(v) {
      return (v != null) && _.isFunction(v.call);
    },
    message: function(p) {
      return "" + p + " is not callable";
    }
  };

  exports.HasProperty = function(prop) {
    return {
      name: "has-property(" + prop + ")",
      test: function(v) {
        return (v != null) && prop in v;
      },
      message: function(p) {
        return "" + p + "." + prop + " not found";
      }
    };
  };

  exports.UnionOf = function() {
    var types;
    types = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    return {
      name: function() {
        return "Union of " + (types.map(function(t) {
          return t.name;
        }).join(', '));
      },
      test: function(v) {
        return _.all(types, (function(_this) {
          return function(t) {
            _this._t = t;
            return t.test(v);
          };
        })(this));
      },
      message: function(p) {
        var _ref;
        return (_ref = this._t) != null ? _ref.message(p) : void 0;
      }
    };
  };

  exports.Listenable = new StructuralTypeAssertion('Listenable', {
    on: exports.Function,
    off: exports.Function
  });

  exports.Service = InterMineService = new StructuralTypeAssertion('Service', {
    root: StringType,
    query: Callable,
    fetchModel: Callable,
    fetchLists: Callable,
    count: Callable,
    whoami: Callable
  });

  exports.DataModel = DataModel = new StructuralTypeAssertion('imjs.Model', {
    name: StringType,
    makePath: Callable,
    findCommonType: Callable
  });

  exports.Query = new StructuralTypeAssertion('Query', {
    service: InterMineService,
    model: DataModel,
    rows: Callable,
    count: Callable,
    toXML: Callable,
    root: StringType,
    views: ArrayOf(StringType)
  });

}).call(this);
