(function() {
  var NestedModel, Templates, bool, enableFormatter, formatters, getAncestors, getFormatter, lastCd, _;

  _ = require('underscore');

  Templates = require('./templates');

  NestedModel = require('./core/nested-model');

  lastCd = function(path) {
    if (path.isAttribute()) {
      return path.getParent().getType();
    } else {
      return path.getType();
    }
  };

  getAncestors = function(path) {
    var cd;
    cd = lastCd(path);
    return [cd.name].concat(path.model.getAncestorsOf(cd));
  };

  bool = function(x) {
    return !!x;
  };

  formatters = new NestedModel;

  exports.getFormatter = getFormatter = function(path) {
    var a, ancestors, fieldName, formatter, formattersFor, model, _i, _len, _ref;
    if ((path == null) || path.isRoot()) {
      throw new Error('No path or path is root');
    }
    model = path.model;
    formattersFor = (_ref = formatters.get([model.name])) != null ? _ref : {};
    ancestors = getAncestors(path);
    fieldName = path.end.name;
    for (_i = 0, _len = ancestors.length; _i < _len; _i++) {
      a = ancestors[_i];
      formatter = formattersFor["" + a + "." + fieldName] || formattersFor["" + a + ".*"];
      if (formatter === true) {
        formatter = formattersFor[a];
      }
      if (formatter != null) {
        return formatter;
      }
    }
    return null;
  };

  exports.shouldFormat = function(path) {
    if (path == null) {
      throw new Error('no path');
    }
    if (!path.isAttribute()) {
      return false;
    }
    return bool(getFormatter(path));
  };

  exports.registerFormatter = function(formatter, model, type, paths) {
    if (paths == null) {
      paths = ['*'];
    }
    if ((formatter != null ? formatter.call : void 0) == null) {
      throw new Error('formatter is not a function');
    }
    if (paths.length === 1 && paths[0] !== '*') {
      return formatters.set([model, "" + type + "." + paths[0]], formatter);
    } else {
      formatters.set([model, type], formatter);
      return enableFormatter(model, type, paths);
    }
  };

  exports.disableFormatter = function(model, type, paths) {
    var p, _i, _len, _results;
    if (paths == null) {
      paths = ['*'];
    }
    _results = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      p = paths[_i];
      _results.push(formatters.set([model, "" + type + "." + p], false));
    }
    return _results;
  };

  exports.list = function(model) {
    var key, value, _ref, _results;
    _ref = formatters.get([model]);
    _results = [];
    for (key in _ref) {
      value = _ref[key];
      if (value) {
        _results.push(key);
      }
    }
    return _results;
  };

  exports.enableFormatter = enableFormatter = function(model, type, paths) {
    var p, _i, _len, _results;
    if (paths == null) {
      paths = ['*'];
    }
    _results = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      p = paths[_i];
      _results.push(formatters.set([model, "" + type + "." + p], true));
    }
    return _results;
  };

  exports.defaultFormatter = function(imobject, service, value) {
    if (value != null) {
      return _.escape(value);
    } else {
      return Templates.null_value;
    }
  };

  exports.reset = function() {
    formatters.clear();
    return formatters.trigger('reset');
  };

}).call(this);
