(function() {
  var Dashboard, Formatting, Messages, Options, Promise, Table, Types, asElement, connect, createView, load, simpleFormatter, version, _;

  _ = require('underscore');

  require('./shim');

  connect = require('imjs').Service.connect;

  Promise = require('es6-promise').Promise;

  version = require('./version');

  Types = require('./core/type-assertions');

  Dashboard = require('./views/dashboard');

  Table = require('./views/table');

  Options = require('./options');

  Messages = require('./messages');

  Formatting = require('./formatting');

  simpleFormatter = require('./utils/simple-formatter');

  load = function(create) {
    var loadView;
    return loadView = function(elem, opts, queryDef) {
      var conn, query, service;
      if (Types.Query.test(queryDef)) {
        return new Promise(createView(create, elem, queryDef, opts));
      } else {
        service = queryDef.service, query = queryDef.query;
        conn = Types.Service.test(service) ? service : connect(service);
        return conn.query(query).then(_.partial(loadView, elem, opts));
      }
    };
  };

  createView = function(create, elem, query, opts) {
    if (opts == null) {
      opts = {};
    }
    return function(resolve) {
      var c, element, model, view, _i, _len, _ref;
      element = asElement(elem);
      model = typeof page !== "undefined" && page !== null ? _.pick(opts, 'start', 'size') : null;
      opts = _.extend(_.omit(opts, 'start', 'size'), {
        model: model,
        query: query
      });
      view = create(opts);
      view.setElement(element);
      _ref = (_.result(view, 'className')).split(' ');
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        c = _ref[_i];
        element.classList.add(c);
      }
      element.classList.add(Options.get('StylePrefix'));
      view.render();
      return resolve(view);
    };
  };

  asElement = function(e) {
    var ret, _ref;
    if (e == null) {
      throw new Error('No target element provided');
    }
    ret = _.isString(e) ? document.querySelector(e) : (_ref = e[0]) != null ? _ref : e;
    if (ret == null) {
      throw new Error("Target(" + e + ") not found on page");
    }
    return ret;
  };

  exports.version = version;

  exports.setMessages = Messages.set.bind(Messages);

  exports.configure = Options.set.bind(Options);

  exports.loadTable = load(Table.create);

  exports.loadDash = load(function(opts) {
    return new Dashboard(opts);
  });

  exports.createFormatter = simpleFormatter;

  exports.formatting = _.omit(Formatting, 'getFormatter', 'shouldFormat', 'reset');

}).call(this);
