(function() {
  var CACHE, _;

  _ = require('underscore');

  CACHE = {};

  exports.runQuery = function(overrides) {
    var endpoint, format, key, params, _ref;
    if (overrides == null) {
      overrides = {};
    }
    params = this.getExportParameters(overrides);
    key = "results:" + this.query.service.root + ":" + (JSON.stringify(params));
    endpoint = 'query/results';
    format = this.model.get('format');
    if ((_ref = format.needs) != null ? _ref.length : void 0) {
      endpoint += "/" + format.id;
    }
    return CACHE[key] != null ? CACHE[key] : CACHE[key] = this.query.service.post(endpoint, params);
  };

  exports.getEstimatedSize = function() {
    var key, q;
    q = this.getExportQuery();
    key = "count:" + q.service.root + ":" + (q.toXML());
    return CACHE[key] != null ? CACHE[key] : CACHE[key] = q.count();
  };

  exports.getExportQuery = function() {
    var columns, toRun;
    toRun = this.query.clone();
    columns = this.model.get('columns');
    if (columns != null ? columns.length : void 0) {
      toRun.select(columns);
    }
    return toRun;
  };

  exports.getExportURI = function(overrides) {
    return this.getExportQuery().getExportURI(this.model.get('format').id, this.getExportParameters(overrides));
  };

  exports.getFileName = function() {
    return "" + (this.getBaseName()) + "." + (this.getFileExtension());
  };

  exports.getBaseName = function() {
    return this.model.get('filename');
  };

  exports.getFileExtension = function() {
    return this.model.get('format').ext;
  };

  exports.getExportParameters = function(overrides) {
    var data;
    if (overrides == null) {
      overrides = {};
    }
    data = this.model.pick('start', 'size', 'format', 'filename');
    data.format = data.format.id;
    data.query = this.getExportQuery().toXML();
    if (this.model.get('compress')) {
      data.compress = this.model.get('compression');
    }
    if (this.model.get('headers')) {
      data.columnheaders = this.model.get('headerType');
    }
    if ((data.format === 'json') && ('rows' !== this.model.get('jsonFormat'))) {
      data.format += this.model.get('jsonFormat');
    }
    if ((data.format === 'fasta') && (this.model.get('fastaExtension'))) {
      data.extension = this.model.get('fastaExtension');
    }
    return _.extend(data, overrides);
  };

}).call(this);
