(function() {
  var Format, formats, _;

  _ = require('underscore');

  Format = (function() {
    function Format(_arg) {
      var EXT, desc, ext, icon, name, needs;
      this.id = _arg.id, this.group = _arg.group, icon = _arg.icon, needs = _arg.needs, this.maxColumns = _arg.maxColumns;
      ext = this.id === 'tab' ? 'tsv' : this.id;
      EXT = ext.toUpperCase();
      if (icon == null) {
        icon = ext;
      }
      if (needs == null) {
        needs = [];
      }
      desc = "export.format.description." + EXT;
      name = "export.format.name." + EXT;
      _.extend(this, {
        icon: icon,
        desc: desc,
        name: name,
        needs: needs,
        ext: ext,
        EXT: EXT
      });
    }

    Format.prototype.toString = function() {
      return this.ext;
    };

    Format.prototype.toJSON = function() {
      return {
        id: this.id,
        group: this.group,
        icon: this.icon,
        needs: this.needs,
        ext: this.ext
      };
    };

    Format.prototype.isSuitable = function(availableTypes) {
      if (this.needs.length === 0) {
        return true;
      }
      return _.any(this.needs, function(needed) {
        return availableTypes[needed];
      });
    };

    return Format;

  })();

  formats = [
    new Format({
      id: 'tab',
      group: 'flat'
    }), new Format({
      id: 'csv',
      group: 'flat'
    }), new Format({
      id: 'xml',
      group: 'machine'
    }), new Format({
      id: 'json',
      group: 'machine'
    }), new Format({
      id: 'fasta',
      group: 'bio',
      icon: 'dna',
      needs: ['Protein', 'SequenceFeature'],
      maxColumns: 1
    }), new Format({
      id: 'gff3',
      group: 'bio',
      icon: 'dna',
      needs: ['SequenceFeature']
    }), new Format({
      id: 'bed',
      group: 'bio',
      icon: 'dna',
      needs: ['SequenceFeature']
    }), new Format({
      id: 'fake',
      group: 'fake',
      icon: 'fake',
      needs: ['Department']
    }), new Format({
      id: 'fake_2',
      group: 'fake',
      icon: 'fake',
      needs: ['Company']
    })
  ];

  exports.getFormat = function(id) {
    return _.findWhere(formats, {
      id: id
    });
  };

  exports.getFormats = function(availableTypes) {
    var f, _i, _len, _results;
    _results = [];
    for (_i = 0, _len = formats.length; _i < _len; _i++) {
      f = formats[_i];
      if (f.isSuitable(availableTypes)) {
        _results.push(f);
      }
    }
    return _results;
  };

  exports.registerFormat = function(_arg) {
    var icons, id, needs;
    id = _arg.id, icons = _arg.icons, needs = _arg.needs;
    return formats.push(new Format(id, icons, needs));
  };

}).call(this);
