(function() {
  var HeaderModel, Options, PathModel, firstResult, looksLikePath, uc, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  PathModel = require('./path');

  Options = require('../options');

  uc = function(s) {
    return s != null ? s.toUpperCase() : void 0;
  };

  firstResult = _.compose(_.first, _.compact, _.map);

  looksLikePath = function(r) {
    return (r != null) && (r.isReference != null) && (r.isAttribute != null);
  };

  module.exports = HeaderModel = (function(_super) {
    __extends(HeaderModel, _super);

    HeaderModel.prototype.defaults = function() {
      return _.extend(HeaderModel.__super__.defaults.apply(this, arguments), {
        replaces: [],
        isFormatted: false,
        isComposed: false,
        sortable: true,
        sortDirection: null,
        numOfCons: 0,
        minimised: false,
        outerJoined: false,
        expanded: Options.get('Subtables.Initially.Expanded')
      });
    };

    function HeaderModel(opts, query) {
      var _ref;
      this.query = query;
      if (!((_ref = this.query) != null ? _ref.on : void 0)) {
        throw new Error('no query');
      }
      if (opts == null) {
        throw new Error('no options');
      }
      HeaderModel.__super__.constructor.call(this, opts.path);
      this.set(_.omit(opts, 'path'));
      this.set({
        id: "" + (this.query.toXML()) + "-" + (this.get('path'))
      });
      this.listenTo(this.query, 'change:joins', this.setOuterJoined);
      this.listenTo(this.query, 'change:sortorder', this.setSortDirection);
      this.listenTo(this.query, 'change:constraints', this.setConstraintNum);
      this._update_attrs();
    }

    HeaderModel.prototype._update_attrs = function() {
      this.setOuterJoined();
      this.setSortDirection();
      return this.setConstraintNum();
    };

    HeaderModel.prototype.toJSON = function() {
      return _.extend(HeaderModel.__super__.toJSON.apply(this, arguments), {
        replaces: this.get('replaces').map(String)
      });
    };

    HeaderModel.prototype.getView = function() {
      var isFormatted, path, replaces, _ref;
      _ref = this.pick('replaces', 'isFormatted', 'path'), replaces = _ref.replaces, isFormatted = _ref.isFormatted, path = _ref.path;
      return String(isFormatted && replaces.length === 1 ? replaces[0] : path);
    };

    HeaderModel.prototype.validate = function(attrs, opts) {
      var rs;
      if ('replaces' in attrs) {
        rs = attrs.replaces;
        if (!_.isArray(rs)) {
          return new Error('replaces must be an array');
        }
        if (rs.length && (!_.all(rs, looksLikePath))) {
          return new Error('all elements in replaces must be PathInfo objects');
        }
      }
    };

    HeaderModel.prototype.setOuterJoined = function() {
      var isComposed, outerJoined, replaces, sortable, view;
      view = this.getView();
      replaces = this.get('replaces');
      outerJoined = this.query.isOuterJoined(view);
      isComposed = this.get('isFormatted') && (replaces.length > 1);
      sortable = !outerJoined;
      return this.set({
        outerJoined: outerJoined,
        isComposed: isComposed,
        sortable: sortable
      });
    };

    HeaderModel.prototype.setSortDirection = function() {
      var direction, getDirection, replaces, view;
      replaces = this.get('replaces');
      view = this.getView();
      getDirection = (function(_this) {
        return function(p) {
          return _this.query.getSortDirection(p);
        };
      })(this);
      direction = uc(firstResult(replaces.concat([view]), getDirection));
      return this.set({
        sortDirection: direction
      });
    };

    HeaderModel.prototype.setConstraintNum = function() {
      var c, numOfCons, view;
      view = this.getView();
      numOfCons = _.size((function() {
        var _i, _len, _ref, _results;
        _ref = this.query.constraints;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          c = _ref[_i];
          if (c.path.match(view)) {
            _results.push(c);
          }
        }
        return _results;
      }).call(this));
      return this.set({
        numOfCons: numOfCons
      });
    };

    return HeaderModel;

  })(PathModel);

}).call(this);
