(function() {
  var Collection, History, StepModel, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Collection = require('../core/collection');

  StepModel = require('./step');

  module.exports = History = (function(_super) {
    __extends(History, _super);

    function History() {
      return History.__super__.constructor.apply(this, arguments);
    }

    History.prototype.model = StepModel;

    History.prototype.currentQuery = null;

    History.prototype.initialize = function() {
      History.__super__.initialize.apply(this, arguments);
      this.listenTo(this, 'remove', this.unwatch);
      return this.listenTo(this, 'add', this.watch);
    };

    History.prototype.currentStep = 0;

    History.prototype.comparator = 'revision';

    History.prototype.getCurrentQuery = function() {
      return this.currentQuery;
    };

    History.prototype.setCurrentQuery = function(m) {
      return this.currentQuery = m.get('query').clone();
    };

    History.prototype.unwatch = function(model) {
      var q, _ref;
      q = (_ref = model != null ? model.get('query') : void 0) != null ? _ref : this.getCurrentQuery();
      if (q == null) {
        return;
      }
      return this.stopListening(q);
    };

    History.prototype.watch = function(m) {
      var q;
      this.setCurrentQuery(m);
      q = this.getCurrentQuery();
      if (q == null) {
        return;
      }
      this.listenTo(q, "change:constraints", this.onChangeConstraints);
      this.listenTo(q, "change:views", this.onChangeViews);
      this.listenTo(q, "change:joins", this.onChangeJoins);
      this.listenTo(q, "change:sortorder", this.onChangeSortOrder);
      this.listenTo(q, "change:logic", this.onChangeLogic);
      return this.listenTo(q, "undo", this.popState);
    };

    History.prototype.onChangeConstraints = function() {
      return this.onChange('constraints', 'filter', function(cons) {
        return _.map(cons, JSON.stringify);
      });
    };

    History.prototype.onChangeViews = function() {
      return this.onChange('views', 'column');
    };

    History.prototype.onChangeJoins = function() {
      return this.onChange('joins', 'join', function(joins) {
        return _.map(joins, function(style, path) {
          return "" + path + ":" + style;
        });
      });
    };

    History.prototype.onChangeSortOrder = function() {
      return this.onChange('sortOrder', 'sort order element', function(so) {
        return _.map(so, JSON.stringify);
      });
    };

    History.prototype.onChangeLogic = function() {
      return this.onChange('constraintLogic', 'constraint logic element', function(expr) {
        var _ref;
        return (_ref = expr != null ? expr.match(/([A-Z]+)/) : void 0) != null ? _ref : [];
      });
    };

    History.prototype.triggerChangedCurrent = function() {
      return this.trigger('changed:current', this.last());
    };

    History.prototype.onChange = function(prop, label, f) {
      var n, now, prev, query, verb, was, xs, ys;
      if (f == null) {
        f = function(x) {
          return x;
        };
      }
      query = this.currentQuery;
      prev = this.last().get('query');
      xs = f(prev[prop]);
      ys = f(query[prop]);
      was = xs.length;
      now = ys.length;
      n = Math.abs(was - now);
      verb = (function() {
        switch (false) {
          case !(was < now):
            return 'Added';
          case !(was > now):
            return 'Removed';
          case now !== _.union(xs, ys).length:
            return 'Rearranged';
          default:
            return 'Changed';
        }
      })();
      return this.addStep(verb, n, label, query);
    };

    History.prototype.setInitialState = function(q) {
      if (this.size()) {
        this.reset();
      }
      return this.addStep(null, null, 'Initial', q);
    };

    History.prototype.addStep = function(verb, number, label, query) {
      var now, was;
      was = this.currentQuery;
      now = query.clone();
      this.unwatch();
      this.add({
        query: now,
        revision: this.currentStep++,
        title: {
          verb: verb,
          number: number,
          label: label
        }
      });
      if (was != null) {
        was.trigger('replaced:by', now);
      }
      return this.triggerChangedCurrent();
    };

    History.prototype.popState = function() {
      return this.revertToIndex(-2);
    };

    History.prototype.revertToIndex = function(index) {
      var now;
      if (index < 0) {
        index = this.length + index;
      }
      now = this.at(index);
      return this.revertTo(now);
    };

    History.prototype.revertTo = function(now) {
      var revision, was;
      if (!this.contains(now)) {
        throw new Error('State not in history');
      }
      was = this.getCurrentQuery();
      revision = now.get('revision');
      while (this.last().get('revision') > revision) {
        this.pop();
      }
      this.watch(now);
      if (was != null) {
        was.trigger('replaced:by', now.get('query'));
      }
      return this.triggerChangedCurrent();
    };

    return History;

  })(Collection);

}).call(this);
