(function() {
  var Collection, CoreModel, Options, SummaryHistogram, SummaryItemModel, SummaryItems, SummaryModel, getColumnSummary, inty, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreModel = require('../core-model');

  Collection = require('../core/collection');

  Options = require('../options');

  getColumnSummary = require('../services/column-summary').getColumnSummary;

  inty = function(type) {
    return type === 'int' || type === 'Integer' || type === 'long' || type === 'Long' || type === 'short' || type === 'Short';
  };

  module.exports = SummaryModel = (function(_super) {
    __extends(SummaryModel, _super);

    SummaryModel.prototype.defaults = function() {
      return {
        error: null,
        maxCount: null,
        loading: false,
        initialized: false,
        canHaveMultipleValues: false,
        numeric: false,
        filterTerm: null
      };
    };

    function SummaryModel(_arg) {
      var integral, type;
      this.query = _arg.query, this.view = _arg.view;
      SummaryModel.__super__.constructor.call(this);
      if (this.query == null) {
        throw new Error('No query in call to new SummaryModel');
      }
      if (this.view == null) {
        throw new Error('No view in call to new SummaryModel');
      }
      this.fetch = _.partial(getColumnSummary, this.query, this.view);
      type = this.view.getType();
      integral = inty(type);
      this.set({
        limit: Options.get('INITIAL_SUMMARY_ROWS'),
        canHaveMultipleValues: this.query.canHaveMultipleValues(this.view),
        type: type,
        integral: integral
      });
      this.histogram = new SummaryHistogram();
      this.items = new SummaryItems();
      this.listenTo(this, 'change:filterTerm', this.onFilterChange);
      this.listenTo(this, 'change:summaryLimit', this.onLimitChange);
      this.listenTo(this.query, 'change:constraints', this.load);
      this.load();
    }

    SummaryModel.prototype.destroy = function() {
      this.histogram.close();
      this.items.close();
      return SummaryModel.__super__.destroy.apply(this, arguments);
    };

    SummaryModel.prototype.getMaxCount = function() {
      return this.get('maxCount');
    };

    SummaryModel.prototype.hasMore = function() {
      return (!this.get('numeric')) && (this.get('got') < this.get('uniqueValues'));
    };

    SummaryModel.prototype.hasAll = function() {
      return !this.hasMore();
    };

    SummaryModel.prototype.toJSON = function() {
      return _.extend(SummaryModel.__super__.toJSON.apply(this, arguments), {
        items: this.items.toJSON(),
        histogram: this.getHistogram()
      });
    };

    SummaryModel.prototype.onFilterChange = function() {
      if (this.hasAll()) {
        return this.filterLocally();
      } else {
        return this.load();
      }
    };

    SummaryModel.prototype.filterLocally = function() {
      var current, parts, test;
      current = this.get('filterTerm');
      if (current != null) {
        parts = current.toLowerCase().split(/\s+/);
        test = function(str) {
          return _.all(parts, function(part) {
            return !!(str && ~str.toLowerCase().indexOf(part));
          });
        };
        return this.items.each(function(x) {
          return x.show(test(x.get('item')));
        });
      } else {
        return this.items.each(function(x) {
          return x.show();
        });
      }
    };

    SummaryModel.prototype.increaseLimit = function(factor) {
      var limit;
      if (factor == null) {
        factor = 2;
      }
      limit = factor * this.get('limit');
      return this.set({
        limit: limit
      });
    };

    SummaryModel.prototype.onLimitChange = function() {
      return this.load();
    };

    SummaryModel.prototype.setFilterTerm = function(filterTerm) {
      return this.set({
        filterTerm: filterTerm
      });
    };

    SummaryModel.prototype.fetchAll = function() {
      return this.set({
        limit: null
      });
    };

    SummaryModel.prototype.load = function() {
      this.set({
        loading: true
      });
      return this.fetch(this.get('filterTerm'), this.get('limit')).then(this.getSummaryHandler()).then(null, (function(_this) {
        return function(error) {
          return _this.set({
            error: error
          });
        };
      })(this));
    };

    SummaryModel.prototype.getSummaryHandler = function() {
      var created;
      this.lastSummaryHandlerCreatedAt = created = _.now();
      return (function(_this) {
        return function(summary) {
          return _this.handleSummary(created, summary);
        };
      })(this);
    };

    SummaryModel.prototype.getHistogram = function() {
      var i, n, _i, _ref, _ref1, _results;
      n = this.get('buckets');
      if (!n) {
        return [];
      }
      _results = [];
      for (i = _i = 1; 1 <= n ? _i <= n : _i >= n; i = 1 <= n ? ++_i : --_i) {
        _results.push((_ref = (_ref1 = this.histogram.get(i)) != null ? _ref1.get('count') : void 0) != null ? _ref : 0);
      }
      return _results;
    };

    SummaryModel.prototype.handleSummary = function(time, _arg) {
      var average, bucket, buckets, count, id, item, max, min, newStats, numeric, results, stats, stdev, _ref;
      stats = _arg.stats, results = _arg.results;
      if (time !== this.lastSummaryHandlerCreatedAt) {
        return;
      }
      numeric = stats.max != null;
      newStats = {
        filteredCount: stats.filteredCount,
        uniqueValues: stats.uniqueValues,
        available: (_ref = stats.filteredCount) != null ? _ref : stats.uniqueValues,
        got: results.length,
        numeric: numeric,
        initialized: true,
        loading: false
      };
      if (numeric) {
        buckets = stats.buckets, max = stats.max, min = stats.min, stdev = stats.stdev, average = stats.average;
        _.extend(newStats, {
          buckets: buckets,
          max: max,
          min: min,
          stdev: parseFloat(stdev),
          average: parseFloat(average)
        });
        if (this.items.size()) {
          this.items.reset();
        }
        this.histogram.set((function() {
          var _i, _len, _ref1, _results;
          _results = [];
          for (_i = 0, _len = results.length; _i < _len; _i++) {
            _ref1 = results[_i], bucket = _ref1.bucket, count = _ref1.count;
            _results.push({
              item: bucket,
              count: count,
              id: bucket
            });
          }
          return _results;
        })());
        newStats.maxCount = this.histogram.getMaxCount();
      } else {
        if (this.histogram.size()) {
          this.histogram.reset();
        }
        this.items.set((function() {
          var _i, _len, _ref1, _results;
          _results = [];
          for (id = _i = 0, _len = results.length; _i < _len; id = ++_i) {
            _ref1 = results[id], item = _ref1.item, count = _ref1.count;
            _results.push({
              item: item,
              count: count,
              id: id
            });
          }
          return _results;
        })());
        newStats.maxCount = this.items.getMaxCount();
      }
      return this.set(newStats);
    };

    return SummaryModel;

  })(CoreModel);

  SummaryItemModel = (function(_super) {
    __extends(SummaryItemModel, _super);

    function SummaryItemModel() {
      return SummaryItemModel.__super__.constructor.apply(this, arguments);
    }

    SummaryItemModel.prototype.defaults = function() {
      return {
        symbol: null,
        share: null,
        visible: true,
        selected: false,
        hover: false,
        count: 0,
        item: null
      };
    };

    SummaryItemModel.prototype.select = function() {
      return this.set({
        selected: false
      });
    };

    SummaryItemModel.prototype.deselect = function() {
      return this.set({
        selected: false
      });
    };

    SummaryItemModel.prototype.hide = function() {
      return this.set({
        visible: false
      });
    };

    SummaryItemModel.prototype.show = function(ifCondition) {
      if (ifCondition == null) {
        ifCondition = true;
      }
      return this.set({
        visible: ifCondition
      });
    };

    SummaryItemModel.prototype.mousein = function() {
      return this.set({
        hover: true
      });
    };

    SummaryItemModel.prototype.mouseout = function() {
      return this.set({
        hover: false
      });
    };

    return SummaryItemModel;

  })(CoreModel);

  SummaryItems = (function(_super) {
    __extends(SummaryItems, _super);

    function SummaryItems() {
      return SummaryItems.__super__.constructor.apply(this, arguments);
    }

    SummaryItems.prototype.model = SummaryItemModel;

    SummaryItems.prototype.getMaxCount = function() {
      var _ref;
      return (_ref = this.first()) != null ? _ref.get('count') : void 0;
    };

    return SummaryItems;

  })(Collection);

  SummaryHistogram = (function(_super) {
    __extends(SummaryHistogram, _super);

    function SummaryHistogram() {
      return SummaryHistogram.__super__.constructor.apply(this, arguments);
    }

    SummaryHistogram.prototype.model = SummaryItemModel;

    SummaryHistogram.prototype.getMaxCount = function() {
      var b;
      return _.max((function() {
        var _i, _len, _ref, _results;
        _ref = this.models;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          b = _ref[_i];
          _results.push(b.get('count'));
        }
        return _results;
      }).call(this));
    };

    return SummaryHistogram;

  })(Collection);

}).call(this);
