(function() {
  var Options, fs, pluralize, select_html, select_templ, _;

  

  _ = require('underscore');

  Options = require('../options');

  pluralize = require('pluralize');

  select_html = "<select class=\"<%- data.classes %>\">\n  <% _.each(data.options, function (opt) { %>\n    <option value=\"<%- data.key(opt) %>\"\n      <%- data.selectedTest(opt) ? 'selected' : void 0 %> >\n      <% if (data.contentHandler) { %>\n        <%- data.contentHandler(opt) %>\n      <% } else { %>\n        <%- opt.text %>\n      <% } %>\n    </option>\n  <% }); %>\n</select>\n";

  select_templ = _.template(select_html, {
    variable: 'data'
  });

  exports.select = function(options, selectedTest, classes, contentHandler, key) {
    if (selectedTest == null) {
      selectedTest = (function() {
        return false;
      });
    }
    if (classes == null) {
      classes = '';
    }
    if (contentHandler == null) {
      contentHandler = null;
    }
    if (key == null) {
      key = (function(x) {
        return x != null ? x.value : void 0;
      });
    }
    return select_templ({
      options: options,
      selectedTest: selectedTest,
      classes: classes,
      contentHandler: contentHandler,
      key: key
    });
  };

  exports.pluralise = function(str, n) {
    if (str != null) {
      return pluralize(str, n);
    } else {
      return str;
    }
  };

  exports.numToString = function(number) {
    var chars, every, frac, groups, i, len, rets, sep, whole, _ref;
    if (!(number != null ? number.toFixed : void 0)) {
      return number;
    }
    sep = Options.get('NUM_SEPARATOR');
    every = Options.get('NUM_CHUNK_SIZE');
    rets = [];
    i = 0;
    if ((-1 < number && number < 1)) {
      return String(number);
    }
    _ref = number.toFixed(3).split('.'), whole = _ref[0], frac = _ref[1];
    chars = whole.split("");
    len = chars.length;
    groups = _(chars).groupBy(function(c, i) {
      return Math.floor((len - (i + 1)) / every).toFixed();
    });
    while (groups[i]) {
      rets.unshift(groups[i].join(""));
      i++;
    }
    return rets.join(sep) + (frac === '000' ? '' : "." + frac);
  };

  exports.errorAlert = function(_arg) {
    var level;
    level = _arg.level;
    switch (level) {
      case 'Error':
        return 'alert-danger';
      case 'Warning':
        return 'alert-warning';
      case 'Info':
        return 'alert-info';
      case 'OK':
        return 'alert-success';
      default:
        return 'alert-danger';
    }
  };

  exports.errorIcon = function(_arg) {
    var level;
    level = _arg.level;
    return level != null ? level : 'Error';
  };

}).call(this);
