(function() {
  var CellModel, CellModelFactory, FPObject, NestedTableModel, NullObject, ObjectStore, _;

  _ = require('underscore');

  ObjectStore = require('../models/object-store');

  NestedTableModel = require('../models/nested-table');

  CellModel = require('../models/cell');

  NullObject = require('../models/null-object');

  FPObject = require('../models/fast-path-object');

  module.exports = CellModelFactory = (function() {
    function CellModelFactory(service, model) {
      this.itemModels = new ObjectStore(service.root, model);
    }

    CellModelFactory.prototype.getCreator = function(query) {
      var creator;
      if (!query) {
        throw new Error('No query');
      }
      return creator = (function(_this) {
        return function(cell) {
          var path;
          path = query.makePath(cell.column);
          if (_.has(cell, 'rows')) {
            return _this._make_sub_table_model(cell, path, creator);
          } else {
            return _this._make_simple_cell_model(cell, path);
          }
        };
      })(this);
    };

    CellModelFactory.prototype._make_sub_table_model = function(nestedTable, column, createModel) {
      var r;
      return new NestedTableModel(_.extend({}, nestedTable, {
        node: column,
        column: column,
        rows: (function() {
          var _i, _len, _ref, _results;
          _ref = nestedTable.rows;
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            r = _ref[_i];
            _results.push(r.map(function(subcell) {
              return createModel(subcell);
            }));
          }
          return _results;
        })()
      }));
    };

    CellModelFactory.prototype._make_simple_cell_model = function(obj, column) {
      var entity, field, node;
      node = column.getParent();
      field = column.end.name;
      entity = obj.id != null ? this.itemModels.get(obj, field) : obj["class"] == null ? new NullObject(node.getType().name, field) : new FPObject(obj, field);
      return new CellModel({
        entity: entity,
        node: node,
        column: column,
        field: field,
        value: obj.value
      });
    };

    CellModelFactory.prototype.destroy = function() {
      var _ref;
      if ((_ref = this.itemModels) != null) {
        _ref.destroy();
      }
      delete this.itemModels;
      return this;
    };

    return CellModelFactory;

  })();

}).call(this);
