(function() {
  var Messages, Options, Promise, getBranding, getOrganisms, getParameters, getResultClass, openWindowWithPost, parseUrl, send, _;

  _ = require('underscore');

  Promise = require('es6-promise').Promise;

  Options = require('../options');

  Messages = require('../messages');

  getOrganisms = require('./get-organisms');

  getBranding = require('./branding');

  getResultClass = require('./get-result-class');

  openWindowWithPost = require('./open-window-with-post');

  parseUrl = require('./parse-url');

  module.exports = send = function(url, filename, onProgress) {
    var Galaxy, gettingBranding, gettingOrganisms, gettingResultClass, uri, _ref;
    Galaxy = Options.get('Destination.Galaxy');
    uri = (_ref = Galaxy.Current) != null ? _ref : Galaxy.Main;
    if (!/tool_runner$/.test(uri)) {
      uri += '/tool_runner';
    }
    gettingBranding = getBranding(this.query.service);
    gettingResultClass = getResultClass(this.query);
    gettingOrganisms = getOrganisms(this.query);
    return Promise.all([gettingResultClass, gettingOrganisms, gettingBranding]).then(getParameters(url, this.query, this.model.format.ext)).then(_.partial(openWindowWithPost, uri, 'Upload'));
  };

  getParameters = function(url, query, ext) {
    return function(_arg) {
      var URL, branding, c, cls, currentLocation, data_type, info, lists, name, organism, orgs, params, tool_id, _ref;
      cls = _arg[0], orgs = _arg[1], branding = _arg[2];
      _ref = parseUrl(url), URL = _ref.URL, params = _ref.params;
      lists = (function() {
        var _i, _len, _ref1, _results;
        _ref1 = query.constraints;
        _results = [];
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          c = _ref1[_i];
          if (c.op === 'IN') {
            _results.push(c.value);
          }
        }
        return _results;
      })();
      data_type = ext === 'tsv' ? 'tabular' : ext;
      currentLocation = window.location.toString().replace(/\?.*$/, '');
      tool_id = Options.get('Destination.Galaxy.Tool');
      name = Messages.getText('export.galaxy.name', {
        cls: cls,
        orgs: orgs,
        branding: branding
      });
      info = Messages.getText('export.galaxy.info', {
        query: query,
        lists: lists,
        orgs: orgs,
        currentLocation: currentLocation
      });
      if (orgs != null) {
        organism = orgs.join(', ');
      }
      return _.extend(params, {
        tool_id: tool_id,
        URL: URL,
        name: name,
        info: info,
        data_type: data_type,
        organism: organism,
        URL_method: 'post'
      });
    };
  };

  send.after = function() {
    var Galaxy;
    Galaxy = Options.get('Destination.Galaxy');
    if (Galaxy.Current && Galaxy.Save) {
      return this.query.service.whoami().then(function(user) {
        if (user.preferences['galaxy-url'] !== Galaxy.Current) {
          return user.setPreference('galaxy-url', Galaxy.Current);
        }
      });
    } else {
      return Promise.resolve(null);
    }
  };

}).call(this);
