(function() {
  var $, BOUNDARY, CLOSE_DELIM, DELIMITER, DRIVE_METHOD, DRIVE_PATH, ERR, FILE_CT, GoogleExporter, LIB, METADATA_CT, MetaData, Options, Promise, REQ_CT, REQ_PARAMS, SCOPE, VERSION, loadResource, sendToGoogleDrive, withExporter, _, __GOOGLE;

  $ = require('jquery');

  _ = require('underscore');

  Promise = require('es6-promise').Promise;

  Options = require('../options');

  loadResource = require('./load-resource');

  __GOOGLE = null;

  LIB = 'Destination.Drive.Library';

  BOUNDARY = "-------314159265358979323846";

  DELIMITER = "\r\n--" + BOUNDARY + "\r\n";

  CLOSE_DELIM = "\r\n--" + BOUNDARY + "--";

  VERSION = 'v2';

  SCOPE = "https://www.googleapis.com/auth/drive.file";

  ERR = 'No configuration available for Google Drive';

  METADATA_CT = "Content-Type: application/json\r\n\r\n";

  FILE_CT = "Content-Type: text/tab-separated-values\r\n\r\n";

  REQ_CT = "multipart/mixed; boundary=\"" + BOUNDARY + "\"";

  DRIVE_PATH = "/upload/drive/v2/files";

  DRIVE_METHOD = 'POST';

  REQ_PARAMS = {
    path: DRIVE_PATH,
    method: DRIVE_METHOD,
    params: {
      uploadType: 'multipart'
    },
    headers: {
      'Content-Type': REQ_CT
    }
  };

  withExporter = function() {
    return __GOOGLE != null ? __GOOGLE : __GOOGLE = loadResource(LIB, 'gapi').then(function(api) {
      return new GoogleExporter(api);
    });
  };

  module.exports = sendToGoogleDrive = function(uri, filename, onProgress) {
    return withExporter().then(function(e) {
      onProgress(1);
      return e.upload(uri, filename);
    });
  };

  MetaData = (function() {
    MetaData.prototype.mimetype = 'text/plain';

    function MetaData(title) {
      this.title = title;
    }

    MetaData.prototype.toString = function() {
      return JSON.stringify({
        title: this.title,
        mimetype: this.mimetype
      });
    };

    return MetaData;

  })();

  GoogleExporter = (function() {
    function GoogleExporter(gapi) {
      this.gapi = gapi;
      if (!this.gapi) {
        throw new Error('No api');
      }
      if (!Options.get('auth.drive')) {
        throw new Error(ERR);
      }
      console.log(this.gapi);
    }

    GoogleExporter.prototype.authorize = function() {
      return new Promise((function(_this) {
        return function(resolve, reject) {
          var abort, checkOrWait, gapi, nextStep, opts, timeout;
          gapi = _this.gapi;
          timeout = null;
          opts = {
            client_id: Options.get('auth.drive'),
            scope: SCOPE,
            immediate: false
          };
          nextStep = function() {
            return gapi.auth.authorize(opts, function(auth) {
              if (auth == null) {
                return reject(new Error('Not authorized'));
              }
              if (auth.error) {
                return reject(new Error(auth.error));
              }
              return resolve();
            });
          };
          abort = function() {
            clearTimeout(timeout);
            return reject(new Error('timed out'));
          };
          checkOrWait = function() {
            var _ref;
            clearTimeout(timeout);
            if ((_ref = gapi.auth) != null ? _ref.authorize : void 0) {
              return nextStep();
            } else {
              return timeout = setTimeout(checkOrWait, 50);
            }
          };
          checkOrWait();
          setTimeout(abort, 5000);
        };
      })(this));
    };

    GoogleExporter.prototype.loadClient = function() {
      return new Promise((function(_this) {
        return function(resolve, reject) {
          return _this.gapi.client.load("drive", VERSION, resolve);
        };
      })(this));
    };

    GoogleExporter.prototype.makeRequestBody = function(metadata, data) {
      return DELIMITER + METADATA_CT + String(metadata) + DELIMITER + FILE_CT + data + CLOSE_DELIM;
    };

    GoogleExporter.prototype.makeRequest = function(body) {
      return this.gapi.client.request(_.extend({
        body: body
      }, REQ_PARAMS));
    };

    GoogleExporter.prototype.upload = function(uri, filename) {
      return this.authorize().then((function(_this) {
        return function() {
          return _this.loadClient();
        };
      })(this)).then(function() {
        return Promise.resolve($.get(uri));
      }).then((function(_this) {
        return function(data) {
          return _this.makeRequestBody(new MetaData(filename), data);
        };
      })(this)).then((function(_this) {
        return function(body) {
          return _this.makeRequest(body);
        };
      })(this)).then(function(resp) {
        return resp.result.alternateLink;
      });
    };

    return GoogleExporter;

  })();

}).call(this);
