(function() {
  var ALREADY_DONE, CACHES, Options, Page, Promise, ResultCache,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  Promise = require('es6-promise').Promise;

  Options = require('../options');

  Page = require('../models/page');

  CACHES = {};

  ALREADY_DONE = Promise.resolve(true);

  exports.getCache = function(query) {
    var root, xml, _name;
    xml = query.toXML();
    root = query.service.root;
    return CACHES[_name = root + ':' + xml] != null ? CACHES[_name] : CACHES[_name] = new ResultCache(query);
  };

  exports.onListChange = function(listName) {
    var cache, name;
    for (name in CACHES) {
      cache = CACHES[name];
      if (__indexOf.call(listNamesFor(cache.query), listName) >= 0) {
        cache.dropRows();
      }
    }
    return false;
  };

  ResultCache = (function() {
    ResultCache.prototype.offset = 0;

    ResultCache.prototype.rows = null;

    function ResultCache(query) {
      this.query = query.clone();
    }

    ResultCache.prototype.toString = function() {
      var _ref;
      if ((_ref = this.rows) != null ? _ref.length : void 0) {
        return "ResultCache(" + this.offset + " .. " + (this.offset + this.rows.length) + ")";
      } else {
        return "ResultCache(EMPTY)";
      }
    };

    ResultCache.prototype.fetchRows = function(start, size) {
      var page, updating;
      if (start == null) {
        start = 0;
      }
      if (size == null) {
        size = null;
      }
      page = new Page(start, size);
      updating = this.updateCache(page);
      return updating.then((function(_this) {
        return function() {
          return _this.getRows(page);
        };
      })(this));
    };

    ResultCache.prototype.updateCache = function(page) {
      var p;
      if (this.contains(page)) {
        return ALREADY_DONE;
      }
      p = this.getRequestPage(page);
      return this.query.tableRows({
        start: p.start,
        size: p.size
      }).then(this.addRowsToCache(p));
    };

    ResultCache.prototype.contains = function(page) {
      var cache, end;
      cache = this.rows;
      if (!(cache != null ? cache.length : void 0)) {
        return false;
      }
      end = this.offset + cache.length;
      return (!page.all()) && (page.start >= this.offset) && (page.end() <= end);
    };

    ResultCache.prototype.dropRows = function() {
      this.rows = null;
      return this.offset = 0;
    };

    ResultCache.prototype.getRequestPage = function(page) {
      var cache, factor, gap, requestLimit, requestPage, size, start, upperBound;
      cache = this.rows;
      factor = Options.get('TableResults.CacheFactor');
      requestLimit = Options.get('TableResults.RequestLimit');
      size = page.all() ? page.size : page.size * factor;
      if (cache == null) {
        return new Page(page.start, size);
      }
      upperBound = this.offset + cache.length;
      start = page.all() || (page.start >= this.offset) ? page.start : Math.max(0, page.start - size);
      requestPage = new Page(start, size);
      if (requestPage.isBefore(this.offset)) {
        gap = requestPage.rightGap(this.offset);
        if ((gap + requestPage.size) > requestLimit) {
          this.dropRows();
        } else {
          requestPage.size += gap;
        }
      } else if (requestPage.isAfter(upperBound)) {
        gap = requestPage.leftGap(upperBound);
        if ((gap + requestPage.size) > requestLimit) {
          this.dropRows();
        } else {
          requestPage.size += gap;
          requestPage.start = upperBound;
        }
      }
      return requestPage;
    };

    ResultCache.prototype.addRowsToCache = function(page) {
      return (function(_this) {
        return function(rows) {
          var cache, offset, upperBound;
          cache = _this.rows;
          offset = _this.offset;
          if (cache != null) {
            upperBound = _this.offset + cache.length;
            if (page.start < offset) {
              if (page.end() < offset) {
                throw new Error("Cannot add " + page + " to " + _this + " - non contiguous");
              }
              if (page.all() || page.end() > upperBound) {
                cache = rows.slice();
              } else {
                cache = rows.concat(cache.slice(page.end() - offset));
              }
            } else if (page.start > upperBound) {
              throw new Error("Cannot add " + page + " to " + _this + " - non contiguous");
            } else if (page.all() || upperBound < page.end()) {
              cache = cache.slice(0, page.start - offset).concat(rows);
            } else {
              console.error("Useless cache add - we already had these rows: " + page);
            }
            offset = Math.min(offset, page.start);
          } else {
            cache = rows.slice();
            offset = page.start;
          }
          _this.offset = offset;
          _this.rows = cache;
          return true;
        };
      })(this);
    };

    ResultCache.prototype.getRows = function(_arg) {
      var rows, size, start;
      start = _arg.start, size = _arg.size;
      if (this.rows == null) {
        throw new Error('Cache has not been updated');
      }
      rows = this.rows.slice();
      rows.splice(0, start - this.offset);
      if ((size != null) && size > 0) {
        rows.splice(size, rows.length);
      }
      return rows;
    };

    return ResultCache;

  })();

  exports.ResultCache = ResultCache;

}).call(this);
