(function() {
  var $, ActiveConstraint, ConstraintEditor, ConstraintModel, ConstraintSummary, CoreModel, Icons, Messages, Model, Options, Promise, Query, Templates, View, aeql, basicEql, _, _ref,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  $ = require('jquery');

  Promise = require('es6-promise').Promise;

  _ref = require('imjs'), Query = _ref.Query, Model = _ref.Model;

  CoreModel = require('../core-model');

  Messages = require('../messages');

  Templates = require('../templates');

  Icons = require('../icons');

  Options = require('../options');

  View = require('../core-view');

  ConstraintSummary = require('./constraint-summary');

  ConstraintEditor = require('./constraint-editor');

  Messages.setWithPrefix('conbuilder', {
    Add: 'Add constraint',
    Update: 'Update',
    Cancel: 'Cancel',
    Remove: 'Remove',
    EditCon: 'Edit filter',
    NotEditable: 'This constraint is not editable',
    ValuePlaceholder: 'David*',
    ExtraPlaceholder: 'Wernham-Hogg',
    ExtraLabel: 'in',
    IsA: 'is a',
    NoValue: 'No value selected. Please enter a value.',
    NoOperator: 'No operator selected. Please choose an operator.',
    BadLoop: 'The selected path is not in the query.',
    NaN: 'The value provided is not a number.',
    Duplicate: 'This constraint is already on the query',
    TooManySuggestions: 'We cannot show you all the possible values',
    NoSuitableLists: 'No lists of this type are available',
    NoSuitableLoops: 'No suitable loop paths were found'
  });

  aeql = function(xs, ys) {
    var longer, shorter, _ref1;
    if (!xs && !ys) {
      return true;
    }
    if (!xs || !ys) {
      return false;
    }
    _ref1 = _.sortBy([xs, ys], function(a) {
      return a.length;
    }), shorter = _ref1[0], longer = _ref1[1];
    return _.all(longer, function(x) {
      return __indexOf.call(shorter, x) >= 0;
    });
  };

  basicEql = function(a, b) {
    var k, keys, same, va, vb, x, _i, _len, _ref1;
    if (!(a && b)) {
      return a === b;
    }
    keys = _.union.apply(_, [a, b].map(_.keys));
    same = true;
    for (_i = 0, _len = keys.length; _i < _len; _i++) {
      k = keys[_i];
      _ref1 = (function() {
        var _j, _len1, _ref1, _results;
        _ref1 = [a, b];
        _results = [];
        for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
          x = _ref1[_j];
          _results.push(x[k]);
        }
        return _results;
      })(), va = _ref1[0], vb = _ref1[1];
      same && (same = (_.isArray(va) ? aeql(va, vb) : va === vb));
    }
    return same;
  };

  ConstraintModel = (function(_super) {
    __extends(ConstraintModel, _super);

    function ConstraintModel() {
      return ConstraintModel.__super__.constructor.apply(this, arguments);
    }

    ConstraintModel.prototype.defaults = function() {
      return {
        code: null
      };
    };

    ConstraintModel.prototype.isTypeConstraint = function() {
      return this.get('op') == null;
    };

    return ConstraintModel;

  })(CoreModel);

  module.exports = ActiveConstraint = (function(_super) {
    var IS_BLANK;

    __extends(ActiveConstraint, _super);

    function ActiveConstraint() {
      return ActiveConstraint.__super__.constructor.apply(this, arguments);
    }

    ActiveConstraint.prototype.Model = ConstraintModel;

    ActiveConstraint.prototype.tagName = "div";

    ActiveConstraint.prototype.className = "im-constraint row-fluid";

    ActiveConstraint.prototype.parameters = ['query', 'constraint'];

    ActiveConstraint.prototype.optionalParameters = ['buttonDelegate'];

    ActiveConstraint.prototype.initialize = function() {
      var e;
      ActiveConstraint.__super__.initialize.apply(this, arguments);
      this.model.set(this.constraint);
      this.state.set({
        editing: false
      });
      this.listenTo(this.state, 'change:editing', this.toggleEditor);
      this.listenTo(this.model, 'change:type', this.setTypeName);
      this.listenTo(this.model, 'change:path', this.setDisplayName);
      this.listenTo(Messages, 'change', this.reRender);
      this.listenTo(this.model, 'cancel', this.cancelEditing);
      this.listenTo(this.model, 'apply', this.applyChanges);
      try {
        this.model.set({
          path: this.query.makePath(this.constraint.path)
        });
      } catch (_error) {
        e = _error;
        this.model.set({
          error: e
        });
        this.state.set({
          editing: true
        });
      }
      return this.setTypeName();
    };

    ActiveConstraint.prototype.setDisplayName = function() {
      return this.model.get('path').getDisplayName((function(_this) {
        return function(error, displayName) {
          _this.model.set({
            error: error,
            displayName: displayName
          });
          if (error != null) {
            return _this.listenToOnce(_this.query, 'change:constraints', function() {
              _this.model.set({
                path: _this.query.getPathInfo(_this.constraint.path)
              });
              return _this.setDisplayName();
            });
          }
        };
      })(this));
    };

    ActiveConstraint.prototype.cancelEditing = function() {
      this.state.set({
        editing: false
      });
      this.model.set(_.omit(this.constraint, 'path'));
      return this.model.set({
        error: null
      });
    };

    ActiveConstraint.prototype.toggleEditing = function() {
      if (this.state.get('editing')) {
        return this.cancelEditing();
      } else {
        return this.state.set({
          editing: true
        });
      }
    };

    ActiveConstraint.prototype.setTypeName = function() {
      var e, type;
      type = this.model.get('type');
      if (type == null) {
        return this.model.unset('typeName');
      } else {
        try {
          return this.query.model.makePath(type).getDisplayName((function(_this) {
            return function(error, typeName) {
              return _this.model.set({
                error: error,
                typeName: typeName
              });
            };
          })(this));
        } catch (_error) {
          e = _error;
          return this.model.set({
            error: e,
            typeName: type
          });
        }
      }
    };

    ActiveConstraint.prototype.events = function() {
      return {
        'click .im-edit': 'toggleEditing',
        'click .im-remove-constraint': 'removeConstraint'
      };
    };

    IS_BLANK = /^\s*$/;

    ActiveConstraint.prototype.getLoopProblem = function(con) {
      var e, problem;
      problem = (function() {
        try {
          this.query.getPathInfo(con.value);
          return null;
        } catch (_error) {
          e = _error;
          return 'BadLoop';
        }
      }).call(this);
      return problem;
    };

    ActiveConstraint.prototype.getValueProblem = function(con) {
      var op, path, value, _ref1;
      path = con.path, op = con.op, value = con.value;
      console.debug(con);
      if ((value == null) || (IS_BLANK.test(value))) {
        return 'NoValue';
      }
      if ((_ref1 = path.getType(), __indexOf.call(Model.NUMERIC_TYPES, _ref1) >= 0) && (_.isNaN(1 * value))) {
        return 'NaN';
      }
      return null;
    };

    ActiveConstraint.prototype.getProblem = function(con) {
      var _ref1, _ref2;
      if (con.type != null) {
        return null;
      }
      if (!con.op || IS_BLANK.test(con.op)) {
        return 'NoOperator';
      }
      if (con.path.isReference() && ((_ref1 = con.op) === '=' || _ref1 === '!=')) {
        return this.getLoopProblem(con);
      }
      if (_ref2 = con.op, __indexOf.call(Query.ATTRIBUTE_VALUE_OPS.concat(Query.REFERENCE_OPS), _ref2) >= 0) {
        return this.getValueProblem(con);
      }
      if (this.isDuplicate(con)) {
        return 'Duplicate';
      }
      return null;
    };

    ActiveConstraint.prototype.isDuplicate = function(con) {
      return _.any(this.query.constraints, _.partial(basicEql, con));
    };

    ActiveConstraint.prototype.setError = function(key) {
      var msg;
      msg = Messages.get("conbuilder." + key);
      return this.model.set({
        error: msg
      });
    };

    ActiveConstraint.prototype.applyChanges = function(con) {
      var problem, silently;
      problem = this.getProblem(con);
      if (problem != null) {
        return this.setError(problem);
      }
      this.state.set({
        editing: false
      });
      this.removeConstraint(null, silently = true);
      if ((con.values != null) && !con.values.length) {
        return this.query.trigger("change:constraints");
      } else {
        console.debug('Adding constraint');
        con.path = con.path.toString();
        this.query.addConstraint(con);
        this.constraint = con;
        return this.model.unset('new');
      }
    };

    ActiveConstraint.prototype.removeConstraint = function(e, silently) {
      if (silently == null) {
        silently = false;
      }
      if (e != null) {
        e.preventDefault();
      }
      if (e != null) {
        e.stopPropagation();
      }
      if (!this.model.get('new')) {
        this.query.removeConstraint(this.constraint, silently);
      }
      if (e != null) {
        return this.remove();
      }
    };

    ActiveConstraint.prototype.getData = function() {
      var con, icons, messages;
      messages = Messages;
      icons = Icons;
      con = this.model.toJSON();
      return {
        icons: icons,
        messages: messages,
        con: con
      };
    };

    ActiveConstraint.prototype.template = Templates.template('active-constraint', {
      variable: 'data'
    });

    ActiveConstraint.prototype.toggleEditor = function() {
      var opts;
      if (this.state.get('editing') && this.rendered) {
        opts = {
          model: this.model,
          query: this.query,
          buttonDelegate: this.buttonDelegate
        };
        return this.renderChild('editor', new ConstraintEditor(opts), this.$('.im-constraint-editor'));
      } else {
        return this.removeChild('editor');
      }
    };

    ActiveConstraint.prototype.renderSummary = function() {
      var opts;
      opts = {
        model: this.model
      };
      return this.renderChild('summary', new ConstraintSummary(opts), this.$('.im-con-overview'));
    };

    ActiveConstraint.prototype.postRender = function() {
      this.renderSummary();
      this.toggleEditor();
      return this.$('[title]').tooltip();
    };

    return ActiveConstraint;

  })(View);

}).call(this);
