(function() {
  var $, AttributeValueControls, CoreView, HasTypeaheads, INTEGRAL_TYPES, IS_BLANK, Messages, NUMERIC_TYPES, NestedModel, Options, Promise, Query, SuggestionSource, Templates, getBranding, numToString, numify, selectTemplate, trim, _, _ref, _ref1,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  $ = require('jquery');

  _ = require('underscore');

  Promise = require('es6-promise').Promise;

  Messages = require('../messages');

  Templates = require('../templates');

  CoreView = require('../core-view');

  Options = require('../options');

  NestedModel = require('../core/nested-model');

  getBranding = require('../utils/branding');

  IS_BLANK = require('../patterns').IS_BLANK;

  HasTypeaheads = require('../mixins/has-typeaheads');

  SuggestionSource = require('../utils/suggestion-source');

  _ref = require('imjs'), (_ref1 = _ref.Model, INTEGRAL_TYPES = _ref1.INTEGRAL_TYPES, NUMERIC_TYPES = _ref1.NUMERIC_TYPES), Query = _ref.Query;

  selectTemplate = Templates.template('attribute_value_select');

  trim = function(s) {
    return String(s).replace(/^\s+/, '').replace(/\s+$/, '');
  };

  numify = function(x) {
    return 1 * trim(x);
  };

  numToString = require('../templates/helpers').numToString;

  Messages.set({
    'constraintvalue.NoValues': 'There are no possible values. This query returns no results',
    'constraintvalue.OneValue': "There is only one possible value: <%- value %>. You might want to remove this constraint"
  });

  module.exports = AttributeValueControls = (function(_super) {
    __extends(AttributeValueControls, _super);

    function AttributeValueControls() {
      return AttributeValueControls.__super__.constructor.apply(this, arguments);
    }

    AttributeValueControls.include(HasTypeaheads);

    AttributeValueControls.prototype.className = 'im-attribute-value-controls';

    AttributeValueControls.prototype.template = Templates.template('attribute_value_controls');

    AttributeValueControls.prototype.getData = function() {
      return _.extend(this.getBaseData(), {
        messages: Messages,
        con: this.model.toJSON()
      });
    };

    AttributeValueControls.prototype.initialize = function(_arg) {
      var _ref2;
      this.query = _arg.query;
      AttributeValueControls.__super__.initialize.apply(this, arguments);
      this.sliders = [];
      this.branding = new NestedModel;
      this.cast = (_ref2 = this.model.get('path').getType(), __indexOf.call(NUMERIC_TYPES, _ref2) >= 0) ? numify : trim;
      this.listenTo(Messages, 'change', this.reRender);
      this.state.set({
        valuePlaceholder: Messages.getText('conbuilder.ValuePlaceholder')
      });
      this.listenTo(this.branding, 'change:defaults.value', function() {
        return this.state.set({
          valuePlaceholder: this.branding.get('defaults.value')
        });
      });
      if (this.query != null) {
        this.listenTo(this.query, 'change:constraints', this.clearCachedData);
        this.listenTo(this.query, 'change:constraints', this.reRender);
        return getBranding(this.query.service).then((function(_this) {
          return function(branding) {
            return _this.branding.set(branding);
          };
        })(this));
      }
    };

    AttributeValueControls.prototype.modelEvents = function() {
      return {
        destroy: function() {
          return this.stopListening();
        },
        'change:value': this.onChangeValue,
        'change:op': this.onChangeOp
      };
    };

    AttributeValueControls.prototype.stateEvents = function() {
      return {
        'change:valuePlaceholder': this.reRender
      };
    };

    AttributeValueControls.prototype.onChangeOp = function() {
      var newOp;
      newOp = this.model.get('op');
      if ((this.model.get('value') != null) && __indexOf.call(Query.MULTIVALUE_OPS, newOp) >= 0) {
        this.model.set({
          value: null,
          values: [this.model.get('value')]
        });
      }
      return this.reRender();
    };

    AttributeValueControls.prototype.onChangeValue = function() {
      return this.reRender();
    };

    AttributeValueControls.prototype.removeAllChildren = function() {
      this.removeTypeAheads();
      this.removeSliders();
      return AttributeValueControls.__super__.removeAllChildren.apply(this, arguments);
    };

    AttributeValueControls.prototype.removeSliders = function() {
      var sl, _results;
      _results = [];
      while (sl = this.sliders.pop()) {
        try {
          sl.slider('destroy');
          _results.push(sl.remove());
        } catch (_error) {}
      }
      return _results;
    };

    AttributeValueControls.prototype.events = function() {
      return {
        'change .im-con-value-attr': 'setAttributeValue'
      };
    };

    AttributeValueControls.prototype.updateInput = function() {
      var input, _ref2;
      input = (_ref2 = this.lastTypeahead()) != null ? _ref2 : this.$('.im-con-value-attr');
      return input.val(this.model.get('value'));
    };

    AttributeValueControls.prototype.readAttrValue = function() {
      var e, raw, _ref2;
      raw = ((_ref2 = this.lastTypeahead()) != null ? _ref2 : this.$('.im-con-value-attr')).val();
      try {
        if ((raw != null) && !IS_BLANK.test(raw)) {
          return this.cast(raw);
        } else {
          return null;
        }
      } catch (_error) {
        e = _error;
        this.model.set({
          error: new Error("" + raw + " might not be a legal value for " + this.path)
        });
        return raw;
      }
    };

    AttributeValueControls.prototype.setAttributeValue = function() {
      return this.model.set({
        value: this.readAttrValue()
      });
    };

    AttributeValueControls.prototype.postRender = function() {
      this.provideSuggestions().then(null, (function(_this) {
        return function(error) {
          return _this.model.set({
            error: error
          });
        };
      })(this));
      return this.$('.im-con-value-attr').focus();
    };

    AttributeValueControls.prototype.provideSuggestions = function() {
      return this.getSuggestions().then((function(_this) {
        return function(_arg) {
          var msg, results, stats;
          stats = _arg.stats, results = _arg.results;
          if (stats.uniqueValues === 0) {
            msg = Messages.getText('constraintvalue.NoValues');
            return _this.model.set({
              error: {
                message: msg,
                level: 'warning'
              }
            });
          } else if (stats.uniqueValues === 1) {
            msg = Messages.getText('constraintvalue.OneValue', {
              value: results[0].item
            });
            return _this.model.set({
              error: {
                message: msg,
                level: 'warning'
              }
            });
          } else if (stats.max != null) {
            return _this.handleNumericSummary(stats);
          } else if (results[0].item != null) {
            return _this.handleSummary(results, stats.uniqueValues);
          }
        };
      })(this));
    };

    AttributeValueControls.prototype.clearCachedData = function() {
      delete this.__suggestions;
      return this.model.unset('error');
    };

    AttributeValueControls.prototype.getSuggestions = function() {
      return this.__suggestions != null ? this.__suggestions : this.__suggestions = (function(_this) {
        return function() {
          var c, clone, maxSuggestions, pstr, value;
          if (!_this.model.get('path')) {
            return Promise.reject(new Error('no path'));
          }
          clone = _this.query.clone();
          value = _this.model.get('value');
          pstr = String(_this.model.get('path'));
          maxSuggestions = Options.get('MaxSuggestions');
          clone.constraints = (function() {
            var _i, _len, _ref2, _results;
            _ref2 = clone.constraints;
            _results = [];
            for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
              c = _ref2[_i];
              if (!(c.path === pstr && c.value === value)) {
                _results.push(c);
              }
            }
            return _results;
          })();
          return clone.summarise(pstr, maxSuggestions);
        };
      })(this)();
    };

    AttributeValueControls.prototype.replaceInputWithSelect = function(items) {
      var value;
      console.log("Select of " + items.length + " items");
      if (this.model.has('value')) {
        value = this.model.get('value');
        if ((value != null) && !(_.any(items, function(_arg) {
          var item;
          item = _arg.item;
          return item === value;
        }))) {
          items.push({
            item: value
          });
        }
      } else {
        this.model.set({
          value: items[0].item
        });
      }
      return this.$el.html(selectTemplate({
        Messages: Messages,
        items: items,
        value: value
      }));
    };

    AttributeValueControls.prototype.handleSummary = function(items, total) {
      var dataset, handleSuggestion, input, opts, source, _ref2;
      if (((_ref2 = this.model.get('op')) === '=' || _ref2 === '!=') && items.length < Options.get('DropdownMax')) {
        return this.replaceInputWithSelect(items);
      }
      input = this.$('.im-con-value-attr');
      source = new SuggestionSource(items, total);
      opts = {
        minLength: 0,
        highlight: true
      };
      dataset = {
        name: 'summary_suggestions',
        source: source.suggest,
        displayKey: 'item',
        templates: {
          footer: source.tooMany
        }
      };
      this.removeTypeAheads();
      handleSuggestion = (function(_this) {
        return function(evt, suggestion) {
          return _this.model.set({
            value: suggestion.item
          });
        };
      })(this);
      return this.activateTypeahead(input, opts, dataset, items[0].item, handleSuggestion, (function(_this) {
        return function() {
          return _this.setAttributeValue();
        };
      })(this));
    };

    AttributeValueControls.prototype.clearer = '<div class="" style="clear:both;">';

    AttributeValueControls.prototype.getMarkers = function(min, max, isInt) {
      var f, getPercent, getValue, span, _i, _len, _ref2, _results;
      span = max - min;
      getValue = function(frac) {
        var val;
        val = frac * span + min;
        if (isInt) {
          return Math.round(val);
        } else {
          return val;
        }
      };
      getPercent = function(frac) {
        return Math.round(100 * frac);
      };
      _ref2 = [0, 0.5, 1];
      _results = [];
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        f = _ref2[_i];
        _results.push({
          percent: getPercent(f),
          value: numToString(getValue(f))
        });
      }
      return _results;
    };

    AttributeValueControls.prototype.makeSlider = Templates.template('slider', {
      variable: 'markers'
    });

    AttributeValueControls.prototype.handleNumericSummary = function(_arg) {
      var $slider, average, caster, container, input, isInt, markers, max, min, path, step, _ref2;
      min = _arg.min, max = _arg.max, average = _arg.average;
      path = this.model.get('path');
      isInt = (_ref2 = path.getType(), __indexOf.call(INTEGRAL_TYPES, _ref2) >= 0);
      step = isInt ? 1 : max - min / 100;
      caster = isInt ? (function(x) {
        return parseInt(x, 10);
      }) : parseFloat;
      container = this.$el;
      input = this.$('input');
      container.append(this.clearer);
      markers = this.getMarkers(min, max, isInt);
      this.removeSliders();
      input.off('change.slider');
      $slider = $(this.makeSlider(markers));
      $slider.appendTo(container).slider({
        min: min,
        max: max,
        value: (this.model.has('value') ? this.model.get('value') : caster(average)),
        step: step,
        slide: function(e, ui) {
          return input.val(ui.value).change();
        }
      });
      input.attr({
        placeholder: caster(average)
      });
      container.append(this.clearer);
      input.on('change.slider', function(e) {
        return $slider.slider('value', caster(input.val()));
      });
      return this.sliders.push($slider);
    };

    AttributeValueControls.prototype.remove = function() {
      AttributeValueControls.__super__.remove.apply(this, arguments);
      return this.removeTypeAheads();
    };

    return AttributeValueControls;

  })(CoreView);

}).call(this);
