(function() {
  var CANNOT_SAVE, C_STYLE_COMMENTS, Checkbox, CodeGenDialogue, CodeGenModel, HTML_MIMETYPE, JS_MIMETYPE, MIMETYPES, Messages, Modal, OCTOTHORPE_COMMENTS, Options, Promise, Templates, XML_MIMETYPE, alreadyRejected, canSaveFromMemory, indentXml, stripEmptyValues, stripExtraneousWhiteSpace, withFileSaver, withPrettyPrintOne, withResource, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Promise = require('es6-promise').Promise;

  Modal = require('./modal');

  Messages = require('../messages');

  Options = require('../options');

  Templates = require('../templates');

  CodeGenModel = require('../models/code-gen');

  Checkbox = require('../core/checkbox');

  require('../messages/code-gen');

  indentXml = require('../utils/indent-xml');

  stripExtraneousWhiteSpace = require('../utils/strip-extra-whitespace');

  withResource = require('../utils/with-cdn-resource');

  OCTOTHORPE_COMMENTS = /\s*#.*$/gm;

  C_STYLE_COMMENTS = /\/\*(\*(?!\/)|[^*])*\*\//gm;

  XML_MIMETYPE = 'application/xml;charset=utf8';

  JS_MIMETYPE = 'text/javascript;charset=utf8';

  HTML_MIMETYPE = 'text/html;charset=utf8';

  CANNOT_SAVE = {
    level: 'Info',
    key: 'codegen.CannotExportXML'
  };

  MIMETYPES = {
    js: JS_MIMETYPE,
    xml: XML_MIMETYPE,
    html: HTML_MIMETYPE
  };

  withPrettyPrintOne = _.partial(withResource, 'prettify', 'prettyPrintOne');

  withFileSaver = _.partial(withResource, 'filesaver', 'saveAs');

  alreadyRejected = Promise.reject('Requirements not met');

  stripEmptyValues = function(q) {
    var k, v;
    return _.object((function() {
      var _results;
      _results = [];
      for (k in q) {
        v = q[k];
        if (v && v.length !== 0) {
          _results.push([k, v]);
        }
      }
      return _results;
    })());
  };

  canSaveFromMemory = function() {
    var _ref;
    if (_ref = !'Blob', __indexOf.call(global, _ref) >= 0) {
      return alreadyRejected;
    } else {
      return withFileSaver(_.identity);
    }
  };

  module.exports = CodeGenDialogue = (function(_super) {
    __extends(CodeGenDialogue, _super);

    function CodeGenDialogue() {
      return CodeGenDialogue.__super__.constructor.apply(this, arguments);
    }

    CodeGenDialogue.prototype.Model = CodeGenModel;

    CodeGenDialogue.prototype.parameters = ['query'];

    CodeGenDialogue.prototype.optionalParameters = ['page'];

    CodeGenDialogue.prototype.page = {
      start: 0,
      size: Options.get('DefaultPageSize')
    };

    CodeGenDialogue.prototype.initialize = function() {
      CodeGenDialogue.__super__.initialize.apply(this, arguments);
      this.generateCode();
      return this.setExportLink();
    };

    CodeGenDialogue.prototype.modalSize = function() {
      return 'lg';
    };

    CodeGenDialogue.prototype.title = function() {
      return Messages.getText('codegen.DialogueTitle', {
        query: this.query,
        lang: this.model.get('lang')
      });
    };

    CodeGenDialogue.prototype.primaryIcon = function() {
      return 'Download';
    };

    CodeGenDialogue.prototype.primaryAction = function() {
      return Messages.getText('codegen.PrimaryAction');
    };

    CodeGenDialogue.prototype.body = Templates.template('code_gen_body');

    CodeGenDialogue.prototype.invariants = function() {
      return {
        hasQuery: "No query"
      };
    };

    CodeGenDialogue.prototype.hasQuery = function() {
      return this.query != null;
    };

    CodeGenDialogue.prototype.modelEvents = function() {
      return {
        'change': this.onChangeLang,
        'change:showBoilerPlate': this.reRenderBody,
        'change:highlightSyntax': this.reRenderBody
      };
    };

    CodeGenDialogue.prototype.stateEvents = function() {
      return {
        'change:generatedCode': this.reRenderBody
      };
    };

    CodeGenDialogue.prototype.events = function() {
      return _.extend(CodeGenDialogue.__super__.events.apply(this, arguments), {
        'click .dropdown-menu.im-code-gen-langs li': 'chooseLang'
      });
    };

    CodeGenDialogue.prototype.getBoilerPlateRegex = function() {
      if (this.model.get('showBoilerPlate')) {
        return;
      }
      switch (this.model.get('lang')) {
        case 'pl':
        case 'py':
        case 'rb':
          return OCTOTHORPE_COMMENTS;
        case 'java':
        case 'js':
          return C_STYLE_COMMENTS;
        default:
          return null;
      }
    };

    CodeGenDialogue.prototype.act = function() {
      var blob, filename, lang, _ref;
      lang = this.model.get('lang');
      if (lang === 'js' && this.model.get('extrajs')) {
        lang = 'html';
      }
      blob = new Blob([this.state.get('generatedCode')], {
        type: MIMETYPES[lang]
      });
      filename = "" + ((_ref = this.query.name) != null ? _ref : 'name') + "." + lang;
      return withFileSaver((function(_this) {
        return function(saveAs) {
          return saveAs(blob, filename);
        };
      })(this));
    };

    CodeGenDialogue.prototype.onChangeLang = function() {
      var lang;
      lang = this.model.get('lang');
      this.$('.im-current-lang').text(Messages.getText('codegen.Lang', {
        lang: lang
      }));
      this.$('.modal-title').text(this.title());
      if (lang === 'js' || lang === 'xml') {
        canSaveFromMemory().then((function(_this) {
          return function() {
            return _this.state.unset('error');
          };
        })(this)).then(null, (function(_this) {
          return function() {
            return _this.state.set({
              error: CANNOT_SAVE
            });
          };
        })(this));
      } else {
        this.state.unset('error');
      }
      this.generateCode();
      return this.setExportLink();
    };

    CodeGenDialogue.prototype.generateCode = function() {
      var lang;
      lang = this.model.get('lang');
      switch (lang) {
        case 'xml':
          return this.state.set({
            generatedCode: this.generateXML()
          });
        case 'js':
          return this.state.set({
            generatedCode: this.generateJS()
          });
        default:
          return this.query.fetchCode(lang).then((function(_this) {
            return function(code) {
              return _this.state.set({
                generatedCode: code
              });
            };
          })(this));
      }
    };

    CodeGenDialogue.prototype.generateXML = function() {
      return indentXml(this.query.toXML());
    };

    CodeGenDialogue.prototype.generateJS = function() {
      var cdnBase, data, query, t;
      t = Templates.template('code-gen-js');
      query = stripEmptyValues(this.query.toJSON());
      cdnBase = Options.get('CDN.server') + Options.get(['CDN', 'imtables']);
      data = {
        service: this.query.service,
        query: query,
        page: this.page,
        asHTML: this.model.get('extrajs'),
        imtablesJS: cdnBase + 'imtables.js',
        imtablesCSS: cdnBase + 'main.sandboxed.css'
      };
      return t(data);
    };

    CodeGenDialogue.prototype.setExportLink = function() {
      var lang;
      lang = this.model.get('lang');
      switch (lang) {
        case 'xml':
        case 'js':
          return this.state.set({
            exportLink: null
          });
        default:
          return this.state.set({
            exportLink: this.query.getCodeURI(lang)
          });
      }
    };

    CodeGenDialogue.prototype.reRenderBody = function() {
      if (this.rendered) {
        this.$('.modal-body').html(this.body(this.getData()));
        return this.trigger('rendered', this.rendered);
      }
    };

    CodeGenDialogue.prototype.postRender = function() {
      CodeGenDialogue.__super__.postRender.apply(this, arguments);
      this.addCheckboxes();
      this.highlightCode();
      return this.setMaxHeight();
    };

    CodeGenDialogue.prototype.setMaxHeight = function() {
      var maxHeight;
      maxHeight = Math.max(250, this.$el.closest('.modal').height() - 200);
      return this.$('.im-generated-code').css({
        'max-height': maxHeight
      });
    };

    CodeGenDialogue.prototype.addCheckboxes = function() {
      var opt;
      this.renderChildAt('.im-show-boilerplate', new Checkbox({
        model: this.model,
        attr: 'showBoilerPlate',
        label: 'codegen.ShowBoilerPlate'
      }));
      this.renderChildAt('.im-highlight-syntax', new Checkbox({
        model: this.model,
        attr: 'highlightSyntax',
        label: 'codegen.HighlightSyntax'
      }));
      if ((opt = Options.get(['CodeGen', 'Extra', this.model.get('lang')]))) {
        return this.renderChildAt('.im-extra-options', new Checkbox({
          model: this.model,
          attr: 'extra' + this.model.get('lang'),
          label: opt
        }));
      }
    };

    CodeGenDialogue.prototype.highlightCode = function() {
      var code, lang, pre;
      if (this.model.get('highlightSyntax')) {
        lang = this.model.get('lang');
        pre = this.$('.im-generated-code');
        code = this.getCode();
        if (code == null) {
          return;
        }
        return withPrettyPrintOne(function(prettyPrintOne) {
          return pre.html(prettyPrintOne(_.escape(code)));
        });
      }
    };

    CodeGenDialogue.prototype.getData = function() {
      return _.extend(CodeGenDialogue.__super__.getData.apply(this, arguments), {
        options: Options.get('CodeGen'),
        generatedCode: this.getCode()
      });
    };

    CodeGenDialogue.prototype.getCode = function() {
      var code, regex;
      code = this.state.get('generatedCode');
      regex = this.getBoilerPlateRegex();
      if (!regex) {
        return code;
      }
      return stripExtraneousWhiteSpace(code != null ? code.replace(regex, '') : void 0);
    };

    CodeGenDialogue.prototype.toggleShowBoilerPlate = function() {
      return this.model.toggle('showBoilerPlate');
    };

    CodeGenDialogue.prototype.toggleHighlightSyntax = function() {
      return this.model.toggle('highlightSyntax');
    };

    CodeGenDialogue.prototype.chooseLang = function(e) {
      var lang;
      e.stopPropagation();
      lang = this.$(e.target).closest('li').data('lang');
      return this.model.set({
        lang: lang
      });
    };

    return CodeGenDialogue;

  })(Modal);

}).call(this);
