(function() {
  var AvailableColumns, Collection, ColumnManager, ColumnManagerTabs, IndexedCollection, Messages, Modal, OrderByList, OrderByModel, PathModel, SelectList, SelectListEditor, SortOrderEditor, Templates, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Modal = require('./modal');

  Templates = require('../templates');

  Messages = require('../messages');

  Collection = require('../core/collection');

  PathModel = require('../models/path');

  ColumnManagerTabs = require('./column-manager/tabs');

  SelectListEditor = require('./column-manager/select-list');

  SortOrderEditor = require('./column-manager/sort-order');

  AvailableColumns = require('../models/available-columns');

  OrderByModel = require('../models/order-element');

  require('../messages/columns');

  IndexedCollection = (function(_super) {
    __extends(IndexedCollection, _super);

    IndexedCollection.prototype.comparator = 'index';

    function IndexedCollection() {
      this.model = __bind(this.model, this);
      IndexedCollection.__super__.constructor.apply(this, arguments);
      this.listenTo(this, 'change:index', function() {
        return _.defer((function(_this) {
          return function() {
            return _this.sort();
          };
        })(this));
      });
    }

    IndexedCollection.prototype.modelFactory = Collection.prototype.model;

    IndexedCollection.prototype.model = function(args) {
      var index, model;
      index = this.size();
      model = new this.modelFactory(args);
      model.set({
        index: index
      });
      return model;
    };

    return IndexedCollection;

  })(Collection);

  SelectList = (function(_super) {
    __extends(SelectList, _super);

    function SelectList() {
      return SelectList.__super__.constructor.apply(this, arguments);
    }

    SelectList.prototype.modelFactory = PathModel;

    return SelectList;

  })(IndexedCollection);

  OrderByList = (function(_super) {
    __extends(OrderByList, _super);

    function OrderByList() {
      return OrderByList.__super__.constructor.apply(this, arguments);
    }

    OrderByList.prototype.modelFactory = OrderByModel;

    return OrderByList;

  })(IndexedCollection);

  module.exports = ColumnManager = (function(_super) {
    __extends(ColumnManager, _super);

    function ColumnManager() {
      return ColumnManager.__super__.constructor.apply(this, arguments);
    }

    ColumnManager.prototype.parameters = ['query'];

    ColumnManager.prototype.modalSize = function() {
      return 'lg';
    };

    ColumnManager.prototype.className = function() {
      return ColumnManager.__super__.className.apply(this, arguments) + ' im-column-manager';
    };

    ColumnManager.prototype.title = function() {
      return Messages.getText('columns.DialogueTitle');
    };

    ColumnManager.prototype.primaryAction = function() {
      return Messages.getText('columns.ApplyChanges');
    };

    ColumnManager.prototype.dismissAction = function() {
      return Messages.getText('Cancel');
    };

    ColumnManager.prototype.act = function() {
      var opts, orderChanged, viewChanged, _ref;
      if (!this.state.get('disabled')) {
        _ref = this.state.pick('viewChanged', 'orderChanged'), viewChanged = _ref.viewChanged, orderChanged = _ref.orderChanged;
        if (viewChanged && (!orderChanged)) {
          this.query.select(this.getCurrentView());
        } else if (orderChanged && (!viewChanged)) {
          this.query.orderBy(this.getCurrentSortOrder());
        } else {
          opts = {
            silent: true,
            events: []
          };
          this.query.select(this.getCurrentView(), opts);
          this.query.orderBy(this.getCurrentSortOrder(), opts);
          this.query.trigger(opts.events.join(' '));
        }
        return this.resolve('changed');
      }
    };

    ColumnManager.prototype.stateEvents = function() {
      return {
        'change:currentTab': this.renderTabContent,
        'change:adding': this.setDisabled
      };
    };

    ColumnManager.prototype.initialize = function() {
      var direction, path, v, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2, _ref3;
      ColumnManager.__super__.initialize.apply(this, arguments);
      this.selectList = new SelectList;
      this.rubbishBin = new SelectList;
      this.sortOrder = new OrderByList;
      this.availableColumns = new AvailableColumns;
      _ref = this.query.views;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        v = _ref[_i];
        this.selectList.add(this.query.makePath(v));
      }
      _ref1 = this.query.sortOrder;
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        _ref2 = _ref1[_j], path = _ref2.path, direction = _ref2.direction;
        this.sortOrder.add({
          direction: direction,
          path: this.query.makePath(path)
        });
      }
      _ref3 = this.getRelevantPaths();
      for (_k = 0, _len2 = _ref3.length; _k < _len2; _k++) {
        path = _ref3[_k];
        if (!this.sortOrder.get(path.toString())) {
          this.availableColumns.add(path, {
            sort: false
          });
        }
      }
      this.availableColumns.sort();
      this.listenTo(this.selectList, 'sort add remove', this.setDisabled);
      return this.listenTo(this.sortOrder, 'sort add remove', this.setDisabled);
    };

    ColumnManager.prototype.getRelevantPaths = function() {
      return _.chain(this.query.getQueryNodes()).filter((function(_this) {
        return function(n) {
          return !_this.query.isOuterJoined(n);
        };
      })(this)).map(function(n) {
        var cn, _i, _len, _ref, _results;
        _ref = n.getChildNodes();
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          cn = _ref[_i];
          if (cn.isAttribute() && (cn.end.name !== 'id')) {
            _results.push(cn);
          }
        }
        return _results;
      }).flatten().value();
    };

    ColumnManager.prototype.getCurrentView = function() {
      return this.selectList.pluck('path');
    };

    ColumnManager.prototype.getCurrentSortOrder = function() {
      return this.sortOrder.map(function(m) {
        return m.asOrderElement();
      });
    };

    ColumnManager.prototype.setDisabled = function() {
      var currentSO, currentView, initialSO, initialView, soUnchanged, viewUnchanged;
      if (this.state.get('adding')) {
        return this.state.set({
          disabled: true
        });
      }
      currentView = this.getCurrentView().join(' ');
      initialView = this.query.views.join(' ');
      currentSO = this.sortOrder.map(function(m) {
        return m.toOrderString();
      }).join(' ');
      initialSO = this.query.getSorting();
      viewUnchanged = currentView === initialView;
      soUnchanged = currentSO === initialSO;
      return this.state.set({
        viewChanged: !viewUnchanged,
        orderChanged: !soUnchanged,
        disabled: viewUnchanged && soUnchanged
      });
    };

    ColumnManager.prototype.initState = function() {
      return this.state.set({
        disabledReason: 'columns.NoChangesToApply',
        disabled: true,
        currentTab: ColumnManagerTabs.TABS[0]
      });
    };

    ColumnManager.prototype.renderTabs = function() {
      return this.renderChild('tabs', new ColumnManagerTabs({
        state: this.state
      }), this.$('.modal-body'));
    };

    ColumnManager.prototype.renderTabContent = function() {
      var main;
      if (this.rendered) {
        main = (function() {
          switch (this.state.get('currentTab')) {
            case 'view':
              return new SelectListEditor({
                state: this.state,
                query: this.query,
                rubbishBin: this.rubbishBin,
                collection: this.selectList
              });
            case 'sortorder':
              return new SortOrderEditor({
                query: this.query,
                availableColumns: this.availableColumns,
                collection: this.sortOrder
              });
            default:
              throw new Error("Cannot render " + (this.state.get('currentTab')));
          }
        }).call(this);
        return this.renderChild('main', main, this.$('.modal-body'));
      }
    };

    ColumnManager.prototype.postRender = function() {
      ColumnManager.__super__.postRender.apply(this, arguments);
      this.renderTabs();
      return this.renderTabContent();
    };

    ColumnManager.prototype.remove = function() {
      this.selectList.close();
      this.rubbishBin.close();
      this.sortOrder.close();
      this.availableColumns.close();
      return ColumnManager.__super__.remove.apply(this, arguments);
    };

    return ColumnManager;

  })(Modal);

}).call(this);
