(function() {
  var Buttons, ColumnChooser, CoreView, OpenNodes, Options, PathChooser, PathSet, Templates,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Options = require('../../options');

  PathSet = require('../../models/path-set');

  OpenNodes = require('../../models/open-nodes');

  PathChooser = require('../path-chooser');

  Buttons = (function(_super) {
    __extends(Buttons, _super);

    function Buttons() {
      return Buttons.__super__.constructor.apply(this, arguments);
    }

    Buttons.prototype.parameters = ['collection', 'selectList'];

    Buttons.prototype.template = Templates.template('column-manager-path-chooser-buttons');

    Buttons.prototype.collectionEvents = function() {
      return {
        'add remove reset': 'reRender'
      };
    };

    Buttons.prototype.events = function() {
      return {
        'click .im-add-column': 'addColumn',
        'click .im-rearrange-columns': 'cancel'
      };
    };

    Buttons.prototype.cancel = function() {
      return this.trigger('done');
    };

    Buttons.prototype.addColumn = function() {
      var paths, selectList;
      selectList = this.selectList;
      paths = this.collection.paths();
      this.cancel();
      return selectList.add(paths);
    };

    return Buttons;

  })(CoreView);

  module.exports = ColumnChooser = (function(_super) {
    __extends(ColumnChooser, _super);

    function ColumnChooser() {
      return ColumnChooser.__super__.constructor.apply(this, arguments);
    }

    ColumnChooser.prototype.parameters = ['query', 'collection'];

    ColumnChooser.prototype.template = Templates.template('column-manager-path-chooser');

    ColumnChooser.prototype.initialize = function() {
      var p;
      ColumnChooser.__super__.initialize.apply(this, arguments);
      this.chosenPaths = new PathSet;
      this.view = new PathSet((function() {
        var _i, _len, _ref, _results;
        _ref = this.collection.pluck('path');
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          p = _ref[_i];
          _results.push(this.query.makePath(p));
        }
        return _results;
      }).call(this));
      return this.openNodes = new OpenNodes(this.query.getViewNodes());
    };

    ColumnChooser.prototype.initState = function() {
      return this.query.makePath(this.query.root).getDisplayName().then((function(_this) {
        return function(name) {
          return _this.state.set({
            rootName: name
          });
        };
      })(this));
    };

    ColumnChooser.prototype.stateEvents = function() {
      return {
        'change:rootName': 'reRender'
      };
    };

    ColumnChooser.prototype.postRender = function() {
      this.renderButtons();
      return this.openPathChooser();
    };

    ColumnChooser.prototype.renderButtons = function() {
      var btns;
      btns = new Buttons({
        state: this.state,
        collection: this.chosenPaths,
        selectList: this.collection
      });
      this.listenTo(btns, 'done', (function(_this) {
        return function() {
          return _this.trigger('done');
        };
      })(this));
      return this.renderChildAt('.btn-group', btns);
    };

    ColumnChooser.prototype.openPathChooser = function() {
      var model, pathChooser;
      model = {
        dontSelectView: true,
        multiSelect: Options.get('ColumnManager.SelectColumn.Multi')
      };
      pathChooser = new PathChooser({
        model: model,
        query: this.query,
        chosenPaths: this.chosenPaths,
        openNodes: this.openNodes,
        view: this.view,
        trail: []
      });
      this.renderChild('pathChooser', pathChooser);
      return this.setPathChooserHeight();
    };

    ColumnChooser.prototype.setPathChooserHeight = function() {
      return this.$('.im-path-chooser').css({
        'max-height': this.$el.closest('.modal').height() - 350
      });
    };

    ColumnChooser.prototype.remove = function() {
      this.chosenPaths.close();
      this.view.close();
      this.openNodes.close();
      return ColumnChooser.__super__.remove.apply(this, arguments);
    };

    return ColumnChooser;

  })(CoreView);

}).call(this);
