(function() {
  var ColumnChooser, CoreView, HandlesDOMReSort, SelectListEditor, SelectedColumn, Templates, UnselectedColumn, binnedId, childId, incr, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  HandlesDOMReSort = require('../../mixins/handles-dom-resort');

  SelectedColumn = require('./selected-column');

  UnselectedColumn = require('./unselected-column');

  ColumnChooser = require('./path-chooser');

  childId = function(model) {
    return "path_" + (model.get('id'));
  };

  binnedId = function(model) {
    return "expath_" + (model.get('id'));
  };

  incr = function(x) {
    return x + 1;
  };

  module.exports = SelectListEditor = (function(_super) {
    __extends(SelectListEditor, _super);

    function SelectListEditor() {
      return SelectListEditor.__super__.constructor.apply(this, arguments);
    }

    SelectListEditor.include(HandlesDOMReSort);

    SelectListEditor.prototype.parameters = ['query', 'collection', 'rubbishBin'];

    SelectListEditor.prototype.className = 'im-select-list-editor';

    SelectListEditor.prototype.template = Templates.template('column-manager-select-list');

    SelectListEditor.prototype.getData = function() {
      return _.extend(SelectListEditor.__super__.getData.apply(this, arguments), {
        hasRubbish: this.rubbishBin.size()
      });
    };

    SelectListEditor.prototype.initialize = function() {
      SelectListEditor.__super__.initialize.apply(this, arguments);
      return this.listenTo(this.rubbishBin, 'remove', this.restoreView);
    };

    SelectListEditor.prototype.initState = function() {
      return this.state.set({
        adding: false
      });
    };

    SelectListEditor.prototype.collectionEvents = function() {
      return {
        'remove': 'moveToBin',
        'add remove': 'reRender'
      };
    };

    SelectListEditor.prototype.events = function() {
      return {
        'drop .im-rubbish-bin': 'onDragToBin',
        'sortupdate .im-active-view': 'onOrderChanged',
        'click .im-add-view-path': 'setAddingTrue'
      };
    };

    SelectListEditor.prototype.stateEvents = function() {
      return {
        'change:adding': 'onChangeMode'
      };
    };

    SelectListEditor.prototype.setAddingTrue = function() {
      return this.state.set({
        adding: true
      });
    };

    SelectListEditor.prototype.onDragToBin = function(e, ui) {
      return ui.draggable.trigger('binned');
    };

    SelectListEditor.prototype.moveToBin = function(model) {
      this.rubbishBin.add(this.query.makePath(model.get('path')));
      return this.reIndexFrom(model.get('index'));
    };

    SelectListEditor.prototype.reIndexFrom = function(idx) {
      var i, _i, _ref, _ref1, _results;
      _results = [];
      for (i = _i = idx, _ref = this.collection.size(); idx <= _ref ? _i <= _ref : _i >= _ref; i = idx <= _ref ? ++_i : --_i) {
        _results.push((_ref1 = this.collection.at(i)) != null ? _ref1.set({
          index: i
        }) : void 0);
      }
      return _results;
    };

    SelectListEditor.prototype.restoreView = function(model) {
      return this.collection.add(this.query.makePath(model.get('path')));
    };

    SelectListEditor.prototype.onChangeMode = function() {
      if (this.rendered) {
        if (this.state.get('adding')) {
          this.$('.im-removal-and-rearrangement').hide();
          this.$('.im-addition').show();
          return this.renderPathChooser();
        } else {
          this.$('.im-removal-and-rearrangement').show();
          this.$('.im-addition').hide();
          return this.removeChild('columnChooser');
        }
      }
    };

    SelectListEditor.prototype.renderPathChooser = function() {
      var columns;
      columns = new ColumnChooser({
        query: this.query,
        collection: this.collection
      });
      this.listenTo(columns, 'done', (function(_this) {
        return function() {
          return _this.state.set({
            adding: false
          });
        };
      })(this));
      return this.renderChild('columnChooser', columns, this.$('.im-addition'));
    };

    SelectListEditor.prototype.onOrderChanged = function(e, ui) {
      if (ui.sender != null) {
        return this.restoreFromBin(ui.item);
      } else {
        return this.onDOMResort();
      }
    };

    SelectListEditor.prototype.getInsertPoint = function($el) {
      var addedAfter, kids, prevModel;
      addedAfter = $el.prev();
      kids = this.children;
      prevModel = this.collection.find(function(m) {
        var active;
        active = kids[childId(m)];
        return active.el === addedAfter[0];
      });
      if (prevModel == null) {
        return 0;
      } else {
        return prevModel.get('index') + 1;
      }
    };

    SelectListEditor.prototype.getRestoredModel = function($el) {
      return this.rubbishBin.find((function(_this) {
        return function(m) {
          var binned;
          binned = _this.children[binnedId(m)];
          return (binned != null ? binned.el : void 0) === $el[0];
        };
      })(this));
    };

    SelectListEditor.prototype.restoreFromBin = function($el) {
      var added, addedAt, i, kids, m, preAddSize, toBump, toRestore, _i, _len;
      kids = this.children;
      preAddSize = this.collection.size();
      addedAt = this.getInsertPoint($el);
      toRestore = this.getRestoredModel($el);
      if (toRestore != null) {
        toRestore.destroy();
      } else {
        console.error('could not find model for:', $el);
        return;
      }
      if (addedAt === preAddSize) {
        return;
      }
      toBump = (function() {
        var _i, _results;
        _results = [];
        for (i = _i = addedAt; addedAt <= preAddSize ? _i < preAddSize : _i > preAddSize; i = addedAt <= preAddSize ? ++_i : --_i) {
          _results.push(this.collection.at(i));
        }
        return _results;
      }).call(this);
      added = this.collection.last();
      for (_i = 0, _len = toBump.length; _i < _len; _i++) {
        m = toBump[_i];
        m.swap('index', incr);
      }
      added.set({
        index: addedAt
      });
      return kids[childId(added)].$el.insertBefore(kids[childId(toBump[0])].el);
    };

    SelectListEditor.prototype.onDOMResort = function() {
      return this.setChildIndices(childId);
    };

    SelectListEditor.prototype.postRender = function() {
      var binnedCols, columns, cutoff, modalWidth, wide;
      columns = this.$('.im-active-view');
      binnedCols = this.$('.im-removed-view');
      cutoff = 900;
      modalWidth = this.$el.closest('.modal').width();
      wide = modalWidth >= cutoff;
      this.collection.each((function(_this) {
        return function(model) {
          return _this.renderChild(childId(model), new SelectedColumn({
            model: model
          }), columns);
        };
      })(this));
      this.rubbishBin.each((function(_this) {
        return function(model) {
          return _this.renderChild(binnedId(model), new UnselectedColumn({
            model: model
          }), binnedCols);
        };
      })(this));
      columns.sortable({
        placeholder: 'im-view-list-placeholder',
        opacity: 0.6,
        cancel: 'i,a,button',
        axis: (wide ? null : 'y'),
        appendTo: this.el
      });
      this.$('.im-removed-view').sortable({
        placeholder: 'im-view-list-placeholder',
        connectWith: columns,
        opacity: 0.6,
        cancel: 'i,a,button',
        axis: (wide ? null : 'y'),
        appendTo: this.el
      });
      this.$('.im-rubbish-bin').droppable({
        accept: '.im-selected-column',
        activeClass: 'im-can-remove-column',
        hoverClass: 'im-will-remove-column'
      });
      return this.onChangeMode();
    };

    SelectListEditor.prototype.removeAllChildren = function() {
      if (this.rendered) {
        this.$('.im-active-view').sortable('destroy');
        this.$('.im-removed-view').sortable('destroy');
        this.$('.im-rubbish-bin').droppable('destroy');
      }
      return SelectListEditor.__super__.removeAllChildren.apply(this, arguments);
    };

    return SelectListEditor;

  })(CoreView);

}).call(this);
