(function() {
  var AvailablePath, CoreModel, CoreView, HandlesDOMReSort, OrderElement, SortOrderEditor, Templates, activeId, inactiveId, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  CoreModel = require('../../core-model');

  Templates = require('../../templates');

  HandlesDOMReSort = require('../../mixins/handles-dom-resort');

  AvailablePath = require('./available-path');

  OrderElement = require('./order-element');

  activeId = function(model) {
    return "active_" + (model.get('id'));
  };

  inactiveId = function(model) {
    return "inactive_" + (model.get('id'));
  };

  module.exports = SortOrderEditor = (function(_super) {
    __extends(SortOrderEditor, _super);

    function SortOrderEditor() {
      return SortOrderEditor.__super__.constructor.apply(this, arguments);
    }

    SortOrderEditor.include(HandlesDOMReSort);

    SortOrderEditor.prototype.parameters = ['collection', 'query', 'availableColumns'];

    SortOrderEditor.prototype.className = 'im-sort-order-editor';

    SortOrderEditor.prototype.template = Templates.template('column-manager-sort-order-editor');

    SortOrderEditor.prototype.getData = function() {
      return _.extend(SortOrderEditor.__super__.getData.apply(this, arguments), {
        available: this.availableColumns.size()
      });
    };

    SortOrderEditor.prototype.collectionEvents = function() {
      return {
        'add remove': this.reRender,
        'sort': this.resortSortOrder,
        'remove': this.makeAvailable
      };
    };

    SortOrderEditor.prototype.initialize = function() {
      SortOrderEditor.__super__.initialize.apply(this, arguments);
      this.dragState = new CoreModel;
      this.listenTo(this.availableColumns, 'sort add remove', this.resortAvailable);
      return this.listenTo(this.availableColumns, 'remove', this.addToSortOrder);
    };

    SortOrderEditor.prototype.currentSortOrder = function() {
      return this.collection.map(function(m) {
        return "" + (m.get('path')) + " " + (m.get('direction'));
      }).join(' ');
    };

    SortOrderEditor.prototype.postRender = function() {
      this.resortSortOrder();
      this.activateSortables();
      this.resortAvailable();
      return this.setAvailableHeight();
    };

    SortOrderEditor.prototype.activateSortables = function() {
      var active, cutoff, modalWidth, wide;
      active = this.$('.im-active-oes');
      cutoff = 900;
      modalWidth = this.$el.closest('.modal').width();
      wide = modalWidth >= cutoff;
      if (this.collection.size()) {
        return this.$actives = active.sortable({
          placeholder: 'im-view-list-placeholder',
          opacity: 0.6,
          cancel: 'i,a,button',
          axis: (wide ? null : 'y'),
          appendTo: this.el
        });
      } else {
        return this.$droppable = this.$('.im-empty-collection').droppable({
          accept: '.im-selected-column',
          activeClass: 'im-can-add-column',
          hoverClass: 'im-will-add-column'
        });
      }
    };

    SortOrderEditor.prototype.removeAllChildren = function() {
      var _ref, _ref1;
      if ((_ref = this.$actives) != null) {
        _ref.sortable('destroy');
      }
      if ((_ref1 = this.$droppable) != null) {
        _ref1.droppable('destroy');
      }
      this.$droppable = null;
      this.$actives = null;
      return SortOrderEditor.__super__.removeAllChildren.apply(this, arguments);
    };

    SortOrderEditor.prototype.events = function() {
      return {
        'drop .im-empty-collection': 'addSortElement',
        'sortupdate .im-active-oes': 'onDOMResort'
      };
    };

    SortOrderEditor.prototype.onDOMResort = function(e, ui) {
      var indexAt, model, path;
      if (path = this.dragState.get('dragged')) {
        model = this.availableColumns.findWhere({
          path: path
        });
        indexAt = ui.item.prevAll().length;
        this.addToSortOrder(model, indexAt);
        return this.dragState.unset('dragged');
      } else {
        return this.setChildIndices(activeId);
      }
    };

    SortOrderEditor.prototype.makeAvailable = function(active) {
      return this.availableColumns.add(this.query.makePath(active.get('path')));
    };

    SortOrderEditor.prototype.findAvailable = function(el) {
      return this.availableColumns.find((function(_this) {
        return function(m) {
          var _ref;
          return ((_ref = _this.children[inactiveId(m)]) != null ? _ref.el : void 0) === el;
        };
      })(this));
    };

    SortOrderEditor.prototype.addSortElement = function(e, ui) {
      var $el, available;
      $el = ui.draggable;
      available = this.findAvailable($el[0]);
      return this.addToSortOrder(available);
    };

    SortOrderEditor.prototype.addToSortOrder = function(availableColumnModel, atIndex) {
      var added, b, oe, path, sizeBeforeAdd, toBump, _i, _len;
      path = this.query.makePath(availableColumnModel.get('path'));
      oe = {
        id: String(path),
        path: path
      };
      sizeBeforeAdd = this.collection.size();
      availableColumnModel.destroy();
      this.collection.add(oe);
      if ((atIndex != null) && (atIndex < sizeBeforeAdd)) {
        toBump = this.collection.filter(function(m) {
          return m.get('index') >= atIndex;
        });
        added = this.collection.last();
        for (_i = 0, _len = toBump.length; _i < _len; _i++) {
          b = toBump[_i];
          b.swap('index', function(idx) {
            return idx + 1;
          });
        }
        return added.set({
          index: atIndex
        });
      }
    };

    SortOrderEditor.prototype.resortAvailable = function() {
      var frag;
      if (this.rendered) {
        frag = global.document.createDocumentFragment();
        this.availableColumns.each((function(_this) {
          return function(model) {
            return _this._renderAvailable(model, frag);
          };
        })(this));
        return this.$('.im-available-oes').html(frag);
      }
    };

    SortOrderEditor.prototype._renderAvailable = function(model, frag) {
      var findActives, name, view;
      name = inactiveId(model);
      findActives = (function(_this) {
        return function() {
          return _this.$actives;
        };
      })(this);
      view = new AvailablePath({
        model: model,
        findActives: findActives,
        state: this.dragState
      });
      return this.renderChild(name, view, frag);
    };

    SortOrderEditor.prototype.resortSortOrder = function() {
      var frag;
      if (this.rendered) {
        frag = global.document.createDocumentFragment();
        this.collection.each((function(_this) {
          return function(model) {
            return _this.renderChild(activeId(model), new OrderElement({
              model: model
            }), frag);
          };
        })(this));
        return this.$('.im-active-oes').html(frag);
      }
    };

    SortOrderEditor.prototype.setAvailableHeight = function() {
      return this.$('.im-rubbish-bin').css({
        'max-height': Math.max(200, this.$el.closest('.modal').height() - 450)
      });
    };

    SortOrderEditor.prototype.remove = function() {
      this.dragState.destroy();
      return SortOrderEditor.__super__.remove.apply(this, arguments);
    };

    return SortOrderEditor;

  })(CoreView);

}).call(this);
