(function() {
  var ConstraintAdderOptions, HasTypeaheads, IS_BLANK, Options, Promise, Templates, View, getPathSuggestions, pathSuggester, shortenLongName, _,
    __slice = [].slice,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Promise = require('es6-promise').Promise;

  View = require('../core-view');

  Options = require('../options');

  Templates = require('../templates');

  HasTypeaheads = require('../mixins/has-typeaheads');

  IS_BLANK = require('../patterns').IS_BLANK;

  pathSuggester = require('../utils/path-suggester');

  getPathSuggestions = require('../utils/path-suggestions');

  shortenLongName = function(name) {
    var parts, rest, x, y, z, _i;
    parts = name.split(' > ');
    if (parts.length > 3) {
      rest = 4 <= parts.length ? __slice.call(parts, 0, _i = parts.length - 3) : (_i = 0, []), x = parts[_i++], y = parts[_i++], z = parts[_i++];
      return "..." + x + " > " + y + " > " + z;
    } else {
      return name;
    }
  };

  module.exports = ConstraintAdderOptions = (function(_super) {
    __extends(ConstraintAdderOptions, _super);

    function ConstraintAdderOptions() {
      return ConstraintAdderOptions.__super__.constructor.apply(this, arguments);
    }

    ConstraintAdderOptions.include(HasTypeaheads);

    ConstraintAdderOptions.prototype.className = 'row';

    ConstraintAdderOptions.prototype.initialize = function(_arg) {
      this.query = _arg.query, this.openNodes = _arg.openNodes, this.chosenPaths = _arg.chosenPaths;
      ConstraintAdderOptions.__super__.initialize.apply(this, arguments);
      this.state.set({
        chosen: []
      });
      this.listenTo(this.openNodes, 'add remove reset', this.reRender);
      this.listenTo(this.chosenPaths, 'add remove reset', this.reRender);
      this.listenTo(this.chosenPaths, 'add remove reset', this.setChosen);
      this.setChosen();
      return this.generatePathSuggestions();
    };

    ConstraintAdderOptions.prototype.modelEvents = function() {
      return {
        destroy: this.stopListeningTo,
        'change:showTree change:allowRevRefs': this.reRender
      };
    };

    ConstraintAdderOptions.prototype.stateEvents = function() {
      return {
        'change:chosen change:suggestions': this.reRender
      };
    };

    ConstraintAdderOptions.prototype.pathAcceptable = function(path) {
      var _ref;
      if (((_ref = path.end) != null ? _ref.name : void 0) === 'id') {
        return false;
      }
      if (!this.model.get('canSelectReferences')) {
        return path.isAttribute();
      }
      return true;
    };

    ConstraintAdderOptions.prototype.generatePathSuggestions = function() {
      var depth;
      depth = Options.get('SuggestionDepth');
      return getPathSuggestions(this.query, depth).then((function(_this) {
        return function(suggestions) {
          return _this.state.set({
            suggestions: suggestions.filter(function(s) {
              return _this.pathAcceptable(s.path);
            })
          });
        };
      })(this));
    };

    ConstraintAdderOptions.prototype.getData = function() {
      var anyNodeChosen, anyNodesOpen;
      anyNodesOpen = this.openNodes.size();
      anyNodeChosen = this.chosenPaths.size();
      return _.extend({
        anyNodesOpen: anyNodesOpen,
        anyNodeChosen: anyNodeChosen
      }, this.state.toJSON(), ConstraintAdderOptions.__super__.getData.apply(this, arguments));
    };

    ConstraintAdderOptions.prototype.template = Templates.template('constraint_adder_options');

    ConstraintAdderOptions.prototype.render = function() {
      ConstraintAdderOptions.__super__.render.apply(this, arguments);
      if (this.state.has('suggestions')) {
        this.installTypeahead();
      }
      return this;
    };

    ConstraintAdderOptions.prototype.installTypeahead = function() {
      var dataset, input, opts, suggest, suggestions;
      this.removeTypeAheads();
      input = this.$('.im-tree-filter');
      suggestions = this.state.get('suggestions');
      suggest = pathSuggester(suggestions);
      opts = {
        minLength: 3,
        highlight: true
      };
      dataset = {
        name: 'path_suggestions',
        source: suggest,
        displayKey: 'name'
      };
      return this.activateTypeahead(input, opts, dataset, suggestions[0].name, (function(_this) {
        return function(e, suggestion) {
          var path;
          path = suggestion.path;
          _this.openNodes.add(path);
          if (_this.model.get('multiSelect')) {
            return _this.chosenPaths.add(path);
          } else {
            return _this.chosenPaths.reset([path]);
          }
        };
      })(this));
    };

    ConstraintAdderOptions.prototype.events = function() {
      return {
        'click .im-collapser': 'collapseBranches',
        'change .im-allow-rev-ref': 'toggleReverseRefs',
        'change .im-tree-filter': 'setFilter',
        'click .im-choose': 'toggleShowTree',
        'click .im-approve': 'triggerApproval',
        'click .im-clear-filter': 'clearFilter'
      };
    };

    ConstraintAdderOptions.prototype.triggerApproval = function() {
      return this.model.trigger('approved');
    };

    ConstraintAdderOptions.prototype.remove = function() {
      this.removeTypeAheads();
      return ConstraintAdderOptions.__super__.remove.apply(this, arguments);
    };

    ConstraintAdderOptions.prototype.clearFilter = function() {
      this.model.set({
        filter: null
      });
      return this.reRender();
    };

    ConstraintAdderOptions.prototype.setFilter = function(e) {
      var value;
      value = e.target.value;
      return this.model.set({
        filter: (IS_BLANK.test(value) ? null : value)
      });
    };

    ConstraintAdderOptions.prototype.collapseBranches = function() {
      return this.openNodes.reset();
    };

    ConstraintAdderOptions.prototype.toggleShowTree = function() {
      return this.model.toggle('showTree');
    };

    ConstraintAdderOptions.prototype.toggleReverseRefs = function() {
      return this.model.toggle('allowRevRefs');
    };

    ConstraintAdderOptions.prototype.setConstraint = function() {
      return this.model.trigger('approved');
    };

    ConstraintAdderOptions.prototype.setChosen = function() {
      var naming, p, paths;
      paths = this.chosenPaths.toJSON();
      naming = Promise.all((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = paths.length; _i < _len; _i++) {
          p = paths[_i];
          _results.push(p.getDisplayName());
        }
        return _results;
      })());
      return naming.then(function(names) {
        var n, _i, _len, _results;
        _results = [];
        for (_i = 0, _len = names.length; _i < _len; _i++) {
          n = names[_i];
          _results.push(shortenLongName(n));
        }
        return _results;
      }).then(((function(_this) {
        return function(names) {
          return _this.state.set({
            chosen: names
          });
        };
      })(this)), (function(e) {
        return console.error(e);
      }));
    };

    return ConstraintAdderOptions;

  })(View);

}).call(this);
