(function() {
  var ConstraintAdder, ConstraintAdderModel, ConstraintAdderOptions, CoreModel, Messages, NewConstraint, OPTIONS_SEL, OpenNodes, PathChooser, PathSet, Templates, View, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  Messages = require('../messages');

  Templates = require('../templates');

  View = require('../core-view');

  CoreModel = require('../core-model');

  PathSet = require('../models/path-set');

  OpenNodes = require('../models/open-nodes');

  ConstraintAdderOptions = require('./constraint-adder-options');

  NewConstraint = require('./new-constraint');

  PathChooser = require('./path-chooser');

  Messages.set({
    'constraints.BrowseForColumn': 'Browse for Column',
    'constraints.AddANewFilter': 'Add a new filter',
    'constraints.Choose': 'Choose',
    'constraints.Filter': 'Filter',
    'columns.CollapseAll': 'Collapse columns',
    'columns.AllowRevRef': 'Allow reverse references'
  });

  ConstraintAdderModel = (function(_super) {
    __extends(ConstraintAdderModel, _super);

    function ConstraintAdderModel() {
      return ConstraintAdderModel.__super__.constructor.apply(this, arguments);
    }

    ConstraintAdderModel.prototype.defaults = function() {
      return {
        filter: null,
        showTree: true,
        allowRevRefs: false,
        canSelectReferences: true,
        multiSelect: false
      };
    };

    return ConstraintAdderModel;

  })(CoreModel);

  OPTIONS_SEL = '.im-constraint-adder-options';

  module.exports = ConstraintAdder = (function(_super) {
    __extends(ConstraintAdder, _super);

    function ConstraintAdder() {
      this.showTree = __bind(this.showTree, this);
      return ConstraintAdder.__super__.constructor.apply(this, arguments);
    }

    ConstraintAdder.prototype.tagName = 'div';

    ConstraintAdder.prototype.className = 'im-constraint-adder row-fluid';

    ConstraintAdder.prototype.Model = ConstraintAdderModel;

    ConstraintAdder.prototype.initialize = function(_arg) {
      var p;
      this.query = _arg.query, this.buttonDelegate = _arg.buttonDelegate;
      ConstraintAdder.__super__.initialize.apply(this, arguments);
      this.model.set({
        root: this.query.getPathInfo(this.query.root)
      });
      this.chosenPaths = new PathSet;
      this.view = new PathSet((function() {
        var _i, _len, _ref, _results;
        _ref = this.query.views;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          p = _ref[_i];
          _results.push(this.query.makePath(p));
        }
        return _results;
      }).call(this));
      this.openNodes = new OpenNodes(this.query.getViewNodes());
      return this.listenTo(this.query, 'change:constraints', this.remove);
    };

    ConstraintAdder.prototype.modelEvents = function() {
      return {
        approved: this.handleApproval,
        'change:showTree': this.toggleTree,
        'change:constraint': this.onChangeConstraint
      };
    };

    ConstraintAdder.prototype.getTreeRoot = function() {
      return this.model.get('root');
    };

    ConstraintAdder.prototype.handleApproval = function() {
      var chosen, constraint, current, newPath;
      chosen = this.chosenPaths.toJSON()[0];
      if (chosen != null) {
        current = this.model.get('constraint');
        newPath = chosen.toString();
        if ((current != null ? current.path : void 0) !== newPath) {
          constraint = {
            path: newPath
          };
          this.model.set({
            constraint: constraint
          });
          return this.query.trigger('editing-constraint', constraint);
        } else {
          return this.onChangeConstraint();
        }
      } else {
        console.debug('nothing chosen');
        return this.model.unset('constraint');
      }
    };

    ConstraintAdder.prototype.renderConstraintEditor = function() {
      var constraint, editor;
      constraint = this.model.get('constraint');
      editor = new NewConstraint({
        buttonDelegate: this.buttonDelegate,
        query: this.query,
        constraint: constraint
      });
      this.renderChild('con', editor, this.$('.im-new-constraint'));
      return this.listenTo(editor, 'remove', function() {
        return this.model.set({
          showTree: true
        });
      });
    };

    ConstraintAdder.prototype.onChangeConstraint = function() {
      if (this.rendered) {
        if (this.model.get('constraint')) {
          this.model.set({
            showTree: false
          });
          return this.renderConstraintEditor();
        } else {
          return this.removeChild('con');
        }
      }
    };

    ConstraintAdder.prototype.toggleTree = function() {
      if (this.model.get('showTree')) {
        return this.showTree();
      } else {
        return this.hideTree();
      }
    };

    ConstraintAdder.prototype.hideTree = function() {
      this.trigger('resetting:tree');
      this.$('.im-path-finder').removeClass('open');
      return this.removeChild('tree');
    };

    ConstraintAdder.prototype.showTree = function(e) {
      var pathFinder;
      this.trigger('showing:tree');
      this.removeChild('con');
      pathFinder = new PathChooser({
        model: this.model,
        query: this.query,
        chosenPaths: this.chosenPaths,
        openNodes: this.openNodes,
        view: this.view,
        trail: []
      });
      this.$('.im-path-finder').addClass('open');
      return this.renderChild('tree', pathFinder, this.$('.im-path-finder'));
    };

    ConstraintAdder.prototype.template = Templates.template('constraint_adder');

    ConstraintAdder.prototype.renderOptions = function() {
      var opts;
      opts = {
        model: this.model,
        openNodes: this.openNodes,
        chosenPaths: this.chosenPaths,
        query: this.query
      };
      return this.renderChild('opts', new ConstraintAdderOptions(opts), this.$(OPTIONS_SEL));
    };

    ConstraintAdder.prototype.postRender = function() {
      this.renderOptions();
      return this.toggleTree();
    };

    return ConstraintAdder;

  })(View);

}).call(this);
