(function() {
  var ATTRIBUTE_OPS, ATTRIBUTE_VALUE_OPS, AttributeValueControls, BASIC_OPS, BOOLEAN_TYPES, BooleanValueControls, ConstraintEditor, CoreView, ErrorMessage, Icons, LIST_OPS, ListValueControls, LookupValueControls, LoopValueControls, MULTIVALUE_OPS, Messages, Model, MultiValueControls, NO_OP, NULL_OPS, NUMERIC_TYPES, Query, REFERENCE_OPS, TEMPLATE, TypeValueControls, fs, html, operatorsFor, _, _ref,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  

  Messages = require('../messages');

  Icons = require('../icons');

  CoreView = require('../core-view');

  TypeValueControls = require('./type-value-controls');

  AttributeValueControls = require('./attribute-value-controls');

  MultiValueControls = require('./multi-value-controls');

  BooleanValueControls = require('./boolean-value-controls');

  LookupValueControls = require('./lookup-value-controls');

  LoopValueControls = require('./loop-value-controls');

  ListValueControls = require('./list-value-controls');

  ErrorMessage = require('./error-message');

  _ref = require('imjs'), Query = _ref.Query, Model = _ref.Model;

  REFERENCE_OPS = Query.REFERENCE_OPS, LIST_OPS = Query.LIST_OPS, MULTIVALUE_OPS = Query.MULTIVALUE_OPS, NULL_OPS = Query.NULL_OPS, ATTRIBUTE_OPS = Query.ATTRIBUTE_OPS, ATTRIBUTE_VALUE_OPS = Query.ATTRIBUTE_VALUE_OPS;

  NUMERIC_TYPES = Model.NUMERIC_TYPES, BOOLEAN_TYPES = Model.BOOLEAN_TYPES;

  BASIC_OPS = ATTRIBUTE_VALUE_OPS.concat(NULL_OPS);

  html = "<fieldset class=\"im-constraint-options form-group row\">\n  <label class=\"col-sm-3 path-label\"><%- data.con.displayName %></label>\n  <% if (data.con.op) { %>\n    <div class=\"im-operator col-sm-3\">\n        <select class=\"form-control im-ops\">\n        <option selected><%- data.con.op %></option>\n        <% data.otherOperators.forEach(function (op) { %>\n            <option><%- op %></option>\n        <% }); %>\n        </select>\n    </div>\n  <% } %>\n  <div class=\"col-sm-6 im-value-options im-value-section\">\n  </div>\n</fieldset>\n\n<div class=\"btn-group im-con-buttons\">\n  <% data.buttons.forEach(function (b, i) { %>\n  <button class=\"btn btn-<%= (i === 0) ? 'primary' : 'default' %> <%- b.classes %>\">\n    <%- data.messages.getText(b.key) %>\n  </button>\n  <% }); %>\n</div>\n";

  TEMPLATE = _.template(html, {
    variable: 'data'
  });

  NO_OP = function() {};

  operatorsFor = function(opts) {
    var path, _ref1;
    path = opts.path;
    if (!(path != null ? path.isReference : void 0)) {
      throw new Error('No path or wrong type');
    }
    if (path.isRoot()) {
      REFERENCE_OPS.concat(Query.RANGE_OPS);
    }
    if (path.isReference()) {
      return REFERENCE_OPS.concat(Query.RANGE_OPS).concat(opts["new"] ? ['ISA'] : []);
    } else if (_ref1 = path.getType(), __indexOf.call(BOOLEAN_TYPES, _ref1) >= 0) {
      return ["=", "!="].concat(NULL_OPS);
    } else {
      return ATTRIBUTE_OPS;
    }
  };

  module.exports = ConstraintEditor = (function(_super) {
    __extends(ConstraintEditor, _super);

    function ConstraintEditor() {
      return ConstraintEditor.__super__.constructor.apply(this, arguments);
    }

    ConstraintEditor.prototype.tagName = 'div';

    ConstraintEditor.prototype.className = 'form';

    ConstraintEditor.prototype.parameters = ['query', 'model'];

    ConstraintEditor.prototype.optionalParameters = ['buttonDelegate'];

    ConstraintEditor.prototype.initialize = function() {
      ConstraintEditor.__super__.initialize.apply(this, arguments);
      return this.path = this.model.get('path');
    };

    ConstraintEditor.prototype.invariants = function() {
      return {
        modelHasPath: 'No path found on model'
      };
    };

    ConstraintEditor.prototype.modelHasPath = function() {
      return this.model.get('path');
    };

    ConstraintEditor.prototype.getType = function() {
      return this.model.get('path').getType();
    };

    ConstraintEditor.prototype.modelEvents = function() {
      return {
        'change:op change:displayName': this.reRender,
        'destroy': this.stopListening
      };
    };

    ConstraintEditor.prototype.events = function() {
      return {
        'submit': function(e) {
          e.preventDefault();
          return e.stopPropagation();
        },
        'click .btn-cancel': 'cancelEditing',
        'click .btn-primary': 'applyChanges',
        'change .im-ops': 'setOperator'
      };
    };

    ConstraintEditor.prototype.delegateButtonEvents = function() {
      this.buttonDelegate.off('click.constraint-editor');
      this.buttonDelegate.on('click.constraint-editor', '.btn-cancel', (function(_this) {
        return function() {
          return _this.cancelEditing();
        };
      })(this));
      return this.buttonDelegate.on('click.constraint-editor', '.btn-primary', (function(_this) {
        return function() {
          return _this.applyChanges();
        };
      })(this));
    };

    ConstraintEditor.prototype.setOperator = function() {
      return this.model.set({
        op: this.$('.im-ops').val()
      });
    };

    ConstraintEditor.prototype.cancelEditing = function() {
      return this.model.trigger('cancel');
    };

    ConstraintEditor.prototype.applyChanges = function(e) {
      if (e != null) {
        e.preventDefault();
      }
      if (e != null) {
        e.stopPropagation();
      }
      return this.model.trigger('apply', this.getConstraint());
    };

    ConstraintEditor.prototype.getConstraint = function() {
      var con, _ref1, _ref2;
      con = this.model.pick(['path', 'op', 'value', 'values', 'code', 'type']);
      if (_ref1 = con.op, __indexOf.call(MULTIVALUE_OPS.concat(NULL_OPS), _ref1) >= 0) {
        delete con.value;
      }
      if (_ref2 = con.op, __indexOf.call(ATTRIBUTE_VALUE_OPS.concat(NULL_OPS), _ref2) >= 0) {
        delete con.values;
      }
      if (con.op === 'ISA') {
        delete con.op;
      }
      return con;
    };

    ConstraintEditor.prototype.getValueControls = function() {
      if (this.path == null) {
        return null;
      }
      if (this.isNullConstraint()) {
        return null;
      }
      if (this.isTypeConstraint()) {
        return new TypeValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.isMultiValueConstraint()) {
        return new MultiValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.isListConstraint()) {
        return new ListValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.isBooleanConstraint()) {
        return new BooleanValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.isLoopConstraint()) {
        return new LoopValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.isLookupConstraint()) {
        return new LookupValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.isRangeConstraint()) {
        return new MultiValueControls({
          model: this.model,
          query: this.query
        });
      }
      if (this.path.isAttribute()) {
        return new AttributeValueControls({
          model: this.model,
          query: this.query
        });
      }
      this.model.set({
        error: new Error('cannot handle this constaint type')
      });
      return null;
    };

    ConstraintEditor.prototype.isNullConstraint = function() {
      var _ref1;
      return _ref1 = this.model.get('op'), __indexOf.call(NULL_OPS, _ref1) >= 0;
    };

    ConstraintEditor.prototype.isLoopConstraint = function() {
      var _ref1;
      return this.path.isClass() && ((_ref1 = this.model.get('op')) === '=' || _ref1 === '!=');
    };

    ConstraintEditor.prototype.isTypeConstraint = function() {
      return this.path.isReference() && (!this.model.get('op') || 'ISA' === this.model.get('op'));
    };

    ConstraintEditor.prototype.isBooleanConstraint = function() {
      var _ref1, _ref2;
      return (_ref1 = this.path.getType(), __indexOf.call(BOOLEAN_TYPES, _ref1) >= 0) && !(_ref2 = this.model.get('op'), __indexOf.call(NULL_OPS, _ref2) >= 0);
    };

    ConstraintEditor.prototype.isMultiValueConstraint = function() {
      var _ref1;
      return this.path.isAttribute() && (_ref1 = this.model.get('op'), __indexOf.call(MULTIVALUE_OPS, _ref1) >= 0);
    };

    ConstraintEditor.prototype.isListConstraint = function() {
      var _ref1;
      return this.path.isClass() && (_ref1 = this.model.get('op'), __indexOf.call(LIST_OPS, _ref1) >= 0);
    };

    ConstraintEditor.prototype.isLookupConstraint = function() {
      var _ref1;
      return this.path.isClass() && (_ref1 = this.model.get('op'), __indexOf.call(Query.TERNARY_OPS, _ref1) >= 0);
    };

    ConstraintEditor.prototype.isRangeConstraint = function() {
      var _ref1;
      return this.path.isReference() && (_ref1 = this.model.get('op'), __indexOf.call(Query.RANGE_OPS, _ref1) >= 0);
    };

    ConstraintEditor.prototype.getOtherOperators = function() {
      return _.without(operatorsFor(this.model.pick('new', 'path')), this.model.get('op'));
    };

    ConstraintEditor.prototype.buttons = function() {
      var buttons;
      if (this.buttonDelegate != null) {
        return [];
      }
      buttons = [
        {
          key: "conbuilder.Update",
          classes: "im-update"
        }, {
          key: "conbuilder.Cancel",
          classes: "btn-cancel"
        }
      ];
      if (this.model.get('new')) {
        buttons[0].key = 'conbuilder.Add';
        buttons[1].classes = "im-remove-constraint";
      } else {
        buttons.push({
          key: "conbuilder.Remove",
          classes: "btn btn-default im-remove-constraint"
        });
      }
      return buttons;
    };

    ConstraintEditor.prototype.getData = function() {
      var buttons, con, icons, messages, otherOperators;
      buttons = this.buttons();
      messages = Messages;
      icons = Icons;
      otherOperators = this.getOtherOperators();
      con = this.model.toJSON();
      return {
        buttons: buttons,
        icons: icons,
        messages: messages,
        icons: icons,
        otherOperators: otherOperators,
        con: con
      };
    };

    ConstraintEditor.prototype.template = TEMPLATE;

    ConstraintEditor.prototype.render = function() {
      ConstraintEditor.__super__.render.apply(this, arguments);
      this.renderChild('valuecontrols', this.getValueControls(), this.$('.im-value-options'));
      this.renderChild('error', new ErrorMessage({
        model: this.model
      }));
      if (this.buttonDelegate != null) {
        this.delegateButtonEvents();
      }
      return this;
    };

    ConstraintEditor.prototype.remove = function() {
      var _ref1;
      if ((_ref1 = this.buttonDelegate) != null) {
        _ref1.off('click.constraint-editor');
      }
      return this.$el.slideUp({
        always: (function(_this) {
          return function() {
            return ConstraintEditor.__super__.remove.call(_this);
          };
        })(this)
      });
    };

    return ConstraintEditor;

  })(CoreView);

}).call(this);
