(function() {
  var $, ColumnControls, CompressionControls, ConstraintAdder, DestinationOptions, ExportDialogue, ExportModel, FOOTER, FastaOptions, FlatFileOptions, FormatControls, Formats, INITIAL_STATE, JSONOptions, Menu, Messages, Modal, Model, Options, Preview, RowControls, RunsQuery, Templates, UndismissableError, isa, openWindowWithPost, sendToDropBox, sendToGalaxy, sendToGenomeSpace, sendToGoogleDrive, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  $ = require('jquery');

  Model = require('../core-model');

  Modal = require('./modal');

  ConstraintAdder = require('./constraint-adder');

  Messages = require('../messages');

  Templates = require('../templates');

  Options = require('../options');

  Formats = require('../models/export-formats');

  RunsQuery = require('../mixins/runs-query');

  Menu = require('./export-dialogue/tab-menu');

  FormatControls = require('./export-dialogue/format-controls');

  RowControls = require('./export-dialogue/row-controls');

  ColumnControls = require('./export-dialogue/column-controls');

  CompressionControls = require('./export-dialogue/compression-controls');

  FlatFileOptions = require('./export-dialogue/flat-file-options');

  JSONOptions = require('./export-dialogue/json-options');

  FastaOptions = require('./export-dialogue/fasta-options');

  DestinationOptions = require('./export-dialogue/destination-options');

  Preview = require('./export-dialogue/preview');

  openWindowWithPost = require('../utils/open-window-with-post');

  sendToDropBox = require('../utils/send-to-dropbox');

  sendToGoogleDrive = require('../utils/send-to-google-drive');

  sendToGalaxy = require('../utils/send-to-galaxy');

  sendToGenomeSpace = require('../utils/send-to-genomespace');

  INITIAL_STATE = {
    doneness: null,
    tab: 'dest',
    dest: 'download',
    linkToFile: null
  };

  FOOTER = ['progress_bar', 'modal_error', 'export_dialogue_footer'];

  UndismissableError = (function() {
    UndismissableError.prototype.cannotDismiss = true;

    function UndismissableError(key) {
      this.key = 'export.error.' + key;
      this.message = Messages.get(this.key);
    }

    return UndismissableError;

  })();

  ExportModel = (function(_super) {
    __extends(ExportModel, _super);

    function ExportModel() {
      return ExportModel.__super__.constructor.apply(this, arguments);
    }

    ExportModel.prototype.defaults = function() {
      return {
        filename: 'results',
        format: Formats.getFormat('tab'),
        tablePage: null,
        start: 0,
        columns: [],
        size: null,
        max: null,
        compress: false,
        compression: 'gzip',
        headers: false,
        jsonFormat: 'rows',
        fastaExtension: null,
        headerType: 'friendly'
      };
    };

    return ExportModel;

  })(Model);

  isa = function(target) {
    return function(path) {
      return path.isa(target);
    };
  };

  module.exports = ExportDialogue = (function(_super) {
    __extends(ExportDialogue, _super);

    function ExportDialogue() {
      this.onUploadError = __bind(this.onUploadError, this);
      this.onUploadProgress = __bind(this.onUploadProgress, this);
      this.onUploadComplete = __bind(this.onUploadComplete, this);
      return ExportDialogue.__super__.constructor.apply(this, arguments);
    }

    ExportDialogue.include(RunsQuery);

    ExportDialogue.prototype.Model = ExportModel;

    ExportDialogue.prototype.className = function() {
      return 'im-export-dialogue ' + ExportDialogue.__super__.className.apply(this, arguments);
    };

    ExportDialogue.prototype.parameters = ['query'];

    ExportDialogue.prototype.initialize = function() {
      ExportDialogue.__super__.initialize.apply(this, arguments);
      if ((this.model.has('format')) && !(this.model.get('format').ext)) {
        this.model.set({
          format: Formats.getFormat(this.model.get('format'))
        });
      }
      this.state.set(INITIAL_STATE);
      this.listenTo(this.state, 'change:tab', this.renderMain);
      this.listenTo(this.model, 'change', this.updateState);
      this.listenTo(this.model, 'change:columns', this.setMax);
      this.listenTo(this.model, 'change:format', this.onChangeFormat);
      this.categoriseQuery();
      this.model.set({
        columns: this.query.views
      });
      if (this.query.name != null) {
        this.model.set({
          filename: this.query.name.replace(/\s+/g, '_')
        });
      }
      this.updateState();
      this.setMax();
      return this.readUserPreferences();
    };

    ExportDialogue.prototype.onChangeFormat = function() {
      return _.defer((function(_this) {
        return function() {
          var activeCols, cs, format, maxCols, newColumns, nodecolumns, oldColumns, p, v, _i, _len, _ref, _ref1;
          format = _this.model.get('format');
          activeCols = _this.model.get('columns');
          if ((_ref = format.needs) != null ? _ref.length : void 0) {
            oldColumns = activeCols.slice();
            newColumns = [];
            _ref1 = _this.query.views;
            for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
              v = _ref1[_i];
              p = _this.query.makePath(v).getParent();
              if (_.any(format.needs, function(needed) {
                return p.isa(needed);
              })) {
                newColumns.push(p.append('id').toString());
              }
            }
            nodecolumns = _.uniq(newColumns);
            _this.model.set({
              nodecolumns: nodecolumns
            });
            maxCols = format.maxColumns;
            cs = maxCols ? _.first(nodecolumns, maxCols) : nodecolumns.slice();
            _this.model.set({
              columns: cs
            });
            return _this.model.once('change:format', function() {
              _this.model.set({
                columns: oldColumns
              });
              return _this.model.unset('nodecolumns');
            });
          }
        };
      })(this));
    };

    ExportDialogue.prototype.readUserPreferences = function() {
      return this.query.service.whoami().then((function(_this) {
        return function(user) {
          var myGalaxy;
          if (!user.hasPreferences) {
            return;
          }
          if ((myGalaxy = user.preferences['galaxy-url'])) {
            return Options.set('Destination.Galaxy.Current', myGalaxy);
          }
        };
      })(this));
    };

    ExportDialogue.prototype.setMax = function() {
      return this.getEstimatedSize().then((function(_this) {
        return function(c) {
          return _this.model.set({
            max: c
          });
        };
      })(this));
    };

    ExportDialogue.prototype.categoriseQuery = function() {
      var has, table, type, viewNodes, _ref;
      viewNodes = this.query.getViewNodes();
      has = {};
      _ref = this.query.model.classes;
      for (type in _ref) {
        table = _ref[type];
        has[type] = _.any(viewNodes, isa(type));
      }
      return this.model.set({
        has: has
      });
    };

    ExportDialogue.prototype.title = function() {
      return Messages.getText('ExportTitle', {
        name: this.query.name
      });
    };

    ExportDialogue.prototype.modalSize = 'lg';

    ExportDialogue.prototype.primaryAction = function() {
      return Messages.getText(this.state.get('dest'));
    };

    ExportDialogue.prototype.primaryIcon = function() {
      return this.state.get('dest');
    };

    ExportDialogue.prototype.body = Templates.template('export_dialogue');

    ExportDialogue.prototype.footer = Templates.templateFromParts(FOOTER);

    ExportDialogue.prototype.updateState = function() {
      var columnDesc, columns, compress, compression, error, format, max, rowCount, size, start, _ref;
      _ref = this.model.toJSON(), compress = _ref.compress, compression = _ref.compression, start = _ref.start, size = _ref.size, max = _ref.max, format = _ref.format, columns = _ref.columns;
      columnDesc = _.isEqual(columns, this.query.views) ? Messages.get('All') : columns.length;
      rowCount = this.getRowCount();
      error = columns.length === 0 ? new UndismissableError('NoColumnsSelected') : start >= max ? new UndismissableError('OffsetOutOfBounds') : null;
      this.state.set(this.model.pick('headers', 'headerType', 'jsonFormat', 'fastaExtension'));
      return this.state.set({
        error: error,
        format: format,
        max: max,
        exportURI: this.getExportURI(),
        rowCount: this.getRowCount(),
        compression: (compress ? compression : null),
        columns: columnDesc
      });
    };

    ExportDialogue.prototype.getRowCount = function() {
      var max, size, start, _ref;
      _ref = this.model.pick('start', 'size', 'max'), start = _ref.start, size = _ref.size, max = _ref.max;
      if (start == null) {
        start = 0;
      }
      max -= start != null ? start : 0;
      size = ((size != null) && size > 0 ? size : max);
      return Math.min(max, size);
    };

    ExportDialogue.prototype.getMain = function() {
      switch (this.state.get('tab')) {
        case 'format':
          return FormatControls;
        case 'columns':
          return ColumnControls;
        case 'compression':
          return CompressionControls;
        case 'column-headers':
          return FlatFileOptions;
        case 'opts-json':
          return JSONOptions;
        case 'opts-fasta':
          return FastaOptions;
        case 'dest':
          return DestinationOptions;
        case 'rows':
          return RowControls;
        case 'preview':
          return Preview;
        default:
          return FormatControls;
      }
    };

    ExportDialogue.prototype.onUploadComplete = function(link) {
      return this.state.set({
        doneness: null,
        linkToFile: link
      });
    };

    ExportDialogue.prototype.onUploadProgress = function(doneness) {
      return this.state.set({
        doneness: doneness
      });
    };

    ExportDialogue.prototype.onUploadError = function(err) {
      this.state.set({
        doneness: null,
        error: err
      });
      return console.error(err);
    };

    ExportDialogue.prototype.act = function() {
      var ctx, exporter, exporting, file, model, onProgress, postExport, query, state, uri;
      this.onUploadProgress(0);
      exporter = this.getExporter();
      model = this.model, state = this.state, query = this.query;
      ctx = {
        model: model.toJSON(),
        state: state.toJSON(),
        query: query.clone()
      };
      uri = this.getExportURI();
      file = this.getFileName();
      onProgress = this.onUploadProgress;
      exporting = exporter.call(ctx, uri, file, onProgress);
      exporting.then(this.onUploadComplete, this.onUploadError);
      if (exporter.after != null) {
        postExport = exporter.after.bind(ctx);
        return exporting.then(postExport, postExport).then(null, (function(_this) {
          return function(e) {
            return _this.state.set({
              error: e
            });
          };
        })(this));
      }
    };

    ExportDialogue.prototype.getExporter = function() {
      switch (this.state.get('dest')) {
        case 'download':
          return function() {
            return Promise.resolve(null);
          };
        case 'Dropbox':
          return sendToDropBox;
        case 'Drive':
          return sendToGoogleDrive;
        case 'Galaxy':
          return sendToGalaxy;
        case 'GenomeSpace':
          return sendToGenomeSpace;
        default:
          throw new Error("Cannot export to " + (this.state.get('dest')));
      }
    };

    ExportDialogue.prototype.events = function() {
      var evts;
      evts = ExportDialogue.__super__.events.apply(this, arguments);
      evts.keyup = 'handleKeyup';
      return evts;
    };

    ExportDialogue.prototype.handleKeyup = function(e) {
      var _ref, _ref1;
      if (this.$(e.target).is('input')) {
        return;
      }
      switch (e.which) {
        case 40:
          return (_ref = this.children.menu) != null ? _ref.next() : void 0;
        case 38:
          return (_ref1 = this.children.menu) != null ? _ref1.prev() : void 0;
      }
    };

    ExportDialogue.prototype.renderMain = function() {
      var Main;
      Main = this.getMain();
      return this.renderChild('main', new Main({
        model: this.model,
        query: this.query,
        state: this.state
      }), this.$('div.main'));
    };

    ExportDialogue.prototype.postRender = function() {
      this.renderChild('menu', new Menu({
        model: this.state
      }), this.$('nav.menu'));
      this.renderMain();
      this.$el.focus();
      return ExportDialogue.__super__.postRender.apply(this, arguments);
    };

    return ExportDialogue;

  })(Modal);

}).call(this);
