(function() {
  var CoreView, DestinationOptions, DestinationSubOptions, Formats, Options, RadioButtons, Templates, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Options = require('../../options');

  Formats = require('../../models/export-formats');

  DestinationSubOptions = (function(_super) {
    __extends(DestinationSubOptions, _super);

    function DestinationSubOptions() {
      return DestinationSubOptions.__super__.constructor.apply(this, arguments);
    }

    DestinationSubOptions.prototype.RERENDER_EVENT = 'change:dest';

    DestinationSubOptions.prototype.initialize = function() {
      DestinationSubOptions.__super__.initialize.apply(this, arguments);
      return this.listenTo(Options, 'change:Destination.Galaxy.*', this.reRender);
    };

    DestinationSubOptions.prototype.getData = function() {
      return _.extend(DestinationSubOptions.__super__.getData.apply(this, arguments), {
        Galaxy: Options.get('Destination.Galaxy')
      });
    };

    DestinationSubOptions.prototype.getTemplate = function() {
      switch (this.model.get('dest')) {
        case 'Galaxy':
          return this.galaxyTemplate;
        default:
          return function() {
            return '';
          };
      }
    };

    DestinationSubOptions.prototype.galaxyTemplate = Templates.template('export_destination_galaxy_options');

    DestinationSubOptions.prototype.template = function(data) {
      return this.getTemplate()(data);
    };

    DestinationSubOptions.prototype.events = function() {
      return {
        'change .im-galaxy-uri-param': 'setGalaxyUri',
        'change .im-save-galaxy input': 'toggleSaveGalaxy'
      };
    };

    DestinationSubOptions.prototype.setGalaxyUri = function(_arg) {
      var target;
      target = _arg.target;
      return Options.set('Destination.Galaxy.Current', target.value);
    };

    DestinationSubOptions.prototype.toggleSaveGalaxy = function() {
      var current, key;
      key = 'Destination.Galaxy.Save';
      current = Options.get(key);
      return Options.set(key, !current);
    };

    return DestinationSubOptions;

  })(CoreView);

  RadioButtons = (function(_super) {
    __extends(RadioButtons, _super);

    function RadioButtons() {
      return RadioButtons.__super__.constructor.apply(this, arguments);
    }

    RadioButtons.prototype.RERENDER_EVENT = 'change:dest';

    RadioButtons.prototype.destinations = function() {
      var d, _i, _len, _ref, _results;
      _ref = Options.get('Destinations');
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        d = _ref[_i];
        if (Options.get(['Destination', d, 'Enabled'])) {
          _results.push(d);
        }
      }
      return _results;
    };

    RadioButtons.prototype.getData = function() {
      return _.extend({
        destinations: this.destinations()
      }, RadioButtons.__super__.getData.apply(this, arguments));
    };

    RadioButtons.prototype.template = Templates.template('export_destination_radios');

    RadioButtons.prototype.setDest = function(d) {
      return (function(_this) {
        return function() {
          return _this.model.set({
            dest: d
          });
        };
      })(this);
    };

    RadioButtons.prototype.events = function() {
      var d;
      return _.object((function() {
        var _i, _len, _ref, _results;
        _ref = this.destinations();
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          d = _ref[_i];
          _results.push(["click .im-dest-" + d, this.setDest(d)]);
        }
        return _results;
      }).call(this));
    };

    return RadioButtons;

  })(CoreView);

  module.exports = DestinationOptions = (function(_super) {
    __extends(DestinationOptions, _super);

    function DestinationOptions() {
      return DestinationOptions.__super__.constructor.apply(this, arguments);
    }

    DestinationOptions.prototype.RERENDER_EVENT = 'change:format';

    DestinationOptions.prototype.getData = function() {
      var formats, types;
      types = this.model.get('has');
      formats = Formats.getFormats(types);
      return _.extend({
        formats: formats
      }, DestinationOptions.__super__.getData.apply(this, arguments));
    };

    DestinationOptions.prototype.template = Templates.template('export_destination_options');

    DestinationOptions.prototype.postRender = function() {
      this.renderChildAt('.im-param-dest', new RadioButtons({
        model: this.state
      }));
      return this.renderChildAt('.im-dest-opts', new DestinationSubOptions({
        model: this.state
      }));
    };

    DestinationOptions.prototype.events = function() {
      var evts, fmt, types, _i, _len, _ref;
      evts = {
        '.change .im-param-name input': 'setName'
      };
      types = this.model.get('has');
      _ref = Formats.getFormats(types);
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        fmt = _ref[_i];
        evts["click .im-fmt-" + fmt.id] = this.setFormat.bind(this, fmt);
      }
      return evts;
    };

    DestinationOptions.prototype.setName = function(e) {
      return this.model.set({
        filename: e.target.value
      });
    };

    DestinationOptions.prototype.setFormat = function(format) {
      return this.model.set({
        format: format
      });
    };

    return DestinationOptions;

  })(CoreView);

}).call(this);
