(function() {
  var Options, TABS, Tab, TabMenu, Templates, View, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  View = require('../../core-view');

  Options = require('../../options');

  Templates = require('../../templates');

  Tab = (function() {
    function Tab(ident, key, formats, groups) {
      this.ident = ident;
      this.formats = formats != null ? formats : [];
      this.groups = groups != null ? groups : null;
      this.key = "export.category." + key;
    }

    Tab.prototype.isFor = function(format) {
      var _ref;
      if (this.formats.length) {
        return (_ref = format.ext, __indexOf.call(this.formats, _ref) >= 0);
      }
      if (this.groups != null) {
        return this.groups[format.group];
      }
      return true;
    };

    return Tab;

  })();

  TABS = [
    new Tab('dest', 'Destination'), new Tab('opts-json', 'JsonFormat', ['json']), new Tab('opts-fasta', 'FastaFormat', ['fasta']), new Tab('columns', 'Columns'), new Tab('rows', 'Rows', [], {
      flat: true,
      machine: true
    }), new Tab('compression', 'Compression'), new Tab('column-headers', 'ColumnHeaders', ['tsv', 'csv']), new Tab('preview', 'Preview')
  ];

  module.exports = TabMenu = (function(_super) {
    __extends(TabMenu, _super);

    function TabMenu() {
      return TabMenu.__super__.constructor.apply(this, arguments);
    }

    TabMenu.prototype.tagName = 'ul';

    TabMenu.prototype.RERENDER_EVENT = 'change';

    TabMenu.prototype.className = "nav nav-pills nav-stacked im-export-tab-menu";

    TabMenu.prototype.template = Templates.template('export_tab_menu', {
      variable: 'data'
    });

    TabMenu.prototype.getTabs = function() {
      var tab, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = TABS.length; _i < _len; _i++) {
        tab = TABS[_i];
        if (tab.isFor(this.model.get('format'))) {
          _results.push(tab);
        }
      }
      return _results;
    };

    TabMenu.prototype.getData = function() {
      var tabs;
      tabs = this.getTabs();
      return _.extend({
        tabs: tabs
      }, TabMenu.__super__.getData.apply(this, arguments));
    };

    TabMenu.prototype.setTab = function(tab) {
      return (function(_this) {
        return function() {
          if (!_this.state.get('pinned')) {
            return _this.model.set({
              tab: tab
            });
          }
        };
      })(this);
    };

    TabMenu.prototype.setPinned = function(tab) {
      return (function(_this) {
        return function() {
          if (_this.state.get('pinned') === tab) {
            _this.state.set({
              pinned: false
            });
          } else {
            _this.state.set({
              pinned: tab
            });
          }
          return _this.model.set({
            tab: tab
          });
        };
      })(this);
    };

    TabMenu.prototype.next = function() {
      var current, next, t, tabs;
      tabs = (function() {
        var _i, _len, _ref, _results;
        _ref = this.getTabs();
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          t = _ref[_i];
          _results.push(t.ident);
        }
        return _results;
      }).call(this);
      current = _.indexOf(tabs, this.model.get('tab'));
      next = current + 1;
      if (next === tabs.length) {
        next = 0;
      }
      return this.model.set({
        tab: tabs[next]
      });
    };

    TabMenu.prototype.prev = function() {
      var current, prev, t, tabs;
      tabs = (function() {
        var _i, _len, _ref, _results;
        _ref = this.getTabs();
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          t = _ref[_i];
          _results.push(t.ident);
        }
        return _results;
      }).call(this);
      current = _.indexOf(tabs, this.model.get('tab'));
      prev = current === 0 ? tabs.length - 1 : current - 1;
      return this.model.set({
        tab: tabs[prev]
      });
    };

    TabMenu.prototype.events = function() {
      var events, evt, ident;
      evt = Options.get('Events.ActivateTab');
      events = _.object((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = TABS.length; _i < _len; _i++) {
          ident = TABS[_i].ident;
          _results.push(["" + evt + " .im-tab-" + ident, this.setTab(ident)]);
        }
        return _results;
      }).call(this));
      if (evt === 'mouseenter') {
        _.extend(events, _.object((function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = TABS.length; _i < _len; _i++) {
            ident = TABS[_i].ident;
            _results.push(["click .im-tab-" + ident, this.setPinned(ident)]);
          }
          return _results;
        }).call(this)));
      }
      return events;
    };

    return TabMenu;

  })(View);

}).call(this);
