(function() {
  var CoreView, Event, FacetItems, FacetView, FacetVisualisation, NumericRange, Options, SelectedCount, SetsPathNames, SummaryHeading, SummaryItems,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Event = require('../../event');

  CoreView = require('../../core-view');

  Options = require('../../options');

  SetsPathNames = require('../../mixins/sets-path-names');

  SummaryItems = require('../../models/summary-items');

  NumericRange = require('../../models/numeric-range');

  SummaryHeading = require('./summary-heading');

  FacetItems = require('./items');

  SelectedCount = require('./selected-count');

  FacetVisualisation = require('./visualisation');

  module.exports = FacetView = (function(_super) {
    __extends(FacetView, _super);

    function FacetView() {
      this.Model = __bind(this.Model, this);
      return FacetView.__super__.constructor.apply(this, arguments);
    }

    FacetView.include(SetsPathNames);

    FacetView.prototype.className = function() {
      return 'im-facet-view';
    };

    FacetView.prototype.modelEvents = function() {
      return {
        'change:min change:max': this.setLimits
      };
    };

    FacetView.prototype.stateEvents = function() {
      return {
        'change:open': this.honourOpenness
      };
    };

    FacetView.prototype.parameters = ['query', 'view'];

    FacetView.prototype.optionalParameters = ['noTitle'];

    FacetView.prototype.Model = function() {
      return new SummaryItems({
        query: this.query,
        view: this.view
      });
    };

    FacetView.prototype.initialize = function() {
      FacetView.__super__.initialize.apply(this, arguments);
      this.range = new NumericRange;
      this.setPathNames();
      return this.setLimits();
    };

    FacetView.prototype.initState = function() {
      if (!this.state.has('open')) {
        return this.state.set({
          open: Options.get('Facets.Initally.Open')
        });
      }
    };

    FacetView.prototype.setLimits = function() {
      if (this.model.get('numeric')) {
        return this.range.setLimits(this.model.pick('min', 'max'));
      }
    };

    FacetView.prototype.invariants = function() {
      return {
        hasQuery: "No query",
        hasAttrView: "The view is not an attribute: " + this.view,
        modelIsSummaryItemsModel: "The model is not a summary items model: " + this.model
      };
    };

    FacetView.prototype.modelIsSummaryItemsModel = function() {
      return this.model instanceof SummaryItems;
    };

    FacetView.prototype.hasQuery = function() {
      return this.query != null;
    };

    FacetView.prototype.hasAttrView = function() {
      var _ref;
      return (_ref = this.view) != null ? typeof _ref.isAttribute === "function" ? _ref.isAttribute() : void 0 : void 0;
    };

    FacetView.prototype.postRender = function() {
      this.renderTitle();
      this.renderVisualisation();
      this.renderSelectedCount();
      this.renderItems();
      return this.honourOpenness();
    };

    FacetView.prototype.renderTitle = function() {
      if (!this.noTitle) {
        return this.renderChild('title', new SummaryHeading({
          model: this.model,
          state: this.state
        }));
      }
    };

    FacetView.prototype.renderVisualisation = function() {
      return this.renderChild('viz', new FacetVisualisation({
        model: this.model,
        state: this.state,
        range: this.range
      }));
    };

    FacetView.prototype.renderSelectedCount = function() {
      return this.renderChild('count', new SelectedCount({
        model: this.model,
        range: this.range
      }));
    };

    FacetView.prototype.renderItems = function() {
      return this.renderChild('facet', new FacetItems({
        model: this.model,
        state: this.state,
        range: this.range
      }));
    };

    FacetView.prototype.honourOpenness = function() {
      var facet, isOpen, wasOpen;
      isOpen = this.state.get('open');
      wasOpen = this.state.previous('open');
      facet = this.$('dd.im-facet');
      if (isOpen) {
        facet.slideDown();
        this.trigger('opened', this);
      } else {
        facet.slideUp();
        this.trigger('closed', this);
      }
      if ((wasOpen != null) && (isOpen !== wasOpen)) {
        return this.trigger('toggled', this);
      }
    };

    FacetView.prototype.events = function() {
      return {
        'click .im-summary-heading': 'toggle'
      };
    };

    FacetView.prototype.toggle = function() {
      return this.state.toggle('open');
    };

    FacetView.prototype.close = function() {
      return this.state.set({
        open: false
      });
    };

    FacetView.prototype.open = function() {
      return this.state.set({
        open: true
      });
    };

    return FacetView;

  })(CoreView);

}).call(this);
