(function() {
  var HistoFacet, Messages, Options, VisualisationBase, bool, d3, scale, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  d3 = require('d3-browserify');

  _ = require('underscore');

  Options = require('../../options');

  Messages = require('../../messages');

  VisualisationBase = require('./visualisation-base');

  require('../../messages/summary');

  bool = require('../../utils/casts').bool;

  scale = function(input, output) {
    return d3.scale.linear().domain(input).range(output);
  };

  module.exports = HistoFacet = (function(_super) {
    __extends(HistoFacet, _super);

    function HistoFacet() {
      return HistoFacet.__super__.constructor.apply(this, arguments);
    }

    HistoFacet.prototype.className = 'im-summary-histogram';

    HistoFacet.prototype.chartHeight = 50;

    HistoFacet.prototype.leftMargin = 25;

    HistoFacet.prototype.rightMargin = 25;

    HistoFacet.prototype.bottomMargin = 0.5;

    HistoFacet.prototype.stepWidth = 0;

    HistoFacet.prototype.allCountsAreOne = function() {
      return this.model.get('maxCount') === 1;
    };

    HistoFacet.prototype.initialize = function() {
      HistoFacet.__super__.initialize.apply(this, arguments);
      return this.listenTo(this.model.items, 'add remove change:selected', (function(_this) {
        return function() {
          return _this.updateChart();
        };
      })(this));
    };

    HistoFacet.prototype.invariants = function() {
      return {
        'hasItems': "No items, or not the right thing: " + this.model.items
      };
    };

    HistoFacet.prototype.hasItems = function() {
      var _ref, _ref1;
      return ((_ref = this.model) != null ? (_ref1 = _ref.items) != null ? _ref1.models : void 0 : void 0) != null;
    };

    HistoFacet.prototype.initChart = function() {
      HistoFacet.__super__.initChart.apply(this, arguments);
      return this.stepWidth = (this.chartWidth - (this.leftMargin + this.rightMargin + 1)) / this.model.items.size();
    };

    HistoFacet.prototype.shouldNotDrawChart = function() {
      return this.allCountsAreOne();
    };

    HistoFacet.prototype.getScales = function() {
      var counts, indices, x, xPositions, y, yPositions;
      indices = [0, this.model.items.size()];
      counts = [0, this.model.get('maxCount')];
      yPositions = [0, this.chartHeight - this.bottomMargin];
      xPositions = [this.leftMargin, this.chartWidth - this.rightMargin];
      x = scale(indices, xPositions);
      y = scale(counts, yPositions);
      return {
        x: x,
        y: y
      };
    };

    HistoFacet.prototype.selectNodes = function(chart) {
      return chart.selectAll('rect');
    };

    HistoFacet.prototype.getChartData = function(scales) {
      return this.model.items.models.slice();
    };

    HistoFacet.prototype.enter = function(selection, scales) {
      return selection.append('rect').classed('im-item-bar', true).classed('squashed', this.stepWidth < 4).attr('width', this.stepWidth).attr('y', this.chartHeight).attr('height', 0).attr('x', function(_, i) {
        return scales.x(i);
      }).on('click', function(model) {
        return model.toggle('selected');
      }).on('mouseover', function(model) {
        return model.mousein();
      }).on('mouseout', function(model) {
        return model.mouseout();
      });
    };

    HistoFacet.prototype.update = function(selection, scales) {
      var Duration, Easing, h, height, _ref;
      _ref = Options.get('D3.Transition'), Duration = _ref.Duration, Easing = _ref.Easing;
      h = this.chartHeight - this.bottomMargin;
      height = function(model) {
        return scales.y(model.get('count'));
      };
      selection.classed('selected', function(model) {
        return model.get('selected');
      });
      return selection.transition().duration(Duration).ease(Easing).attr('height', height).attr('y', function(model) {
        return h - (height(model));
      });
    };

    HistoFacet.prototype.drawAxes = function(chart, scales) {
      var y;
      y = this.chartHeight - this.bottomMargin;
      return chart.append('line').classed('x-axis', true).attr('x1', 0).attr('x2', this.chartWidth).attr('y1', y).attr('y2', y);
    };

    return HistoFacet;

  })(VisualisationBase);

}).call(this);
