(function() {
  var $, Messages, NULL_SELECTION_WIDTH, NumericDistribution, Options, VisualisationBase, bucketRange, d3, limited, scale, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  d3 = require('d3-browserify');

  $ = require('jquery');

  _ = require('underscore');

  Options = require('../../options');

  Messages = require('../../messages');

  require('../../messages/summary');

  VisualisationBase = require('./visualisation-base');

  NULL_SELECTION_WIDTH = 25;

  scale = function(input, output) {
    return d3.scale.linear().domain(input).range(output);
  };

  bucketRange = function(bucketVal, bucket) {
    var delta, max, min, _ref;
    _ref = (function() {
      var _i, _len, _ref, _results;
      _ref = [0, 1];
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        delta = _ref[_i];
        _results.push(bucketVal(bucket + delta));
      }
      return _results;
    })(), min = _ref[0], max = _ref[1];
    return {
      min: min,
      max: max
    };
  };

  limited = function(min, max) {
    return function(x) {
      if (x < min) {
        return min;
      } else if (x > max) {
        return max;
      } else {
        return x;
      }
    };
  };

  module.exports = NumericDistribution = (function(_super) {
    __extends(NumericDistribution, _super);

    function NumericDistribution() {
      return NumericDistribution.__super__.constructor.apply(this, arguments);
    }

    NumericDistribution.prototype.className = "im-numeric-distribution";

    NumericDistribution.prototype.leftMargin = 25;

    NumericDistribution.prototype.bottomMargin = 18;

    NumericDistribution.prototype.rightMargin = 14;

    NumericDistribution.prototype.chartHeight = 70;

    NumericDistribution.prototype.__selecting_paths = false;

    NumericDistribution.prototype.selection = null;

    NumericDistribution.prototype.initialize = function(_arg) {
      this.range = _arg.range;
      NumericDistribution.__super__.initialize.apply(this, arguments);
      return this.listenTo(this.range, 'change reset', this.onChangeRange);
    };

    NumericDistribution.prototype.invariants = function() {
      return {
        hasRange: "No range",
        hasHistogramModel: "Wrong model: " + this.model
      };
    };

    NumericDistribution.prototype.hasRange = function() {
      return this.range != null;
    };

    NumericDistribution.prototype.hasHistogramModel = function() {
      var _ref;
      return ((_ref = this.model) != null ? _ref.getHistogram : void 0) != null;
    };

    NumericDistribution.prototype.selectNodes = function(chart) {
      return chart.selectAll('rect');
    };

    NumericDistribution.prototype.getChartData = function(scales) {
      var c, i, _i, _len, _ref, _results;
      if (scales == null) {
        scales = this.getScales();
      }
      _ref = this.model.getHistogram();
      _results = [];
      for (i = _i = 0, _len = _ref.length; _i < _len; i = ++_i) {
        c = _ref[i];
        _results.push({
          count: c,
          bucket: i + 1,
          range: bucketRange(scales.bucketToVal, i + 1)
        });
      }
      return _results;
    };

    NumericDistribution.prototype.initChart = function() {
      NumericDistribution.__super__.initChart.apply(this, arguments);
      this.bucketWidth = (this.model.get('max') - this.model.get('min')) / this.model.get('buckets');
      return this.stepWidth = (this.chartWidth - (this.leftMargin + 1)) / this.model.get('buckets');
    };

    NumericDistribution.prototype.getScales = function() {
      var buckets, counts, histogram, max, min, most, n, scales, toVal, values, xPositions, yPositions, _ref;
      _ref = this.model.pick('min', 'max'), min = _ref.min, max = _ref.max;
      n = this.model.get('buckets');
      histogram = this.model.getHistogram();
      most = d3.max(histogram);
      counts = [0, most];
      values = [min, max];
      buckets = [1, n + 1];
      xPositions = [this.leftMargin, this.chartWidth - this.rightMargin];
      yPositions = [0, this.chartHeight - this.bottomMargin];
      toVal = function(inputs) {
        return _.compose(limited(min, max), scale(inputs, values));
      };
      return scales = {
        x: scale(buckets, xPositions),
        y: scale(counts, yPositions),
        valToX: scale(values, xPositions),
        xToVal: toVal(xPositions),
        bucketToVal: toVal(buckets)
      };
    };

    NumericDistribution.prototype.isIntish = function() {
      var _ref;
      return (_ref = this.model.get('type')) === 'int' || _ref === 'Integer' || _ref === 'long' || _ref === 'Long' || _ref === 'short' || _ref === 'Short';
    };

    NumericDistribution.prototype.getRounder = function() {
      if (this.isIntish()) {
        return Math.round;
      } else {
        return _.identity;
      }
    };

    NumericDistribution.prototype.enter = function(selection, scales) {
      var barClickHandler, container, getTitle, h, round;
      if (scales == null) {
        scales = this.getScales();
      }
      container = this.el;
      round = this.getRounder();
      h = this.chartHeight;
      barClickHandler = (function(_this) {
        return function(d, i) {
          if (d.count > 0) {
            return _this.range.set(d.range);
          } else {
            return _this.range.nullify();
          }
        };
      })(this);
      getTitle = function(_arg) {
        var count, max, min, _ref;
        (_ref = _arg.range, min = _ref.min, max = _ref.max), count = _arg.count;
        return Messages.getText('summary.Bucket', {
          count: count,
          range: {
            min: round(min),
            max: round(max)
          }
        });
      };
      return selection.append('rect').attr('x', function(d, i) {
        return scales.x(d.bucket);
      }).attr('width', function(d) {
        return Math.max(0, (scales.x(d.bucket + 1)) - (scales.x(d.bucket)));
      }).attr('y', h - this.bottomMargin).attr('height', 0).classed('im-bucket', true).classed('im-null-bucket', function(d) {
        return d.bucket === null;
      }).on('click', barClickHandler).each(function(d) {
        return $(this).tooltip({
          container: container,
          title: getTitle(d)
        });
      });
    };

    NumericDistribution.prototype.update = function(selection, scales) {
      var Duration, Easing, bm, h, height, _ref;
      if (scales == null) {
        scales = this.getScales();
      }
      h = this.chartHeight;
      bm = this.bottomMargin;
      _ref = Options.get('D3.Transition'), Duration = _ref.Duration, Easing = _ref.Easing;
      height = function(d) {
        return scales.y(d.count);
      };
      return selection.transition().duration(Duration).ease(Easing).attr('height', height).attr('y', function(d) {
        return h - bm - (height(d)) - 0.5;
      });
    };

    NumericDistribution.prototype.drawAxes = function(chart, scales) {
      var axis, bottom, container, ticks;
      if (chart == null) {
        chart = this.getCanvas();
      }
      if (scales == null) {
        scales = this.getScales();
      }
      bottom = this.chartHeight - this.bottomMargin - .5;
      container = this.el;
      if (Options.get('Facets.DrawAverageLine')) {
        chart.append('line').classed('average', true).attr('x1', scales.valToX(this.model.get('average'))).attr('x2', scales.valToX(this.model.get('average'))).attr('y1', 0).attr('y2', bottom).each(function() {
          return $(this).tooltip({
            container: container,
            title: Messages.getText('summary.Average')
          });
        });
      }
      chart.append('line').classed('axis', true).attr('x1', 0).attr('x2', this.chartWidth).attr('y1', bottom).attr('y2', bottom);
      axis = chart.append('svg:g');
      ticks = scales.x.ticks(this.model.get('buckets'));
      return axis.selectAll('line').data(ticks).enter().append('svg:line').classed('tick-line', true).attr('x1', scales.x).attr('x2', scales.x).attr('y1', this.chartHeight - (this.bottomMargin * 0.75)).attr('y2', this.chartHeight - this.bottomMargin);
    };

    NumericDistribution.prototype.events = function() {
      return {
        'mouseout': (function(_this) {
          return function() {
            return _this.__selecting_paths = false;
          };
        })(this)
      };
    };

    NumericDistribution.prototype.drawSelection = function(x, width) {
      if ((x == null) || (x <= 0) || (width >= this.chartWidth)) {
        return this.removeSelection();
      }
      if (this.selection == null) {
        this.selection = this.getCanvas().append('svg:rect').attr('y', 0).attr('height', this.chartHeight * 0.9).classed('rubberband-selection', true);
      }
      return this.selection.attr('x', x).attr('width', width);
    };

    NumericDistribution.prototype.onChangeRange = function() {
      var max, min, scales, start, width, _ref;
      if (this.shouldDrawBox()) {
        scales = this.getScales();
        _ref = this.range.toJSON(), min = _ref.min, max = _ref.max;
        start = scales.valToX(min);
        width = (scales.valToX(max)) - start;
        return this.drawSelection(start, width);
      } else {
        return this.removeSelection();
      }
    };

    NumericDistribution.prototype.removeSelection = function() {
      var _ref;
      if ((_ref = this.selection) != null) {
        _ref.remove();
      }
      return this.selection = null;
    };

    NumericDistribution.prototype.shouldDrawBox = function() {
      return this.range.isNotAll();
    };

    NumericDistribution.prototype.remove = function() {
      var _ref;
      this.removeSelection();
      if ((_ref = this.paper) != null) {
        _ref.remove();
      }
      return NumericDistribution.__super__.remove.apply(this, arguments);
    };

    return NumericDistribution;

  })(VisualisationBase);

}).call(this);
