(function() {
  var $, DONUT, KEY, Options, PieChart, TWEEN_START, VisualisationBase, d3, getArc, getArcTween, getChartPalette, getEndPosition, getStartPosition, isPastSixOClock, opacity, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  $ = require('jquery');

  d3 = require('d3-browserify');

  Options = require('../../options');

  VisualisationBase = require('./visualisation-base');

  KEY = function(d) {
    return d.data.get('id');
  };

  DONUT = d3.layout.pie().value(function(d) {
    return d.get('count');
  });

  TWEEN_START = {
    startAngle: 0,
    endAngle: 0
  };

  getChartPalette = function() {
    var colors, paint;
    colors = Options.get('PieColors');
    paint = _.isFunction(colors) ? colors : d3.scale[colors]();
    return function(d) {
      return paint(d.data.get('id'));
    };
  };

  getStartPosition = function(model) {
    var _ref;
    return (_ref = model.get('currentPieCoords')) != null ? _ref : TWEEN_START;
  };

  opacity = function(d) {
    if (d.data.get('visible')) {
      return 1;
    } else {
      return 0.25;
    }
  };

  getEndPosition = function(startAngle, endAngle, model) {
    return {
      startAngle: startAngle,
      endAngle: endAngle,
      selected: (model.get('selected') ? 1 : 0)
    };
  };

  getArcTween = function(arc) {
    return function(_arg) {
      var data, end, endAngle, getDatumAtTime, model, start, startAngle;
      startAngle = _arg.startAngle, endAngle = _arg.endAngle, data = _arg.data;
      model = data;
      start = getStartPosition(model);
      end = getEndPosition(startAngle, endAngle, model);
      getDatumAtTime = d3.interpolateObject(start, end);
      model.set({
        currentPieCoords: getDatumAtTime(1)
      });
      return function(t) {
        return arc(getDatumAtTime(t));
      };
    };
  };

  isPastSixOClock = function(d) {
    return (d.endAngle + d.startAngle) / 2 > Math.PI;
  };

  getArc = function(outerRadius, innerRadius, selectionBump) {
    return d3.svg.arc().startAngle(function(d) {
      return d.startAngle;
    }).endAngle(function(d) {
      return d.endAngle;
    }).innerRadius(function(d) {
      return innerRadius + (d.selected * selectionBump);
    }).outerRadius(function(d) {
      return outerRadius + (d.selected * selectionBump);
    });
  };

  module.exports = PieChart = (function(_super) {
    __extends(PieChart, _super);

    function PieChart() {
      return PieChart.__super__.constructor.apply(this, arguments);
    }

    PieChart.prototype.chartWidth = 120;

    PieChart.prototype.chartHeight = 120;

    PieChart.prototype.className = 'im-pie-chart';

    PieChart.prototype.initialize = function() {
      PieChart.__super__.initialize.apply(this, arguments);
      this.listenTo(this.model.items, 'change:selected change:visible', this.update);
      this.listenTo(Options, 'change:PieColors', this.onChangePalette);
      return this.onChangePalette();
    };

    PieChart.prototype.onChangePalette = function() {
      this.colour = getChartPalette();
      return this.update();
    };

    PieChart.prototype._drawD3Chart = function() {
      var centre_group, chart, h, innerRadius, label_group, outerRadius, selectionBump, w, whiteCircle;
      h = this.chartHeight;
      w = this.$el.closest(':visible').width();
      outerRadius = h * 0.4;
      innerRadius = h * 0.1;
      selectionBump = h * 0.08;
      chart = d3.select(this.el).append('svg').attr('class', 'chart').attr('height', h).attr('width', w);
      this.arc = getArc(outerRadius, innerRadius, selectionBump);
      this.arc_group = chart.append('svg:g').attr('class', 'arc').attr('transform', "translate(" + (w / 2) + "," + (h / 2) + ")");
      centre_group = chart.append('svg:g').attr('class', 'center_group').attr('transform', "translate(" + (w / 2) + "," + (h / 2) + ")");
      label_group = chart.append("svg:g").attr("class", "label_group").attr("transform", "translate(" + (w / 2) + "," + (h / 2) + ")");
      whiteCircle = centre_group.append("svg:circle").attr("fill", "white").attr("r", innerRadius);
      return this.update();
    };

    PieChart.prototype.enter = function(selection) {
      var activateTooltip, container, percent, total;
      container = this.el;
      total = this.model.items.reduce((function(sum, m) {
        return sum + m.get('count');
      }), 0);
      percent = function(d) {
        return (d.data.get('count') / total * 100).toFixed(1);
      };
      activateTooltip = function(d) {
        var $el, placement, title;
        $el = $(this);
        title = "" + (d.data.get('item')) + ": " + (percent(d)) + "%";
        placement = isPastSixOClock(d) ? 'left' : 'right';
        return $el.tooltip({
          title: title,
          placement: placement,
          container: container
        });
      };
      return selection.append('svg:path').attr('class', 'donut-arc').on('click', function(d) {
        return d.data.toggle('selected');
      }).on('mouseover', function(d) {
        return d.data.mousein();
      }).on('mouseout', function(d) {
        return d.data.mouseout();
      }).each(activateTooltip);
    };

    PieChart.prototype.exit = function(selection) {
      return selection.remove();
    };

    PieChart.prototype.getChartData = function() {
      return this.model.items.models.slice();
    };

    PieChart.prototype.update = function() {
      var DurationShort, Easing, paths, _ref;
      if (this.arc_group == null) {
        return;
      }
      paths = this.arc_group.selectAll('path').data(DONUT(this.getChartData()), KEY);
      this.exit(paths.exit());
      this.enter(paths.enter());
      _ref = Options.get('D3.Transition'), DurationShort = _ref.DurationShort, Easing = _ref.Easing;
      paths.attr('fill', this.colour);
      return paths.transition().duration(DurationShort).ease(Easing).style('opacity', opacity).attrTween('d', getArcTween(this.arc));
    };

    return PieChart;

  })(VisualisationBase);

}).call(this);
