(function() {
  var Checkbox, CoreView, FacetRow, RowSurrogate, Templates, bool, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Checkbox = require('../../core/checkbox');

  RowSurrogate = require('./row-surrogate');

  require('../../messages/summary');

  bool = function(x) {
    return !!x;
  };

  module.exports = FacetRow = (function(_super) {
    __extends(FacetRow, _super);

    function FacetRow() {
      return FacetRow.__super__.constructor.apply(this, arguments);
    }

    FacetRow.prototype.RERENDER_EVENT = 'change:count change:item change:symbol change:share';

    FacetRow.prototype.tagName = "tr";

    FacetRow.prototype.className = "im-facet-row";

    FacetRow.prototype.modelEvents = function() {
      return {
        "change:visible": this.onChangeVisibility,
        "change:hover": this.onChangeHover,
        "change:selected": this.onChangeSelected
      };
    };

    FacetRow.prototype.invariants = function() {
      return {
        modelHasCollection: "No collection on model"
      };
    };

    FacetRow.prototype.modelHasCollection = function() {
      var _ref;
      return ((_ref = this.model) != null ? _ref.collection : void 0) != null;
    };

    FacetRow.prototype.template = Templates.template('facet_row');

    FacetRow.prototype.getData = function() {
      var max, opacity, percent, ratio;
      max = this.model.collection.getMaxCount();
      ratio = this.model.get('count') / max;
      opacity = (ratio / 2 + 0.5).toFixed();
      percent = (ratio * 100).toFixed();
      return _.extend(FacetRow.__super__.getData.apply(this, arguments), {
        percent: percent,
        opacity: opacity,
        max: max
      });
    };

    FacetRow.prototype.onRenderError = function(e) {
      return console.error(e);
    };

    FacetRow.prototype.postRender = function() {
      this.addCheckbox();
      this.onChangeVisibility();
      this.onChangeHover();
      return this.onChangeSelected();
    };

    FacetRow.prototype.addCheckbox = function() {
      return this.renderChildAt('.checkbox', new Checkbox({
        model: this.model,
        attr: 'selected'
      }));
    };

    FacetRow.prototype.onChangeVisibility = function() {
      return this.$el.toggleClass('im-hidden', !this.model.get("visible"));
    };

    FacetRow.prototype.onChangeHover = function() {
      var isHovered;
      isHovered = bool(this.model.get('hover'));
      this.$el.toggleClass('hover', isHovered);
      if (isHovered) {
        return this.showSurrogateUnlessVisible();
      } else {
        return this.removeSurrogate();
      }
    };

    FacetRow.prototype.onChangeSelected = function() {
      return this.$el.toggleClass('im-selected', this.model.get('selected'));
    };

    FacetRow.prototype.removeSurrogate = function() {
      return this.removeChild('surrogate');
    };

    FacetRow.prototype.showSurrogateUnlessVisible = function() {
      var $s, above, newTop, surrogate, table;
      this.removeSurrogate();
      if (!this.isVisible()) {
        above = this.isAbove();
        surrogate = new RowSurrogate({
          model: this.model,
          above: above
        });
        $s = surrogate.$el;
        table = this.getTable();
        this.renderChild('surrogate', surrogate, table);
        return newTop = above ? $s.css({
          top: table.scrollTop()
        }) : $s.css({
          bottom: 0 - table.scrollTop()
        });
      }
    };

    FacetRow.prototype.getTable = function() {
      return this.$el.closest('.im-item-table');
    };

    FacetRow.prototype.events = function() {
      return {
        'click': 'handleClick'
      };
    };

    FacetRow.prototype.handleClick = function(e) {
      e.stopPropagation();
      return this.model.toggle('selected');
    };

    FacetRow.prototype.isBelow = function() {
      var parent;
      parent = this.getTable();
      return this.$el.offset().top + this.$el.outerHeight() > parent.offset().top + parent.outerHeight();
    };

    FacetRow.prototype.isAbove = function() {
      var parent;
      parent = this.getTable();
      return this.$el.offset().top < parent.offset().top;
    };

    FacetRow.prototype.isVisible = function() {
      return !(this.isAbove() || this.isBelow());
    };

    return FacetRow;

  })(CoreView);

}).call(this);
