(function() {
  var CoreView, SelectedCount, Templates, fracWithinRange, fullyContained, getPartialCount, partiallyOverlapping, sum, sumCounts, sumPartials, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  require('../../messages/summary');

  sum = function(lens, xs) {
    if (lens == null) {
      lens = _.identity;
    }
    return _.reduce(xs, (function(total, x) {
      return total + lens(x);
    }), 0);
  };

  sumCounts = _.partial(sum, function(x) {
    return x.count;
  });

  sumPartials = function(min, max, partials) {
    return sum(_.partial(getPartialCount, min, max), partials);
  };

  fracWithinRange = function(range, min, max) {
    var overlap, rangeSize;
    if (!range) {
      return 0;
    }
    rangeSize = range.max - range.min;
    overlap = range.min < min ? Math.min(range.max, max) - min : max - Math.max(range.min, min);
    return overlap / rangeSize;
  };

  fullyContained = function(min, max) {
    return function(b) {
      return b.min >= min && b.max <= max;
    };
  };

  partiallyOverlapping = function(min, max) {
    return function(b) {
      return (b.min < min && b.max > min) || (b.max > max && b.min < max);
    };
  };

  getPartialCount = function(min, max, b) {
    return b.count * fracWithinRange(b, min, max);
  };

  module.exports = SelectedCount = (function(_super) {
    __extends(SelectedCount, _super);

    function SelectedCount() {
      return SelectedCount.__super__.constructor.apply(this, arguments);
    }

    SelectedCount.prototype.className = 'im-summary-selected-count';

    SelectedCount.prototype.template = Templates.template('summary-selected-count');

    SelectedCount.prototype.stateEvents = function() {
      return {
        'change:selectedCount change:isApprox': this.reRender
      };
    };

    SelectedCount.prototype.initialize = function(_arg) {
      this.range = _arg.range;
      SelectedCount.__super__.initialize.apply(this, arguments);
      this.listenTo(this.model.items, 'change:selected', this.estimateSelectionSize);
      return this.listenTo(this.range, 'change', this.estimateSelectionSize);
    };

    SelectedCount.prototype.estimateSelectionSize = function() {
      if (!this.model.get('initialized')) {
        return;
      }
      if (this.model.get('numeric')) {
        return this.estimateSelectedInRange();
      } else {
        return this.sumSelectedItems();
      }
    };

    SelectedCount.prototype.preRender = function() {
      if (!this.rendered) {
        return this.$el.hide();
      }
      if (!this.state.get('selectedCount')) {
        return this.$el.slideUp();
      }
    };

    SelectedCount.prototype.postRender = function() {
      if (this.state.get('selectedCount')) {
        return this.$el.slideDown();
      }
    };

    SelectedCount.prototype.sumSelectedItems = function() {
      var count, selected;
      selected = this.model.items.where({
        selected: true
      });
      count = sum((function(i) {
        return i.get('count');
      }), selected);
      return this.state.set({
        isApprox: false,
        selectedCount: count
      });
    };

    SelectedCount.prototype.estimateSelectedInRange = function() {
      var count, fullBuckets, histogram, max, min, partials, _ref;
      if (this.range.isAll()) {
        return this.state.unset('selectedCount');
      }
      _ref = this.range.toJSON(), min = _ref.min, max = _ref.max;
      histogram = this.getHistogram();
      fullBuckets = histogram.filter(fullyContained(min, max));
      partials = histogram.filter(partiallyOverlapping(min, max));
      count = Math.round((sumCounts(fullBuckets)) + (sumPartials(min, max, partials)));
      return this.state.set({
        isApprox: true,
        selectedCount: count
      });
    };

    SelectedCount.prototype.getHistogram = function() {
      var buckets, c, i, maxVal, minVal, step, _i, _len, _results;
      buckets = this.model.getHistogram();
      maxVal = this.model.get('max');
      minVal = this.model.get('min');
      step = (maxVal - minVal) / buckets.length;
      _results = [];
      for (i = _i = 0, _len = buckets.length; _i < _len; i = ++_i) {
        c = buckets[i];
        _results.push({
          count: c,
          min: minVal + (i * step),
          max: minVal + step + (i * step)
        });
      }
      return _results;
    };

    return SelectedCount;

  })(CoreView);

}).call(this);
