(function() {
  var BASIC_OPS, CoreView, IGNORE, MIN_VALS_OPTIMISATION, Messages, SUMMARY_FORMATS, SummaryItemsControls, Templates, bool, negateOps, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Messages = require('../../messages');

  require('../../messages/summary');

  bool = function(x) {
    return !!x;
  };

  negateOps = function(ops) {
    var ret;
    ret = {};
    ret.multi = ops.multi === 'ONE OF' ? 'NONE OF' : 'ONE OF';
    ret.single = ops.single === '==' ? '!=' : '==';
    ret.absent = ops.absent === 'IS NULL' ? 'IS NOT NULL' : 'IS NULL';
    return ret;
  };

  IGNORE = function(e) {
    if (e != null) {
      e.preventDefault();
      return e.stopPropagation();
    }
  };

  BASIC_OPS = {
    single: '==',
    multi: 'ONE OF',
    absent: 'IS NULL'
  };

  SUMMARY_FORMATS = {
    tab: 'tsv',
    csv: 'csv',
    xml: 'xml',
    json: 'json'
  };

  MIN_VALS_OPTIMISATION = 10;

  module.exports = SummaryItemsControls = (function(_super) {
    __extends(SummaryItemsControls, _super);

    function SummaryItemsControls() {
      return SummaryItemsControls.__super__.constructor.apply(this, arguments);
    }

    SummaryItemsControls.prototype.RERENDER_EVT = 'change';

    SummaryItemsControls.prototype.initialize = function() {
      SummaryItemsControls.__super__.initialize.apply(this, arguments);
      return this.listenTo(this.model.items, 'change:selected', this.reRender);
    };

    SummaryItemsControls.prototype.invariants = function() {
      return {
        viewIsAttribute: "No view, or view not Attribute: " + this.view,
        hasCollection: "No collection"
      };
    };

    SummaryItemsControls.prototype.viewIsAttribute = function() {
      var _ref, _ref1;
      return (_ref = this.model) != null ? (_ref1 = _ref.view) != null ? typeof _ref1.isAttribute === "function" ? _ref1.isAttribute() : void 0 : void 0 : void 0;
    };

    SummaryItemsControls.prototype.hasCollection = function() {
      var _ref;
      return ((_ref = this.model) != null ? _ref.items : void 0) != null;
    };

    SummaryItemsControls.prototype.template = Templates.template('summary_items_controls');

    SummaryItemsControls.prototype.getData = function() {
      var anyItemSelected;
      anyItemSelected = bool(this.model.items.findWhere({
        selected: true
      }));
      return _.extend(SummaryItemsControls.__super__.getData.apply(this, arguments), {
        anyItemSelected: anyItemSelected
      });
    };

    SummaryItemsControls.prototype.postRender = function() {
      this.activateTooltips();
      return this.activatePopovers();
    };

    SummaryItemsControls.prototype.activateTooltips = function() {
      return this.$btns = this.$('.btn[title]').tooltip({
        placement: 'top',
        container: this.el
      });
    };

    SummaryItemsControls.prototype.activatePopovers = function() {
      return this.$('.im-download').popover({
        placement: 'top',
        html: true,
        container: this.el,
        title: Messages.getText('summary.DownloadFormat'),
        content: this.getDownloadPopover(),
        trigger: 'manual'
      });
    };

    SummaryItemsControls.prototype.downloadPopoverTemplate = Templates.template('download_popover');

    SummaryItemsControls.prototype.getDownloadPopover = function() {
      return this.downloadPopoverTemplate(_.extend(this.getBaseData(), {
        query: this.model.query,
        path: this.model.view.toString(),
        formats: SUMMARY_FORMATS
      }));
    };

    SummaryItemsControls.prototype.events = function() {
      return {
        'click .im-export-summary': 'exportSummary',
        'click': 'hideTooltips',
        'click .btn-cancel': 'unsetSelection',
        'click .btn-toggle-selection': 'toggleSelection',
        'click .im-filter-group .dropdown-toggle': 'toggleDropdown',
        'click .im-download': 'toggleDownloadPopover',
        'click .im-filter-in': _.bind(this.addConstraint, this, BASIC_OPS),
        'click .im-filter-out': _.bind(this.addConstraint, this, negateOps(BASIC_OPS))
      };
    };

    SummaryItemsControls.prototype.exportSummary = function(e) {
      e.stopImmediatePropagation();
      return true;
    };

    SummaryItemsControls.prototype.hideTooltips = function() {
      var _ref;
      return (_ref = this.$btns) != null ? _ref.tooltip('hide') : void 0;
    };

    SummaryItemsControls.prototype.unsetSelection = function(e) {
      return this.changeSelection(function(item) {
        return item.set({
          selected: false
        });
      });
    };

    SummaryItemsControls.prototype.toggleSelection = function(e) {
      return this.changeSelection(function(x) {
        if (x.get('visible')) {
          return x.toggle('selected');
        }
      });
    };

    SummaryItemsControls.prototype.toggleDropdown = function() {
      return this.$('.im-filter-group').toggleClass('open');
    };

    SummaryItemsControls.prototype.toggleDownloadPopover = function() {
      return this.$('.im-download').popover('toggle');
    };

    SummaryItemsControls.prototype.addConstraint = function(ops, e) {
      var anyIsNull, item, unselected, vals, _ref;
      IGNORE(e);
      vals = (function() {
        var _i, _len, _ref, _results;
        _ref = this.model.items.where({
          selected: true
        });
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          item = _ref[_i];
          _results.push(item.get('item'));
        }
        return _results;
      }).call(this);
      unselected = this.model.items.where({
        selected: false
      });
      if (unselected.length === 0) {
        return this.model.set({
          error: new Error('All items are selected')
        });
      }
      anyIsNull = _.any(vals, function(v) {
        return v == null;
      });
      if ((!anyIsNull) && this.model.hasAll() && ((MIN_VALS_OPTIMISATION > (_ref = vals.length) && _ref > unselected.length))) {
        return this.constrainTo(negateOps(ops), (function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = unselected.length; _i < _len; _i++) {
            item = unselected[_i];
            _results.push(item.get('item'));
          }
          return _results;
        })());
      } else {
        return this.constrainTo(ops, vals);
      }
    };

    SummaryItemsControls.prototype.constrainTo = function(ops, vals) {
      var existing, newCon, path, q, val;
      if (!(vals != null ? vals.length : void 0)) {
        return this.model.set({
          error: new Error('No values are selected')
        });
      }
      q = this.model.query;
      path = this.model.view.toString();
      val = vals[0];
      newCon = (function() {
        switch (false) {
          case !(vals.length > 1):
            return {
              op: ops.multi,
              values: vals
            };
          case val == null:
            return {
              op: ops.single,
              value: String(val)
            };
          default:
            return {
              op: ops.absent
            };
        }
      })();
      if ((!/or/.test(q.constraintLogic)) && (newCon.op === 'ONE OF')) {
        if (existing = _.findWhere(q.constraints, {
          path: path,
          op: 'ONE OF'
        })) {
          existing.values = newCon.values;
          return q.trigger('change:constraints', q.constraints);
        }
      }
      return q.addConstraint(_.extend(newCon, {
        path: path
      }));
    };

    SummaryItemsControls.prototype.changeSelection = function(f) {
      return this.model.items.each(function(item) {
        return _.defer(f, item);
      });
    };

    return SummaryItemsControls;

  })(CoreView);

}).call(this);
