(function() {
  var Backbone, CoreView, FacetRow, IGNORE, Messages, SetsPathNames, SummaryItems, SummaryItemsControls, Templates, rowId, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Backbone = require('backbone');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Messages = require('../../messages');

  SetsPathNames = require('../../mixins/sets-path-names');

  SummaryItemsControls = require('./summary-items-controls');

  FacetRow = require('./row');

  require('../../messages/summary');

  IGNORE = function(e) {
    if (e != null) {
      e.preventDefault();
      return e.stopPropagation();
    }
  };

  rowId = function(model) {
    return "row_" + (model.get('id'));
  };

  module.exports = SummaryItems = (function(_super) {
    __extends(SummaryItems, _super);

    function SummaryItems() {
      this.filterItems = __bind(this.filterItems, this);
      return SummaryItems.__super__.constructor.apply(this, arguments);
    }

    SummaryItems.include(SetsPathNames);

    SummaryItems.prototype.tagName = 'div';

    SummaryItems.prototype.className = 'im-summary-items';

    SummaryItems.prototype.stateEvents = function() {
      return {
        'change:error': this.setErrOnModel
      };
    };

    SummaryItems.prototype.setErrOnModel = function() {
      return this.model.set(this.state.pick('error'));
    };

    SummaryItems.prototype.initialize = function() {
      SummaryItems.__super__.initialize.apply(this, arguments);
      this.listenTo(this.model.items, 'add', this.addItem);
      this.listenTo(this.model.items, 'remove', this.removeItem);
      this.listenTo(this.state, 'change:typeName change:endName', this.reRender);
      return this.setPathNames();
    };

    SummaryItems.prototype.invariants = function() {
      return {
        modelHasItems: "expected a SummaryItems model, got: " + this.model,
        modelCanHasMore: "expected the correct model methods, got: " + this.model
      };
    };

    SummaryItems.prototype.modelHasItems = function() {
      var _ref;
      return ((_ref = this.model) != null ? _ref.items : void 0) instanceof Backbone.Collection;
    };

    SummaryItems.prototype.modelCanHasMore = function() {
      var _ref;
      return _.isFunction((_ref = this.model) != null ? _ref.hasMore : void 0);
    };

    SummaryItems.prototype.template = Templates.template('summary_items');

    SummaryItems.prototype.getData = function() {
      return _.extend(SummaryItems.__super__.getData.apply(this, arguments), {
        hasMore: this.model.hasMore(),
        colClasses: _.result(this, 'colClasses'),
        colHeaders: _.result(this, 'colHeaders')
      });
    };

    SummaryItems.prototype.colClasses = ['im-item-selector', 'im-item-value', 'im-item-count'];

    SummaryItems.prototype.colHeaders = function() {
      var itemColHeader;
      itemColHeader = this.state.has('typeName') ? "" + (this.state.get('typeName')) + " " + (this.state.get('endName')) : Messages.getText('summary.Item');
      return [' ', itemColHeader, Messages.getText('summary.Count')];
    };

    SummaryItems.prototype.postRender = function() {
      this.tbody = this.$('.im-item-table tbody');
      if (!this.tbody.length) {
        throw new Error('Could not find table');
      }
      this.addControls();
      return this.addItems();
    };

    SummaryItems.prototype.addControls = function() {
      return this.renderChildAt('.im-summary-controls', new SummaryItemsControls({
        model: this.model
      }));
    };

    SummaryItems.prototype.addItems = function(from) {
      var next;
      if (from == null) {
        from = 0;
      }
      if (this.rendered && from < this.model.items.length) {
        next = from + 100;
        return _.defer((function(_this) {
          return function() {
            var frag;
            frag = document.createDocumentFragment();
            _this.model.items.slice(from, next).forEach(function(item) {
              return _this.addItem(item, frag);
            });
            _this.tbody.append(frag);
            return _this.addItems(next);
          };
        })(this));
      }
    };

    SummaryItems.prototype.addItem = function(model, body) {
      if (this.rendered) {
        return this.renderChild(rowId(model), new FacetRow({
          model: model
        }), body != null ? body : this.tbody);
      }
    };

    SummaryItems.prototype.removeItem = function(model) {
      return this.removeChild(rowId(model));
    };

    SummaryItems.prototype.events = function() {
      return {
        'click .im-load-more': 'loadMoreItems',
        'click .im-clear-value-filter': 'clearValueFilter',
        'keyup .im-filter-values': _.throttle(this.filterItems, 250, {
          leading: false
        }),
        'submit': IGNORE,
        'click': IGNORE
      };
    };

    SummaryItems.prototype.loadMoreItems = function() {
      if (this.model.get('loading')) {
        return;
      }
      return this.model.increaseLimit(2);
    };

    SummaryItems.prototype.clearValueFilter = function() {
      var $input;
      $input = this.$('.im-filter-values');
      $input.val(null);
      return this.model.setFilterTerm(null);
    };

    SummaryItems.prototype.filterItems = function(e) {
      var $input, val;
      $input = this.$('.im-filter-values');
      val = $input.val();
      return this.model.setFilterTerm($input.val());
    };

    return SummaryItems;

  })(CoreView);

}).call(this);
