(function() {
  var CoreView, DOWN, NULL_STATS, SummaryStats, Templates, UP, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  DOWN = 40;

  UP = 38;

  NULL_STATS = {
    min: 0,
    max: 0,
    average: 0,
    stdev: 0
  };

  module.exports = SummaryStats = (function(_super) {
    __extends(SummaryStats, _super);

    function SummaryStats() {
      return SummaryStats.__super__.constructor.apply(this, arguments);
    }

    SummaryStats.prototype.RERENDER_EVENT = 'change';

    SummaryStats.prototype.className = 'im-summary-stats';

    SummaryStats.prototype.template = Templates.template('summary_stats');

    SummaryStats.prototype.getData = function() {
      return _.extend({}, NULL_STATS, SummaryStats.__super__.getData.apply(this, arguments));
    };

    SummaryStats.prototype.initialize = function(_arg) {
      this.range = _arg.range;
      SummaryStats.__super__.initialize.apply(this, arguments);
      this.listenTo(this.range, 'change', this.setSliderValues);
      this.listenTo(this.range, 'change', this.setButtonDisabledness);
      this.listenTo(this.range, 'change:min', this.onChangeMin);
      this.listenTo(this.range, 'change:max', this.onChangeMax);
      this.listenForChange(this.model, this.initType, 'integral', 'min', 'max');
      return this.initType();
    };

    SummaryStats.prototype.invariants = function() {
      return {
        hasRange: "No range"
      };
    };

    SummaryStats.prototype.hasRange = function() {
      return this.range != null;
    };

    SummaryStats.prototype.setSliderValues = function() {
      var max, min, _ref, _ref1;
      _ref = this.range.toJSON(), min = _ref.min, max = _ref.max;
      return (_ref1 = this.$slider) != null ? _ref1.slider('option', 'values', [min, max]) : void 0;
    };

    SummaryStats.prototype.onChangeMin = function() {
      var min;
      min = this.range.get('min');
      this.$('input.im-range-min').val(min);
      if ((this.$slider != null) && (this.$slider.slider('values', 0) !== min)) {
        return this.$slider.slider('values', 0, min);
      }
    };

    SummaryStats.prototype.onChangeMax = function() {
      var max;
      max = this.range.get('max');
      this.$('input.im-range-max').val(max);
      if ((this.$slider != null) && (this.$slider.slider('values', 1) !== max)) {
        return this.$slider.slider('values', 1, max);
      }
    };

    SummaryStats.prototype.setButtonDisabledness = function() {
      var changed;
      changed = this.range.isNotAll();
      return this.$('.btn').toggleClass('disabled', !changed);
    };

    SummaryStats.prototype.events = function() {
      return {
        'click': function(e) {
          return e.stopPropagation();
        },
        'keyup input.im-range-min': 'maybeIncrementMin',
        'keyup input.im-range-max': 'maybeIncrementMax',
        'change input.im-range-min': 'setRangeMin',
        'change input.im-range-max': 'setRangeMax',
        'click .btn-primary': 'changeConstraints',
        'click .btn-cancel': 'clearRange'
      };
    };

    SummaryStats.prototype.clearRange = function() {
      return this.range.set(this.model.pick('min', 'max'));
    };

    SummaryStats.prototype.maybeIncrementMin = function(e) {
      return this.maybeIncrement('min', e);
    };

    SummaryStats.prototype.maybeIncrementMax = function(e) {
      return this.maybeIncrement('max', e);
    };

    SummaryStats.prototype.maybeIncrement = function(prop, e) {
      var value;
      value = this.range.get(prop);
      switch (e.keyCode) {
        case DOWN:
          value -= this.step;
          break;
        case UP:
          value += this.step;
      }
      return this.range.set(prop, value);
    };

    SummaryStats.prototype.setRangeMin = function() {
      return this.range.set({
        min: this.parse(this.$('.im-range-min').val())
      });
    };

    SummaryStats.prototype.setRangeMax = function() {
      return this.range.set({
        max: this.parse(this.$('.im-range-max').val())
      });
    };

    SummaryStats.prototype.changeConstraints = function(e) {
      var c, existingConstraints, newConstraints, path, query, _i, _len;
      e.preventDefault();
      e.stopPropagation();
      path = this.model.view.toString();
      query = this.model.query;
      existingConstraints = _.where(query.constraints, {
        path: path
      });
      newConstraints = this.range.nulled ? [
        {
          path: path,
          op: 'IS NULL'
        }
      ] : [
        {
          path: path,
          op: ">=",
          value: this.range.get('min')
        }, {
          path: path,
          op: "<=",
          value: this.range.get('max')
        }
      ];
      for (_i = 0, _len = existingConstraints.length; _i < _len; _i++) {
        c = existingConstraints[_i];
        query.removeConstraint(c, {
          silent: true
        });
      }
      return query.addConstraints(newConstraints);
    };

    SummaryStats.prototype.postRender = function() {
      return this.drawSlider();
    };

    SummaryStats.prototype.parse = function(str) {
      var e;
      try {
        if (this.step === 1) {
          return parseInt(str, 10);
        } else {
          return parseFloat(str);
        }
      } catch (_error) {
        e = _error;
        this.model.set({
          error: new Error("Could not parse '" + str + "' as a number")
        });
        return null;
      }
    };

    SummaryStats.prototype.initType = function() {
      var integral, max, min, _ref;
      _ref = this.model.toJSON(), integral = _ref.integral, min = _ref.min, max = _ref.max;
      this.step = integral ? 1 : Math.abs((max - min) / 100);
      return this.round = integral ? Math.round : _.identity;
    };

    SummaryStats.prototype.drawSlider = function() {
      var max, min, _ref;
      _ref = this.model.pick('min', 'max'), max = _ref.max, min = _ref.min;
      return this.activateSlider({
        range: true,
        min: min,
        max: max,
        values: [min, max],
        step: this.step,
        slide: (function(_this) {
          return function(e, ui) {
            var _ref1;
            return (_ref1 = _this.range) != null ? _ref1.set({
              min: ui.values[0],
              max: ui.values[1]
            }) : void 0;
          };
        })(this)
      });
    };

    SummaryStats.prototype.reRender = function() {
      this.destroySlider();
      return SummaryStats.__super__.reRender.apply(this, arguments);
    };

    SummaryStats.prototype.activateSlider = function(opts) {
      this.destroySlider();
      this.$slider = this.$('.slider');
      return this.$slider.slider(opts);
    };

    SummaryStats.prototype.destroySlider = function() {
      if (this.$slider != null) {
        this.$slider.slider('destroy');
        return this.$slider = null;
      }
    };

    SummaryStats.prototype.remove = function() {
      this.destroySlider();
      return SummaryStats.__super__.remove.apply(this, arguments);
    };

    return SummaryStats;

  })(CoreView);

}).call(this);
