(function() {
  var CoreView, VisualisationBase, d3, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  d3 = require('d3-browserify');

  _ = require('underscore');

  CoreView = require('../../core-view');

  module.exports = VisualisationBase = (function(_super) {
    __extends(VisualisationBase, _super);

    function VisualisationBase() {
      return VisualisationBase.__super__.constructor.apply(this, arguments);
    }

    VisualisationBase.prototype.chartHeight = 0;

    VisualisationBase.prototype.chartWidth = 0;

    VisualisationBase.prototype.initialize = function() {
      VisualisationBase.__super__.initialize.apply(this, arguments);
      return this.listenTo(this.model, 'change:loading', this.reRender);
    };

    VisualisationBase.prototype.postRender = function() {
      if (this.model.get('loading')) {
        return;
      }
      return this.addChart();
    };

    VisualisationBase.prototype.addChart = function() {
      return _.defer((function(_this) {
        return function() {
          var e;
          try {
            return _this._drawD3Chart();
          } catch (_error) {
            e = _error;
            return _this.model.set({
              error: e
            });
          }
        };
      })(this));
    };

    VisualisationBase.prototype.getScales = function() {
      throw new Error('Not implemented');
    };

    VisualisationBase.prototype.selectNodes = function(chart) {
      throw new Error('not implemented');
    };

    VisualisationBase.prototype.getChartData = function(scales) {
      throw new Error('not implemented');
    };

    VisualisationBase.prototype.exit = function(selection) {
      return selection.remove();
    };

    VisualisationBase.prototype.enter = function(selection, scales) {
      throw new Error('not implemented');
    };

    VisualisationBase.prototype.update = function(selection, scales) {
      throw new Error('not implemented');
    };

    VisualisationBase.prototype.drawAxes = function(chart, scales) {};

    VisualisationBase.prototype.shouldNotDrawChart = function() {
      return false;
    };

    VisualisationBase.prototype._drawD3Chart = function() {
      var chart, scales;
      if (this.shouldNotDrawChart()) {
        return;
      }
      this.initChart();
      scales = this.getScales();
      chart = this.getCanvas();
      this.updateChart(chart, scales);
      return this.drawAxes(chart, scales);
    };

    VisualisationBase.prototype.updateChart = function(chart, scales) {
      var selection;
      if (this.shouldNotDrawChart()) {
        return;
      }
      if (chart == null) {
        chart = this.getCanvas();
      }
      if (scales == null) {
        scales = this.getScales();
      }
      selection = this.selectNodes(chart).data(this.getChartData(scales));
      this.exit(selection.exit());
      this.enter(selection.enter(), scales);
      return this.update(selection, scales);
    };

    VisualisationBase.prototype.initChart = function() {
      return this.chartWidth = this.$el.closest(':visible').width();
    };

    VisualisationBase.prototype.getCanvas = function() {
      return this.paper != null ? this.paper : this.paper = d3.select(this.el).append('svg').attr('class', 'im-summary-chart').attr('width', this.chartWidth).attr('height', this.chartHeight);
    };

    return VisualisationBase;

  })(CoreView);

}).call(this);
