(function() {
  var Body, Constraints, CoreView, FilterDialogue, LogicManager, Messages, Modal, Templates, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../core-view');

  Modal = require('./modal');

  Messages = require('../messages');

  Templates = require('../templates');

  Constraints = require('./constraints');

  require('../messages/constraints');

  require('../messages/logic');

  LogicManager = (function(_super) {
    __extends(LogicManager, _super);

    function LogicManager() {
      return LogicManager.__super__.constructor.apply(this, arguments);
    }

    LogicManager.prototype.className = 'form im-evenly-spaced im-constraint-logic';

    LogicManager.prototype.tagName = 'form';

    LogicManager.prototype.template = Templates.template('logic-manager-body');

    LogicManager.prototype.parameters = ['query'];

    LogicManager.prototype.initialize = function() {
      var c, codes;
      LogicManager.__super__.initialize.apply(this, arguments);
      codes = (function() {
        var _i, _len, _ref, _results;
        _ref = this.query.constraints;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          c = _ref[_i];
          if (c.code) {
            _results.push(c.code);
          }
        }
        return _results;
      }).call(this);
      this.model.set({
        logic: this.query.constraintLogic
      });
      return this.state.set({
        disabled: true,
        defaultLogic: codes.join(' and ')
      });
    };

    LogicManager.prototype.events = function() {
      return {
        'change .im-logic': this.setLogic,
        'submit': this.applyChanges
      };
    };

    LogicManager.prototype.modelEvents = function() {
      return {
        'change:logic': this.setDisabled
      };
    };

    LogicManager.prototype.stateEvents = function() {
      return {
        'change:disabled': this.reRender
      };
    };

    LogicManager.prototype.setDisabled = function() {
      var current, newLogic;
      newLogic = this.model.get('logic');
      current = this.query.constraintLogic;
      return this.state.set({
        disabled: newLogic === current
      });
    };

    LogicManager.prototype.setLogic = function(e) {
      return this.model.set({
        logic: e.target.value
      });
    };

    LogicManager.prototype.applyChanges = function(e) {
      var newLogic;
      if (e != null) {
        e.preventDefault();
      }
      if (e != null) {
        e.stopPropagation();
      }
      if (!this.state.get('disabled')) {
        newLogic = this.model.get('logic');
        this.query.constraintLogic = newLogic;
        return this.query.trigger('change:logic', newLogic);
      }
    };

    return LogicManager;

  })(CoreView);

  Body = (function(_super) {
    var CODES;

    __extends(Body, _super);

    function Body() {
      return Body.__super__.constructor.apply(this, arguments);
    }

    Body.prototype.template = Templates.template('active-constraints');

    Body.prototype.initialize = function() {
      Body.__super__.initialize.apply(this, arguments);
      return this.assignCodes();
    };

    Body.prototype.stateEvents = function() {
      return {
        'change:adding': this.reRender
      };
    };

    CODES = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    Body.prototype.assignCodes = function() {
      var c, code, codes, constraints, _i, _len, _results;
      constraints = (function() {
        var _i, _len, _ref, _results;
        _ref = this.query.constraints;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          c = _ref[_i];
          if (c.op != null) {
            _results.push(c);
          }
        }
        return _results;
      }).call(this);
      if (constraints.length < 2) {
        return;
      }
      codes = CODES.split('');
      _results = [];
      for (_i = 0, _len = constraints.length; _i < _len; _i++) {
        c = constraints[_i];
        if (c.code == null) {
          _results.push((function() {
            var _results1;
            _results1 = [];
            while ((c.code == null) && (code = codes.shift())) {
              if (!(_.any(constraints, function(con) {
                return con.code === code;
              }))) {
                _results1.push(c.code = code);
              } else {
                _results1.push(void 0);
              }
            }
            return _results1;
          })());
        }
      }
      return _results;
    };

    Body.prototype.postRender = function() {
      var constraints, mth;
      Body.__super__.postRender.apply(this, arguments);
      constraints = this.getConstraints();
      if (constraints.length > 1) {
        this.renderChild('logic', new LogicManager({
          query: this.query
        }));
      }
      mth = this.state.get('adding') ? 'slideUp' : 'slideDown';
      return this.$('.im-current-constraints')[mth](400);
    };

    Body.prototype.getConAdder = function() {
      if (this.state.get('adding')) {
        return Body.__super__.getConAdder.apply(this, arguments);
      }
    };

    return Body;

  })(Constraints);

  module.exports = FilterDialogue = (function(_super) {
    __extends(FilterDialogue, _super);

    function FilterDialogue() {
      return FilterDialogue.__super__.constructor.apply(this, arguments);
    }

    FilterDialogue.prototype.parameters = ['query'];

    FilterDialogue.prototype.modalSize = function() {
      return 'lg';
    };

    FilterDialogue.prototype.className = function() {
      return FilterDialogue.__super__.className.apply(this, arguments) + ' im-filter-manager';
    };

    FilterDialogue.prototype.title = function() {
      return Messages.getText('constraints.Heading', {
        n: this.query.constraints.length
      });
    };

    FilterDialogue.prototype.initState = function() {
      return this.state.set({
        adding: false,
        disabled: false
      });
    };

    FilterDialogue.prototype.act = function() {
      return this.state.set({
        adding: true,
        disabled: true
      });
    };

    FilterDialogue.prototype.dismissAction = function() {
      return Messages.getText('Cancel');
    };

    FilterDialogue.prototype.primaryAction = function() {
      return Messages.getText('constraints.DefineNew');
    };

    FilterDialogue.prototype.initialize = function() {
      FilterDialogue.__super__.initialize.apply(this, arguments);
      this.listenTo(this, 'shown', this.renderBodyContent);
      return this.listenTo(this.query, 'change:constraints', this.onChangeConstraints);
    };

    FilterDialogue.prototype.onChangeConstraints = function() {
      this.initState();
      return this.renderTitle();
    };

    FilterDialogue.prototype.renderBodyContent = function() {
      var body;
      if (this.shown) {
        body = this.$('.modal-body');
        return _.defer((function(_this) {
          return function() {
            return _this.renderChild('cons', new Body({
              state: _this.state,
              query: _this.query
            }), body);
          };
        })(this));
      }
    };

    return FilterDialogue;

  })(Modal);

}).call(this);
