(function() {
  var AppendToListBody, AppendToListModel, BaseAppendDialogue, BaseCreateListDialogue, CoreCollection, CoreModel, LIST_NOT_SUITABLE, Messages, NO_SUITABLE_LISTS, NO_TARGET_SELECTED, PossibleList, PossibleLists, Promise, TARGET_DOES_NOT_EXIST, onlyCurrent, theListIsSuitable, unpackLists, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Promise = require('es6-promise').Promise;

  CoreCollection = require('../../core/collection');

  CoreModel = require('../../core-model');

  Messages = require('../../messages');

  BaseCreateListDialogue = require('./base-dialogue');

  AppendToListBody = require('./append-to-list-body');

  AppendToListModel = require('../../models/append-to-list');

  NO_SUITABLE_LISTS = {
    key: 'lists.NoSuitableLists',
    level: 'Warning',
    cannotDismiss: true
  };

  LIST_NOT_SUITABLE = {
    key: 'lists.TargetNotCorrectType',
    level: 'Error',
    cannotDismiss: true
  };

  TARGET_DOES_NOT_EXIST = {
    key: 'lists.TargetDoesNotExist',
    level: 'Error',
    cannotDismiss: true
  };

  NO_TARGET_SELECTED = {
    key: 'lists.NoTargetSelected',
    level: 'Info',
    cannotDismiss: true
  };

  theListIsSuitable = function(path) {
    return function(list) {
      if (!(path.isa(list.type))) {
        return Promise.reject(LIST_NOT_SUITABLE);
      }
    };
  };

  onlyCurrent = function(ls) {
    return _.where(ls, {
      status: 'CURRENT'
    });
  };

  unpackLists = function(ls) {
    var name, size, type, _i, _len, _ref, _results;
    _results = [];
    for (_i = 0, _len = ls.length; _i < _len; _i++) {
      _ref = ls[_i], name = _ref.name, type = _ref.type, size = _ref.size;
      _results.push({
        name: name,
        type: type,
        size: size,
        id: name
      });
    }
    return _results;
  };

  PossibleList = (function(_super) {
    __extends(PossibleList, _super);

    function PossibleList() {
      return PossibleList.__super__.constructor.apply(this, arguments);
    }

    PossibleList.prototype.defaults = function() {
      return {
        typeName: null,
        name: null,
        size: 0
      };
    };

    PossibleList.prototype.initialize = function() {
      PossibleList.__super__.initialize.apply(this, arguments);
      return this.fetchTypeName();
    };

    PossibleList.prototype.fetchTypeName = function() {
      var s, type;
      s = this.collection.service;
      type = this.get('type');
      return s.fetchModel().then(function(model) {
        return model.makePath(type);
      }).then(function(path) {
        return path.getDisplayName();
      }).then((function(_this) {
        return function(name) {
          return _this.set({
            typeName: name
          });
        };
      })(this));
    };

    return PossibleList;

  })(CoreModel);

  PossibleLists = (function(_super) {
    __extends(PossibleLists, _super);

    PossibleLists.prototype.model = PossibleList;

    function PossibleLists(_arg) {
      this.service = _arg.service;
      PossibleLists.__super__.constructor.call(this);
    }

    return PossibleLists;

  })(CoreCollection);

  module.exports = BaseAppendDialogue = (function(_super) {
    __extends(BaseAppendDialogue, _super);

    function BaseAppendDialogue() {
      return BaseAppendDialogue.__super__.constructor.apply(this, arguments);
    }

    BaseAppendDialogue.prototype.Body = AppendToListBody;

    BaseAppendDialogue.prototype.Model = AppendToListModel;

    BaseAppendDialogue.prototype.title = function() {
      return Messages.getText('lists.AppendToListTitle', this.getData());
    };

    BaseAppendDialogue.prototype.primaryAction = function() {
      return Messages.getText('lists.Append');
    };

    BaseAppendDialogue.prototype.initialize = function() {
      BaseAppendDialogue.__super__.initialize.apply(this, arguments);
      this.listenTo(this.getPossibleLists(), 'remove reset', this.verifyState);
      return this.listenTo(this.getPossibleLists(), 'add reset', this.setTargetIfOnlyOne);
    };

    BaseAppendDialogue.prototype.getPossibleLists = function() {
      return this.possibleLists != null ? this.possibleLists : this.possibleLists = new PossibleLists({
        service: this.getService()
      });
    };

    BaseAppendDialogue.prototype.processQuery = function(query) {
      return query.appendToList(this.model.get('target'));
    };

    BaseAppendDialogue.prototype.modelEvents = function() {
      return _.extend(BaseAppendDialogue.__super__.modelEvents.apply(this, arguments), {
        'change:target': 'onChangeTarget'
      });
    };

    BaseAppendDialogue.prototype.initState = function() {
      BaseAppendDialogue.__super__.initState.apply(this, arguments);
      this.fetchSuitableLists();
      return this.verifyState();
    };

    BaseAppendDialogue.prototype.checkThereAreLists = function() {
      if (!this.getPossibleLists().size()) {
        return this.state.set({
          error: NO_SUITABLE_LISTS
        });
      }
    };

    BaseAppendDialogue.prototype.onChangeTarget = function() {
      this.setTitle();
      return this.verifyState();
    };

    BaseAppendDialogue.prototype.verifyState = function() {
      this.state.unset('error');
      this.checkThereAreLists();
      this.verifyTarget();
      return this.verifyTargetExistsAndIsSuitable();
    };

    BaseAppendDialogue.prototype.setTargetIfOnlyOne = function() {
      var pls;
      pls = this.getPossibleLists();
      if (pls.length === 1) {
        return this.model.set({
          target: pls.at(0).get('name')
        });
      }
    };

    BaseAppendDialogue.prototype.verifyTarget = function() {
      if (!this.model.get('target')) {
        return this.state.set({
          error: NO_TARGET_SELECTED
        });
      }
    };

    BaseAppendDialogue.prototype.onChangeType = function() {
      BaseAppendDialogue.__super__.onChangeType.apply(this, arguments);
      return this.fetchSuitableLists();
    };

    BaseAppendDialogue.prototype.getBodyOptions = function() {
      return _.extend(BaseAppendDialogue.__super__.getBodyOptions.apply(this, arguments), {
        collection: this.getPossibleLists()
      });
    };

    BaseAppendDialogue.prototype.verifyTargetExistsAndIsSuitable = function() {
      var path, target, type;
      type = this.getType();
      if (type == null) {
        return;
      }
      target = this.model.get('target');
      if (target == null) {
        return;
      }
      path = type.model.makePath(type.name);
      return this.getService().fetchList(target).then(theListIsSuitable(path), (function() {
        return TARGET_DOES_NOT_EXIST;
      })).then(null, (function(_this) {
        return function(e) {
          return _this.state.set({
            error: e
          });
        };
      })(this));
    };

    BaseAppendDialogue.prototype.fetchSuitableLists = function() {
      var path, type;
      type = this.getType();
      if (type == null) {
        return this.getPossibleLists().reset();
      }
      path = type.model.makePath(type.name);
      return this.getService().fetchLists().then(function(lists) {
        return _.filter(lists, function(list) {
          return path.isa(list.type);
        });
      }).then(onlyCurrent).then(unpackLists).then((function(_this) {
        return function(ls) {
          return _this.getPossibleLists().reset(ls);
        };
      })(this)).then(null, (function(_this) {
        return function(e) {
          return _this.state.set({
            error: e
          });
        };
      })(this));
    };

    return BaseAppendDialogue;

  })(BaseCreateListDialogue);

}).call(this);
