(function() {
  var NO_COMMON_TYPE, NO_OBJECTS_SELECTED, SelectedObjects, TypeAssertions;

  SelectedObjects = require('../../models/selected-objects');

  TypeAssertions = require('../../core/type-assertions');

  NO_COMMON_TYPE = {
    level: 'Error',
    key: 'lists.NoCommonType',
    cannotDismiss: true
  };

  NO_OBJECTS_SELECTED = {
    level: 'Info',
    key: 'lists.NoObjectsSelected',
    cannotDismiss: true
  };

  module.exports = function(Base) {
    return {
      parameters: ['service', 'collection'],
      parameterTypes: {
        collection: TypeAssertions.InstanceOf(SelectedObjects, 'SelectedObjects')
      },
      className: function() {
        return Base.prototype.className.call(this) + ' im-list-picker';
      },
      getQuery: function() {
        return this.service.query({
          from: this.model.get('type'),
          select: ['id'],
          where: [
            {
              path: this.model.get('type'),
              op: 'IN',
              ids: this.getIds()
            }
          ]
        });
      },
      fetchCount: function() {
        return Promise.resolve(this.collection.size());
      },
      getType: function() {
        var _ref;
        if (this.model.get('type')) {
          return (_ref = this.schema) != null ? _ref.makePath(this.model.get('type')).getType() : void 0;
        }
      },
      getService: function() {
        return this.service;
      },
      collectionEvents: function() {
        return {
          'change:commonType change:typeName': this.setType,
          'add remove': this.setCount
        };
      },
      initiallyMinimised: true,
      getIds: function() {
        return this.collection.map(function(o) {
          return o.get('id');
        });
      },
      onChangeType: function() {
        Base.prototype.onChangeType.call(this);
        return this.verifyState();
      },
      verifyState: function() {
        var _ref;
        if ((_ref = Base.prototype.verifyState) != null) {
          _ref.call(this);
        }
        if (!this.collection.size()) {
          return this.state.set({
            error: NO_OBJECTS_SELECTED
          });
        } else if (!this.model.get('type')) {
          return this.state.set({
            error: NO_COMMON_TYPE
          });
        }
      },
      setType: function() {
        this.model.set({
          type: this.collection.state.get('commonType')
        });
        return this.state.set({
          typeName: this.collection.state.get('typeName')
        });
      }
    };
  };

}).call(this);
