(function() {
  var ListValueControls, Messages, Promise, View, formatList, fs, getOptionValue, helpers, html, mustacheSettings, template, withDisplayNames, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  

  Promise = require('es6-promise').Promise;

  Messages = require('../messages');

  View = require('../core-view');

  Messages.set({
    'convalue.BadType': "<%- name %> is of the wrong type (<%- type %>).",
    'convalue.EmptyList': "<%- name %> is empty. This query will always return 0 results",
    'convalue.BadList': "You do not have access to list called \"<%- name %>\". Choose one from the list."
  });

  helpers = require('../templates/helpers');

  mustacheSettings = require('../templates/mustache-settings');

  html = "<% if (suitableLists.length) { %>\n    <%= select(suitableLists, isSelected, 'span7 im-value-options im-con-value im-con-value-list form-control', formatList, getOptionValue) %>\n<% } else { %>\n    <%- messages.getText('conbuilder.NoSuitableLists') %>\n<% } %>\n";

  template = _.template(html);

  getOptionValue = function(list) {
    return list.name;
  };

  formatList = function(list) {
    if (list.size) {
      return "" + list.name + " (" + list.size + " " + list.typeName + "s)";
    } else {
      return list.name;
    }
  };

  withDisplayNames = function(m) {
    return function(l) {
      var p;
      p = m.makePath(l.type);
      return p.getDisplayName().then(function(typeName) {
        return _.extend(l, {
          typeName: typeName
        });
      });
    };
  };

  module.exports = ListValueControls = (function(_super) {
    __extends(ListValueControls, _super);

    function ListValueControls() {
      return ListValueControls.__super__.constructor.apply(this, arguments);
    }

    ListValueControls.prototype.className = 'im-list-value-controls';

    ListValueControls.prototype.initialize = function(_arg) {
      this.query = _arg.query;
      ListValueControls.__super__.initialize.apply(this, arguments);
      this.initialValue = this.model.get('value');
      this.path = this.model.get('path');
      this.setSuitableLists();
      this.listenTo(this.model, 'change', this.reRender);
      this.listenTo(this.model, 'change:value', this.checkCurrentValue);
      return this.checkCurrentValue();
    };

    ListValueControls.prototype.checkCurrentValue = function() {
      var doesntExist, exists, name;
      name = this.model.get('value');
      doesntExist = (function(_this) {
        return function(error) {
          _this.model.set({
            error: new Error(Messages.getText('convalue.BadList', {
              name: name
            }))
          });
          return _this.listenToOnce(_this.model, 'change:value', function() {
            return _this.model.unset('error');
          });
        };
      })(this);
      exists = function(list) {
        var err;
        err = !list.size ? 'convalue.EmptyList' : !this.path.isa(list.type) ? 'convalue.BadType' : void 0;
        if (err != null) {
          this.model.set({
            error: new Error(Messages.getText(err, list))
          });
          return this.listenToOnce(this.model, 'change:value', (function(_this) {
            return function() {
              return _this.model.unset('error');
            };
          })(this));
        }
      };
      return this.query.service.fetchList(name).then(exists, doesntExist);
    };

    ListValueControls.prototype.events = function() {
      return {
        'change select': 'setList'
      };
    };

    ListValueControls.prototype.setList = function() {
      return this.model.set({
        value: this.$('select').val()
      });
    };

    ListValueControls.prototype.setSuitableLists = function() {
      var failed, success;
      if (!this.model.has('suitableLists')) {
        success = (function(_this) {
          return function(suitableLists) {
            return _this.model.set({
              suitableLists: suitableLists
            });
          };
        })(this);
        failed = (function(_this) {
          return function(error) {
            return _this.model.set({
              error: error
            });
          };
        })(this);
        return this.fetchSuitableLists().then(success, failed);
      }
    };

    ListValueControls.prototype.fetchSuitableLists = function() {
      return this.__suitable_lists != null ? this.__suitable_lists : this.__suitable_lists = this.query.service.fetchLists().then((function(_this) {
        return function(lists) {
          var l, selectables;
          selectables = (function() {
            var _i, _len, _results;
            _results = [];
            for (_i = 0, _len = lists.length; _i < _len; _i++) {
              l = lists[_i];
              if (l.size && this.path.isa(l.type)) {
                _results.push(l);
              }
            }
            return _results;
          }).call(_this);
          return Promise.all(selectables.map(withDisplayNames(_this.query.model)));
        };
      })(this));
    };

    ListValueControls.prototype.getSuitableLists = function() {
      var currentValue, currentlySelected, suitableLists;
      currentValue = {
        name: this.initialValue
      };
      suitableLists = this.model.get('suitableLists') || [];
      currentlySelected = _.findWhere(suitableLists, currentValue);
      if (currentlySelected == null) {
        currentlySelected = currentValue;
      }
      return _.uniq([currentlySelected].concat(suitableLists), false, 'name');
    };

    ListValueControls.prototype.template = function(data) {
      data = _.extend({
        formatList: formatList,
        getOptionValue: getOptionValue,
        messages: Messages
      }, helpers, data);
      return template(data);
    };

    ListValueControls.prototype.getData = function() {
      var currentValue, isSelected, suitableLists;
      currentValue = this.model.get('value');
      suitableLists = this.getSuitableLists();
      isSelected = function(opt) {
        return opt.name === currentValue;
      };
      return {
        suitableLists: suitableLists,
        isSelected: isSelected
      };
    };

    return ListValueControls;

  })(View);

}).call(this);
