(function() {
  var Collection, IS_BLANK, Icons, Messages, Model, MultiValueControls, Options, Templates, ValueControl, ValueModel, Values, View, ignore, mustacheSettings, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Collection = require('backbone').Collection;

  Messages = require('../messages');

  Templates = require('../templates');

  Icons = require('../icons');

  View = require('../core-view');

  Model = require('../core-model');

  Collection = require('../core/collection');

  Options = require('../options');

  IS_BLANK = require('../patterns').IS_BLANK;

  ignore = require('../utils/events').ignore;

  Messages.setWithPrefix('multivalue', {
    AddValue: 'Add value',
    AddValueShort: '+',
    SaveValue: 'Save changes'
  });

  mustacheSettings = require('../templates/mustache-settings');

  ValueModel = (function(_super) {
    __extends(ValueModel, _super);

    function ValueModel() {
      return ValueModel.__super__.constructor.apply(this, arguments);
    }

    ValueModel.prototype.defaults = function() {
      return {
        editing: false,
        scratch: null,
        selected: true
      };
    };

    ValueModel.prototype.idAttribute = 'value';

    return ValueModel;

  })(Model);

  Values = (function(_super) {
    __extends(Values, _super);

    function Values() {
      return Values.__super__.constructor.apply(this, arguments);
    }

    Values.prototype.model = ValueModel;

    return Values;

  })(Collection);

  ValueControl = (function(_super) {
    __extends(ValueControl, _super);

    function ValueControl() {
      return ValueControl.__super__.constructor.apply(this, arguments);
    }

    ValueControl.prototype.Model = ValueModel;

    ValueControl.prototype.tagName = 'tr';

    ValueControl.prototype.initialize = function() {
      this.model.set({
        editing: false,
        scratch: this.model.get('value')
      });
      return this.listenTo(this.model, 'change:editing change:value change:selected', this.reRender);
    };

    ValueControl.prototype.events = function() {
      return {
        'click .input-group': function(e) {
          return e != null ? e.stopPropagation() : void 0;
        },
        'click .im-edit': 'editValue',
        'keyup input': 'updateScratch',
        'change input': 'updateScratch',
        'click .im-save': 'saveValue',
        'click .im-cancel': 'cancelEditing',
        'click': 'toggleSelected'
      };
    };

    ValueControl.prototype.updateScratch = function(e) {
      ignore(e);
      return this.model.set({
        scratch: e.target.value
      });
    };

    ValueControl.prototype.saveValue = function(e) {
      ignore(e);
      return this.model.set({
        editing: false,
        value: this.model.get('scratch')
      });
    };

    ValueControl.prototype.cancelEditing = function(e) {
      ignore(e);
      return this.model.set({
        editing: false,
        scratch: this.model.get('value')
      });
    };

    ValueControl.prototype.editValue = function(e) {
      ignore(e);
      return this.model.toggle('editing');
    };

    ValueControl.prototype.toggleSelected = function(e) {
      ignore(e);
      return this.model.toggle('selected');
    };

    ValueControl.prototype.getData = function() {
      return _.extend({
        icons: Icons,
        messages: Messages
      }, this.model.toJSON());
    };

    ValueControl.prototype.template = Templates.template('value-control-row', mustacheSettings);

    return ValueControl;

  })(View);

  module.exports = MultiValueControls = (function(_super) {
    __extends(MultiValueControls, _super);

    function MultiValueControls() {
      return MultiValueControls.__super__.constructor.apply(this, arguments);
    }

    MultiValueControls.prototype.className = 'im-value-options im-multi-value-table';

    MultiValueControls.prototype.initialize = function() {
      var v, _i, _len, _ref;
      MultiValueControls.__super__.initialize.apply(this, arguments);
      this.values = new Values;
      _ref = this.model.get('values') || [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        v = _ref[_i];
        this.values.add({
          value: v
        });
      }
      this.listenTo(this.values, 'change:selected change:value add', this.updateModel);
      this.listenTo(this.values, 'add', this.renderValue);
      return this.listenTo(this.values, 'remove', this.removeValue);
    };

    MultiValueControls.prototype.stateEvents = function() {
      return {
        'change:value': this.onChangeNewValue
      };
    };

    MultiValueControls.prototype.modelEvents = function() {
      return {
        'change:op': this.onChangeOp,
        'change:values': this.onChangeValues
      };
    };

    MultiValueControls.prototype.onChangeValues = function() {
      var current, value;
      if (this.model.previous('values') != null) {
        return;
      }
      current = this.model.get('values');
      return this.values.set((function() {
        var _i, _len, _ref, _results;
        _ref = current != null ? current : [];
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          value = _ref[_i];
          _results.push({
            value: value
          });
        }
        return _results;
      })());
    };

    MultiValueControls.prototype.onChangeOp = function() {
      var newOp;
      newOp = this.model.get('op');
      if ((newOp === '=' || newOp === '!=') && (!this.model.has('value'))) {
        return this.model.set({
          values: null,
          value: this.values.first().get('value')
        });
      }
    };

    MultiValueControls.prototype.events = function() {
      return {
        'keyup .im-new-multi-value': this.updateNewValue,
        'click .im-add': this.addValue
      };
    };

    MultiValueControls.prototype.onChangeNewValue = function() {
      return this.$('.im-new-multi-value').val(this.state.get('value'));
    };

    MultiValueControls.prototype.updateNewValue = function() {
      return this.state.set({
        value: this.$('.im-new-multi-value').val()
      });
    };

    MultiValueControls.prototype.addValue = function(e) {
      var value;
      ignore(e);
      value = this.state.get('value');
      if ((value == null) || IS_BLANK.test(value)) {
        this.model.set({
          error: new Error('please enter a value')
        });
        return this.listenToOnce(this.state, 'change:value', (function(_this) {
          return function() {
            return _this.model.unset('error');
          };
        })(this));
      } else {
        this.values.add({
          value: value
        });
        return this.state.unset('value');
      }
    };

    MultiValueControls.prototype.updateModel = function() {
      var m;
      return this.model.set({
        values: (function() {
          var _i, _len, _ref, _results;
          _ref = this.values.where({
            selected: true
          });
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            m = _ref[_i];
            _results.push(m.get('value'));
          }
          return _results;
        }).call(this)
      });
    };

    MultiValueControls.prototype.getData = function() {
      return {
        messages: Messages
      };
    };

    MultiValueControls.prototype.template = Templates.template('add-value-control');

    MultiValueControls.prototype.postRender = function() {
      this.$table = this.$('table');
      return this.renderRows();
    };

    MultiValueControls.prototype.renderRows = function() {
      return this.values.each((function(_this) {
        return function(m) {
          return _this.renderValue(m);
        };
      })(this));
    };

    MultiValueControls.prototype.removeValue = function() {
      return this.removeChild(m.id);
    };

    MultiValueControls.prototype.renderValue = function(m) {
      return this.renderChild(m.id, new ValueControl({
        model: m
      }), this.$table);
    };

    MultiValueControls.prototype.remove = function() {
      this.values.close();
      return MultiValueControls.__super__.remove.apply(this, arguments);
    };

    return MultiValueControls;

  })(View);

}).call(this);
