(function() {
  var Attribute, CoreView, Options, PathChooser, Reference, ReverseReference, RootClass, appendField, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Options = require('../options');

  CoreView = require('../core-view');

  Attribute = require('./pathtree/attribute');

  RootClass = require('./pathtree/root');

  Reference = require('./pathtree/reference');

  ReverseReference = require('./pathtree/reverse-reference');

  appendField = function(pth, fld) {
    return pth.append(fld);
  };

  module.exports = PathChooser = (function(_super) {
    __extends(PathChooser, _super);

    function PathChooser() {
      this.createSubFinder = __bind(this.createSubFinder, this);
      return PathChooser.__super__.constructor.apply(this, arguments);
    }

    PathChooser.prototype.parameters = ['model', 'query', 'chosenPaths', 'openNodes', 'view', 'trail'];

    PathChooser.prototype.tagName = 'ul';

    PathChooser.prototype.className = 'im-path-chooser';

    PathChooser.prototype.initialize = function() {
      var attr, fieldType, name, toPath, _i, _len, _ref;
      PathChooser.__super__.initialize.apply(this, arguments);
      this.path = _.last(this.trail) || this.model.get('root') || this.query.makePath(this.query.root);
      this.cd = this.path.getEndClass();
      toPath = appendField.bind(null, this.path);
      _ref = ['attributes', 'references', 'collections'];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        fieldType = _ref[_i];
        this[fieldType] = (function() {
          var _ref1, _results;
          _ref1 = this.cd[fieldType];
          _results = [];
          for (name in _ref1) {
            attr = _ref1[name];
            _results.push(toPath(attr));
          }
          return _results;
        }).call(this);
      }
      this.listenTo(this.model, 'change:allowRevRefs', this.render);
      return this.listenTo(this.openNodes, 'reset', this.render);
    };

    PathChooser.prototype.getDepth = function() {
      return this.trail.length;
    };

    PathChooser.prototype.showRoot = function() {
      return this.getDepth() === 0 && this.model.get('canSelectReferences');
    };

    PathChooser.prototype.events = function() {
      return {
        scroll: this.onScroll
      };
    };

    PathChooser.prototype.onScroll = function() {
      var diff, st;
      if (!this.state.get('ignoreScroll')) {
        st = this.el.scrollTop;
        diff = this.state.has('scroll') ? Math.abs(this.state.get('scroll') - st) : 0;
        if ((st !== 0) || (diff < 50)) {
          return this.state.set({
            scroll: st
          });
        } else {
          return _.defer((function(_this) {
            return function() {
              return _this.el.scrollTop = _this.state.get('scroll');
            };
          })(this));
        }
      }
    };

    PathChooser.prototype.startIgnoringScroll = function() {
      return this.state.set({
        ignoreScroll: true
      });
    };

    PathChooser.prototype.startListeningForScroll = function() {
      if (this.state.has('scroll')) {
        this.el.scrollTop = this.state.get('scroll');
      }
      return this.state.set({
        ignoreScroll: false
      });
    };

    PathChooser.prototype.preRender = function() {
      return this.startIgnoringScroll();
    };

    PathChooser.prototype.postRender = function() {
      var attr, path, ref, root, showId, _i, _j, _len, _len1, _ref, _ref1;
      showId = Options.get('ShowId');
      if (this.showRoot()) {
        root = this.createRoot();
        this.renderChild('root', root);
      }
      _ref = this.attributes;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        path = _ref[_i];
        if (showId || (path.end.name !== 'id')) {
          attr = this.createAttribute(path);
          this.renderChild(path.toString(), attr);
        }
      }
      _ref1 = this.references.concat(this.collections);
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        path = _ref1[_j];
        ref = this.createReference(path);
        this.renderChild(path.toString(), ref);
      }
      return this.startListeningForScroll();
    };

    PathChooser.prototype.createRoot = function() {
      return new RootClass({
        query: this.query,
        model: this.model,
        chosenPaths: this.chosenPaths,
        openNodes: this.openNodes,
        cd: this.cd
      });
    };

    PathChooser.prototype.createAttribute = function(path) {
      return new Attribute({
        model: this.model,
        chosenPaths: this.chosenPaths,
        view: this.view,
        query: this.query,
        trail: this.trail,
        path: path
      });
    };

    PathChooser.prototype.createReference = function(path) {
      var Ref, allowingRevRefs, isLoop;
      isLoop = this.isLoop(path);
      allowingRevRefs = this.model.get('allowRevRefs');
      Ref = isLoop && !allowingRevRefs ? ReverseReference : Reference;
      return new Ref({
        model: this.model,
        chosenPaths: this.chosenPaths,
        query: this.query,
        trail: this.trail,
        path: path,
        view: this.view,
        openNodes: this.openNodes,
        createSubFinder: this.createSubFinder
      });
    };

    PathChooser.prototype.createSubFinder = function(args) {
      return new PathChooser(_.extend({
        model: this.model,
        query: this.query,
        chosenPaths: this.chosenPaths,
        view: this.view,
        openNodes: this.openNodes
      }, args));
    };

    PathChooser.prototype.isLoop = function(path) {
      if ((path.end.reverseReference != null) && this.path.isReference()) {
        if (this.path.getParent().isa(path.end.referencedType)) {
          if (this.path.end.name === path.end.reverseReference) {
            return true;
          }
        }
      }
      return false;
    };

    return PathChooser;

  })(CoreView);

}).call(this);
