(function() {
  var Attribute, Icons, Options, View, highlightMatch, notBlank, stripLeadingSegments, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  View = require('../../core-view');

  Options = require('../../options');

  Icons = require('../../icons');


  /*
   * Type expectations:
   *  - @chosenPaths :: UniqItems
   *  - @model :: CoreModel {multiSelect}
   *  - @trail :: [PathInfo]
   *  - @query :: Query
   *  - @path :: PathInfo
   *
   */

  notBlank = function(s) {
    return (s != null) && /\w/.test(s);
  };

  stripLeadingSegments = function(name) {
    return name != null ? name.replace(/^.*\s*>/, '') : void 0;
  };

  highlightMatch = function(match) {
    return "<strong>" + match + "</strong>";
  };

  module.exports = Attribute = (function(_super) {
    __extends(Attribute, _super);

    function Attribute() {
      return Attribute.__super__.constructor.apply(this, arguments);
    }

    Attribute.prototype.tagName = 'li';

    Attribute.prototype.events = function() {
      return {
        'click a': 'handleClick'
      };
    };

    Attribute.prototype.initialize = function(_arg) {
      this.chosenPaths = _arg.chosenPaths, this.view = _arg.view, this.query = _arg.query, this.path = _arg.path, this.trail = _arg.trail;
      Attribute.__super__.initialize.apply(this, arguments);
      this.depth = this.trail.length + 1;
      this.state.set({
        visible: true,
        highlitName: null,
        name: this.path.toString()
      });
      this.listenTo(this.chosenPaths, 'add remove reset', this.handleChoice);
      this.listenTo(this.state, 'change:visible', this.onChangeVisible);
      this.listenTo(this.state, 'change:highlitName', this.render);
      this.listenTo(this.state, 'change:displayName', this.render);
      return this.path.getDisplayName().then((function(_this) {
        return function(displayName) {
          return _this.state.set({
            displayName: displayName,
            name: stripLeadingSegments(displayName)
          });
        };
      })(this));
    };

    Attribute.prototype.onChangeVisible = function() {
      return this.$el.toggle(this.state.get('visible'));
    };

    Attribute.prototype.getFilterPatterns = function() {
      var filterTerms, t, _i, _len, _ref, _results;
      filterTerms = this.model.get('filter');
      if (notBlank(filterTerms)) {
        _ref = filterTerms.split(/\s+/);
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          t = _ref[_i];
          if (t) {
            _results.push(new RegExp(t, 'i'));
          }
        }
        return _results;
      } else {
        return [];
      }
    };

    Attribute.prototype.handleClick = function(e) {
      var _ref;
      e.stopPropagation();
      e.preventDefault();
      if ((this.model.get('dontSelectView')) && ((_ref = this.view) != null ? _ref.contains(this.path) : void 0)) {
        return;
      }
      if (this.chosenPaths.contains(this.path)) {
        return this.chosenPaths.remove(this.path);
      } else {
        return this.choose();
      }
    };

    Attribute.prototype.choose = function() {
      if (this.model.get('multiSelect')) {
        return this.chosenPaths.add(this.path);
      } else {
        return this.chosenPaths.reset([this.path]);
      }
    };

    Attribute.prototype.handleChoice = function() {
      return this.$el.toggleClass('active', this.chosenPaths.contains(this.path));
    };

    Attribute.prototype.setHighlitName = function(regexps) {
      if (this.state.has('name')) {
        return this.state.set({
          highlitName: this.getHighlitName(regexps)
        });
      } else {
        return this.state.once('change:name', (function(_this) {
          return function() {
            return _this.state.set({
              highlitName: _this.getHighlitName(regexps)
            });
          };
        })(this));
      }
    };

    Attribute.prototype.getHighlitName = function(regexps) {
      var highlit, name, pathName, r, _i, _len, _ref;
      name = this.state.get('name');
      pathName = (_ref = this.path.end) != null ? _ref.name : void 0;
      highlit = name;
      for (_i = 0, _len = regexps.length; _i < _len; _i++) {
        r = regexps[_i];
        highlit = highlit.replace(r, highlightMatch);
      }
      if (/strong/.test(highlit)) {
        return highlit;
      } else {
        return highlightMatch(highlit);
      }
    };

    Attribute.prototype.getDisabled = function() {
      return false;
    };

    Attribute.prototype.getData = function() {
      var icon, name, title;
      title = Options.get('ShowId') ? "" + this.path + " (" + (this.path.getType()) + ")" : '';
      name = this.state.get('highlitName') ? this.state.get('highlitName') : this.state.escape('name');
      icon = Icons.icon('Attribute');
      return {
        icon: icon,
        title: title,
        name: name
      };
    };

    Attribute.prototype.template = _.template("<a href=\"#\" title=\"<%- title %>\">\n  <%= icon %>\n  <span>\n    <%= name %>\n  </span>\n</a>");

    Attribute.prototype.render = function() {
      Attribute.__super__.render.apply(this, arguments);
      this.$el.toggleClass('disabled', this.getDisabled());
      if (Options.get('ShowId')) {
        this.$('a').tooltip({
          placement: 'bottom'
        });
      }
      this.handleChoice();
      if (this.view != null) {
        this.$el.toggleClass('in-view', this.view.contains(this.path));
      }
      return this;
    };

    return Attribute;

  })(View);

}).call(this);
