(function() {
  var $, Bus, CodeGenButton, CoreView, ExportDialogueButton, ListDialogueButton, Options, QueryManagement, QueryTools, SUBSECTIONS, Templates, UndoHistory, subsection, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __slice = [].slice;

  _ = require('underscore');

  $ = require('jquery');

  CoreView = require('../core-view');

  Templates = require('../templates');

  Options = require('../options');

  QueryManagement = require('./query-management-tools');

  UndoHistory = require('./undo-history');

  ListDialogueButton = require('./list-dialogue/button');

  CodeGenButton = require('./code-gen-button');

  ExportDialogueButton = require('./export-dialogue/button');

  Bus = require('../utils/events').Bus;

  SUBSECTIONS = ['im-query-management', 'im-history', 'im-query-consumers'];

  subsection = function(s) {
    return "<div class=\"" + s + " clearfix\"></div>";
  };

  module.exports = QueryTools = (function(_super) {
    __extends(QueryTools, _super);

    function QueryTools() {
      return QueryTools.__super__.constructor.apply(this, arguments);
    }

    QueryTools.prototype.className = 'im-query-tools';

    QueryTools.prototype.parameters = ['tableState', 'history', 'selectedObjects'];

    QueryTools.prototype.optionalParameters = ['bus', 'consumerContainer', 'consumerBtnClass'];

    QueryTools.prototype.bus = new Bus;

    QueryTools.prototype.template = function() {
      return (SUBSECTIONS.map(subsection)).join('');
    };

    QueryTools.prototype.initialize = function() {
      QueryTools.__super__.initialize.apply(this, arguments);
      return this.listenTo(this.history, 'changed:current', this.renderQueryConsumers);
    };

    QueryTools.prototype.renderChildren = function() {
      this.renderManagementTools();
      this.renderUndo();
      return this.renderQueryConsumers();
    };

    QueryTools.prototype.renderManagementTools = function() {
      return this.renderChildAt('.im-query-management', new QueryManagement({
        history: this.history
      }));
    };

    QueryTools.prototype.renderUndo = function() {
      var $undo;
      $undo = this.$('.im-history');
      return this.renderChild('undo', new UndoHistory({
        collection: this.history
      }), $undo);
    };

    QueryTools.prototype.getConsumerContainer = function() {
      var cons;
      if (this.consumerContainer != null) {
        this.consumerContainer.classList.add(Options.get('StylePrefix'));
        this.consumerContainer.classList.add('im-query-consumers');
        this.$('.im-query-management').addClass('im-has-more-space');
        this.$('.im-history').addClass('im-has-more-space');
        return this.consumerContainer;
      } else {
        cons = this.$('.im-query-consumers').empty();
        if (cons.length) {
          return cons;
        }
      }
    };

    QueryTools.prototype.renderQueryConsumers = function() {
      var container, kid, listDialogue, query, selected, _i, _len, _ref;
      container = this.getConsumerContainer();
      if (!container) {
        return;
      }
      query = this.history.getCurrentQuery();
      selected = this.selectedObjects;
      listDialogue = new ListDialogueButton({
        query: query,
        tableState: this.tableState,
        selected: selected
      });
      this.listenTo(listDialogue, 'all', (function(_this) {
        return function() {
          var args, evt, _ref, _ref1;
          evt = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
          (_ref = _this.bus).trigger.apply(_ref, ["list-action:" + evt].concat(__slice.call(args)));
          return (_ref1 = _this.bus).trigger.apply(_ref1, ["list-action", evt].concat(__slice.call(args)));
        };
      })(this));
      this.renderChild('save', new ExportDialogueButton({
        query: query,
        tableState: this.tableState
      }), container);
      this.renderChild('code', new CodeGenButton({
        query: query,
        tableState: this.tableState
      }), container);
      this.renderChild('lists', listDialogue, container);
      if (this.consumerContainer && this.consumerBtnClass) {
        _ref = ['save', 'code', 'lists'];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          kid = _ref[_i];
          console.log('listening to', this.children[kid]);
          this.listenTo(this.children[kid], 'rendered', this.setButtonStyle);
        }
      }
      return this.setButtonStyle();
    };

    QueryTools.prototype.setButtonStyle = function() {
      var cls, con;
      if ((con = this.consumerContainer) && (cls = this.consumerBtnClass)) {
        return $('.btn', con).addClass(cls);
      }
    };

    return QueryTools;

  })(CoreView);

}).call(this);
