(function() {
  var CellModelFactory, Collection, ColumnHeaders, CoreModel, CoreView, ErrorNotice, History, Messages, Options, PageSizer, Pagination, ResultsTable, RowsCollection, SelectedObjects, Table, TableModel, TableResults, TableSummary, Templates, Types, UNKNOWN_ERROR, UniqItems, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../core-view');

  Options = require('../options');

  Templates = require('../templates');

  Messages = require('../messages');

  Collection = require('../core/collection');

  CoreModel = require('../core-model');

  Types = require('../core/type-assertions');

  TableModel = require('../models/table');

  ColumnHeaders = require('../models/column-headers');

  UniqItems = require('../models/uniq-items');

  RowsCollection = require('../models/rows');

  SelectedObjects = require('../models/selected-objects');

  History = require('../models/history');

  CellModelFactory = require('../utils/cell-model-factory');

  TableResults = require('../utils/table-results');

  ResultsTable = require('./table/inner');

  ErrorNotice = require('./table/error-notice');

  Pagination = require('./table/pagination');

  PageSizer = require('./table/page-sizer');

  TableSummary = require('./table/summary');

  require('../messages/table');

  UNKNOWN_ERROR = {
    message: 'Unknown error',
    key: 'error.Unknown'
  };

  module.exports = Table = (function(_super) {
    __extends(Table, _super);

    function Table() {
      this.canUseFormatter = __bind(this.canUseFormatter, this);
      this.unsetSelecting = __bind(this.unsetSelecting, this);
      this.setSelecting = __bind(this.setSelecting, this);
      return Table.__super__.constructor.apply(this, arguments);
    }

    Table.create = function(_arg) {
      var history, model, query, selectedObjects;
      query = _arg.query, model = _arg.model;
      Types.assertMatch(Types.Query, query);
      if (model == null) {
        model = new TableModel;
      }
      history = new History;
      selectedObjects = new SelectedObjects(query.service);
      history.setInitialState(query);
      return new Table({
        history: history,
        model: model,
        selectedObjects: selectedObjects
      });
    };

    Table.prototype.Model = TableModel;

    Table.prototype.className = "im-table-container";

    Table.prototype.parameters = ['history', 'selectedObjects'];

    Table.prototype.optionalParameters = ['model', 'columnHeaders', 'blacklistedFormatters'];

    Table.prototype.parameterTypes = {
      history: Types.InstanceOf(History, 'History'),
      selectedObjects: Types.InstanceOf(SelectedObjects, 'SelectedObjects')
    };

    Table.prototype.cellModelFactory = null;

    Table.prototype.initialize = function() {
      Table.__super__.initialize.apply(this, arguments);
      if (this.columnHeaders == null) {
        this.columnHeaders = new ColumnHeaders;
      }
      if (this.blacklistedFormatters == null) {
        this.blacklistedFormatters = new UniqItems;
      }
      this.rows = new RowsCollection;
      this.listenTo(this.history, 'changed:current', this.onChangeQuery);
      this.listenTo(this.blacklistedFormatters, 'reset add remove', this.buildColumnHeaders);
      this.listenTo(this.columnHeaders, 'change:minimised', this.onChangeHeaderMinimised);
      this.onChangeQuery();
      return console.debug('initialised table');
    };

    Table.prototype.onChangeQuery = function() {
      var model, service, _ref;
      _ref = this.query = this.history.getCurrentQuery(), service = _ref.service, model = _ref.model;
      if (this.cellModelFactory == null) {
        this.cellModelFactory = new CellModelFactory(service, model);
      }
      this.buildColumnHeaders();
      return this.fetchVersion().then((function(_this) {
        return function() {
          _this.query.count(function(error, count) {
            return _this.model.set({
              error: error,
              count: count
            });
          });
          return _this.setFreshness();
        };
      })(this));
    };

    Table.prototype.fetchVersion = function() {
      return this.query.service.fetchVersion().then((function(_this) {
        return function(version) {
          return _this.model.set({
            version: version
          });
        };
      })(this)).then(null, (function(_this) {
        return function(e) {
          return onConnectionError(e);
        };
      })(this));
    };

    Table.prototype.onConnectionError = function(e) {
      var err;
      err = new Error('Could not connect to server');
      err.key = 'error.ConnectionError';
      return this.model.set({
        error: err
      });
    };

    Table.prototype.modelEvents = function() {
      return {
        'change:freshness change:start change:size': this.fillRows,
        'change:start change:size': this.overlayTable,
        'change:fill': this.removeOverlay,
        'change:freshness': this.resetPhase,
        'change:phase': this.onChangePhase,
        'change:error': this.onChangeError,
        'change:count': this.onChangeCount
      };
    };

    Table.prototype.onChangePhase = function() {
      this.removeOverlay();
      return this.reRender();
    };

    Table.prototype.onChangeCount = function() {
      var count, size, start, _ref;
      _ref = this.model.pick('start', 'count', 'size'), start = _ref.start, size = _ref.size, count = _ref.count;
      if (start >= count) {
        return this.model.set({
          start: Math.max(0, count - size)
        });
      }
    };

    Table.prototype.resetPhase = function() {
      return this.model.set({
        phase: 'FETCHING'
      });
    };

    Table.prototype.onChangeError = function() {
      if (this.model.get('error')) {
        return this.model.set({
          phase: 'ERROR'
        });
      }
    };

    Table.prototype.remove = function() {
      this.cellModelFactory.destroy();
      delete this.cellModelFactory;
      return Table.__super__.remove.apply(this, arguments);
    };

    Table.prototype.onChangeHeaderMinimised = function(m) {
      var minimisedCols, path;
      path = this.query.makePath(m.get('path'));
      minimisedCols = this.model.get('minimisedColumns');
      if (m.get('minimised')) {
        return minimisedCols.add(path);
      } else {
        return minimisedCols.remove(path);
      }
    };

    Table.prototype.setSelecting = function() {
      return this.model.set({
        selecting: true
      });
    };

    Table.prototype.unsetSelecting = function() {
      return this.model.set({
        selecting: false
      });
    };

    Table.prototype.canUseFormatter = function(formatter) {
      return (formatter != null) && (!this.blacklistedFormatters.contains(formatter));
    };

    Table.prototype.setFreshness = function() {
      return this.model.set({
        freshness: this.query.toXML()
      });
    };

    Table.prototype.buildColumnHeaders = function() {
      var isMin, minimisedCols, silently;
      silently = {
        silent: true
      };
      minimisedCols = this.model.get('minimisedColumns');
      isMin = (function(_this) {
        return function(ch) {
          return minimisedCols.contains(_this.query.makePath(ch.get('path')));
        };
      })(this);
      this.columnHeaders.setHeaders(this.query, this.blacklistedFormatters);
      return this.columnHeaders.forEach((function(_this) {
        return function(ch) {
          return ch.set({
            minimised: isMin(ch)
          }, silently);
        };
      })(this));
    };

    Table.prototype.fillRows = function() {
      var error, size, start, success, _ref;
      console.debug('filling rows');
      _ref = this.model.pick('start', 'size'), start = _ref.start, size = _ref.size;
      success = (function(_this) {
        return function() {
          return _this.model.set({
            phase: 'SUCCESS'
          });
        };
      })(this);
      error = (function(_this) {
        return function(e) {
          if (e == null) {
            e = UNKNOWN_ERROR;
          }
          return _this.model.set({
            phase: 'ERROR',
            error: e
          });
        };
      })(this);
      return TableResults.getCache(this.query).fetchRows(start, size).then((function(_this) {
        return function(rows) {
          return _this.fillRowsCollection(rows);
        };
      })(this)).then(success, error).then(this.model.filled, this.model.filled);
    };

    Table.prototype.fillRowsCollection = function(rows) {
      var createModel, models, offset;
      createModel = this.cellModelFactory.getCreator(this.query);
      offset = this.model.get('start');
      models = rows.map((function(_this) {
        return function(row, i) {
          var c;
          return {
            id: "" + (_this.query.toXML()) + "#" + (offset + i),
            index: offset + i,
            query: _this.query.toXML(),
            cells: (function() {
              var _i, _len, _results;
              _results = [];
              for (_i = 0, _len = row.length; _i < _len; _i++) {
                c = row[_i];
                _results.push(createModel(c));
              }
              return _results;
            })()
          };
        };
      })(this));
      return this.rows.set(models);
    };

    Table.prototype.overlayTable = function() {
      var elOffset, h1, table, tableOffset, _ref;
      if (!((_ref = this.children.inner) != null ? _ref.rendered : void 0)) {
        return;
      }
      table = this.children.inner.$el;
      elOffset = this.$el.offset();
      tableOffset = table.offset();
      this.removeOverlay();
      this.overlay = document.createElement('div');
      this.overlay.className = 'im-table-overlay im-hidden';
      h1 = this.make('h1', {}, Messages.getText('table.OverlayText'));
      h1.style.top = "" + (table.height() / 2) + "px";
      this.overlay.appendChild(h1);
      this.el.appendChild(this.overlay);
      return _.delay(((function(_this) {
        return function() {
          var _ref1;
          return (_ref1 = _this.overlay) != null ? _ref1.classList.remove('im-hidden') : void 0;
        };
      })(this)), 100);
    };

    Table.prototype.removeOverlay = function() {
      if (this.overlay != null) {
        this.el.removeChild(this.overlay);
      }
      return delete this.overlay;
    };

    Table.prototype.template = function() {
      switch (this.model.get('phase')) {
        case 'FETCHING':
          return this.renderFetching();
        case 'ERROR':
          return this.renderError();
        case 'SUCCESS':
          return this.renderTable();
        default:
          throw new Error("Unknown phase: " + (this.model.get('phase')));
      }
    };

    Table.prototype.renderFetching = function() {
      return Templates.template('table-building')(this.getBaseData());
    };

    Table.prototype.renderError = function() {
      var e;
      this.removeChild('error');
      this.children.error = e = new ErrorNotice({
        query: this.query,
        model: this.model
      });
      return e.render().el;
    };

    Table.prototype.renderTable = function() {
      var frag, table;
      frag = document.createDocumentFragment();
      this.renderWidgets(frag);
      table = new ResultsTable(_.extend(_.pick(this, ResultsTable.prototype.parameters), {
        tableState: this.model
      }));
      this.renderChild('inner', table, frag);
      return frag;
    };

    Table.prototype.renderWidgets = function(container) {
      var clear, method, widgetDiv, widgetName, widgets, _i, _len;
      if (container == null) {
        container = this.el;
      }
      widgets = _.chain(Options.get('TableWidgets')).map(function(_arg, name) {
        var enabled, index;
        enabled = _arg.enabled, index = _arg.index;
        return {
          name: name,
          index: index,
          enabled: enabled
        };
      }).where({
        enabled: true
      }).sortBy('index').pluck('name').value();
      if (widgets.length) {
        widgetDiv = document.createElement('div');
        widgetDiv.className = 'im-table-controls';
        clear = document.createElement('div');
        clear.style.clear = 'both';
        for (_i = 0, _len = widgets.length; _i < _len; _i++) {
          widgetName = widgets[_i];
          if (!(("place" + widgetName) in this)) {
            continue;
          }
          method = "place" + widgetName;
          this[method](widgetDiv);
        }
        widgetDiv.appendChild(clear);
        return container.appendChild(widgetDiv);
      }
    };

    Table.prototype.renderWidget = function(name, container, Child) {
      var component;
      component = new Child({
        model: this.model,
        getQuery: (function(_this) {
          return function() {
            return _this.history.getCurrentQuery();
          };
        })(this)
      });
      return this.renderChild(name, component, container);
    };

    Table.prototype.placePagination = function(widgets) {
      return this.renderWidget('pagination', widgets, Pagination);
    };

    Table.prototype.placePageSizer = function(widgets) {
      return this.renderWidget('pagesizer', widgets, PageSizer);
    };

    Table.prototype.placeTableSummary = function(widgets) {
      return this.renderWidget('tablesummary', widgets, TableSummary);
    };

    return Table;

  })(CoreView);

}).call(this);
