(function() {
  var $, Cell, CellModel, CoreView, Formatting, Messages, Options, SelectedObjects, Templates, compatible, popoverTemplate, types, _, _compatible,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  $ = require('jquery');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Options = require('../../options');

  Messages = require('../../messages');

  CellModel = require('../../models/cell');

  Formatting = require('../../formatting');

  Messages.setWithPrefix('table.cell', {
    Link: 'link',
    NullEntity: 'No <%= type %>',
    PreviewTitle: function(_arg) {
      var types;
      types = _arg.types;
      return types.join(Options.get('DynamicObjects.NameDelimiter'));
    }
  });

  SelectedObjects = require('../../models/selected-objects');

  types = require('../../core/type-assertions');

  popoverTemplate = Templates.template('classy-popover')({
    classes: 'item-preview'
  });

  _compatible = function(path, ct) {
    if (ct != null) {
      return path.model.findSharedAncestor(path, ct) != null;
    } else {
      return false;
    }
  };

  compatible = _.memoize(_compatible, function(p, ct) {
    return "" + p + "<" + (ct != null ? ct : '!');
  });

  module.exports = Cell = (function(_super) {
    __extends(Cell, _super);

    function Cell() {
      return Cell.__super__.constructor.apply(this, arguments);
    }

    Cell.prototype.Model = CellModel;

    Cell.prototype.tagName = 'td';

    Cell.prototype.className = 'im-result-field';

    Cell.prototype.id = function() {
      return _.uniqueId('im_table_cell_');
    };

    Cell.prototype.formatter = Formatting.defaultFormatter;

    Cell.prototype.parameters = ['model', 'service', 'selectedObjects', 'tableState', 'popovers'];

    Cell.prototype.optionalParameters = ['formatter'];

    Cell.prototype.parameterTypes = {
      model: types.InstanceOf(CellModel, 'models/cell'),
      selectedObjects: types.InstanceOf(SelectedObjects, 'SelectedObjects'),
      formatter: types.Callable,
      tableState: types.Model,
      service: types.Service,
      popovers: new types.Structure('HasGet', {
        get: types.Function
      })
    };

    Cell.prototype.initialize = function() {
      Cell.__super__.initialize.apply(this, arguments);
      return this.listen();
    };

    Cell.prototype.initState = function() {
      this.setSelected();
      this.setSelectable();
      this.setHighlit();
      return this.setMinimised();
    };

    Cell.prototype.getPath = function() {
      return this.model.get('column');
    };

    Cell.prototype.getTypes = function() {
      var c, classes, node, _i, _len, _results;
      node = this.model.get('node');
      classes = this.model.get('entity').get('classes');
      if (classes != null) {
        _results = [];
        for (_i = 0, _len = classes.length; _i < _len; _i++) {
          c = classes[_i];
          _results.push(node.model.makePath(c));
        }
        return _results;
      } else {
        return [node];
      }
    };

    Cell.prototype.listen = function() {
      this.listenToEntity();
      this.listenToSelectedObjects();
      this.listenToOptions();
      return this.listenToTableState();
    };

    Cell.prototype.listenToOptions = function() {
      this.listenTo(Options, 'change:TableCell.IndicateOffHostLinks', this.reRender);
      this.listenTo(Options, 'change:TableCell.ExternalLinkIcons', this.reRender);
      return this.listenTo(Options, 'change:TableCell.PreviewTrigger', this.onChangeTrigger);
    };

    Cell.prototype.listenToTableState = function() {
      var ts;
      ts = this.tableState;
      this.listenTo(ts, 'change:selecting', this.setInputDisplay);
      this.listenTo(ts, 'change:previewOwner', this.closeOwnPreview);
      this.listenTo(ts, 'change:highlitNode', this.setHighlit);
      return this.listenTo(ts, 'change:minimisedColumns', this.setMinimised);
    };

    Cell.prototype.listenToSelectedObjects = function() {
      var objs;
      objs = this.selectedObjects;
      this.listenTo(objs, 'add remove reset', this.setSelected);
      return this.listenTo(objs, 'add remove reset change:commonType change:node', this.setSelectable);
    };

    Cell.prototype.modelEvents = function() {
      return {
        'change:entity': this.onChangeEntity
      };
    };

    Cell.prototype.stateEvents = function() {
      return {
        'change:highlit change:selected': this.setActiveClass,
        'change:selectable': this.onChangeSelectable,
        'change:selected': this.setInputChecked,
        'change:showPreview': this.onChangeShowPreview,
        'change:minimised': this.reRender
      };
    };

    Cell.prototype.events = function() {
      var events, opts, trigger;
      events = {
        'show.bs.popover': this.onShowPreview,
        'hide.bs.popover': this.onHidePreview,
        'click a.im-cell-link': this.onClickCellLink
      };
      opts = Options.get('TableCell');
      trigger = opts.PreviewTrigger;
      if (trigger === 'hover') {
        events['mouseover .im-cell-link'] = this.showPreview;
        events['mouseout .im-cell-link'] = this.hidePreview;
        events['click'] = this.activateChooser;
      } else if (trigger === 'click') {
        events['click'] = this.clickTogglePreview;
      } else {
        throw new Error("Unknown cell preview: " + trigger);
      }
      return events;
    };

    Cell.prototype.onChangeTrigger = function() {
      this.destroyPreview();
      this.initPreview();
      return this.delegateEvents();
    };

    Cell.prototype.onClickCellLink = function(e) {
      var viewEvent;
      if (this.tableState.get('selecting')) {
        e.preventDefault();
        return;
      }
      viewEvent = $.Event('view.im.object');
      viewEvent.object = this.model.get('entity').toJSON();
      this.$el.trigger(viewEvent);
      if (viewEvent.isDefaultPrevented()) {
        return e.preventDefault();
      }
    };

    Cell.prototype.closeOwnPreview = function() {
      var currentOwner, myId;
      myId = this.el.id;
      currentOwner = this.tableState.get('previewOwner');
      if (myId !== currentOwner) {
        return this.hidePreview();
      }
    };

    Cell.prototype.listenToEntity = function() {
      return this.listenTo(this.model.get('entity'), 'change', this.updateValue);
    };

    Cell.prototype.onShowPreview = function() {
      return this.tableState.set({
        previewOwner: this.el.id
      });
    };

    Cell.prototype.onHidePreview = function() {
      var currentOwner, myId;
      myId = this.el.id;
      currentOwner = this.tableState.get('previewOwner');
      if (myId === currentOwner) {
        return this.tableState.set({
          previewOwner: null
        });
      }
    };

    Cell.prototype.toggleSelection = function() {
      var ent, found, id, toAdd;
      ent = this.model.get('entity');
      id = ent.get('id');
      if (ent == null) {
        return;
      }
      if (found = this.selectedObjects.get(id)) {
        return this.selectedObjects.remove(found);
      } else {
        toAdd = this.getMostSuitableIdentity();
        if (toAdd == null) {
          throw new Error("None of our identities is a compatible");
        }
        return this.selectedObjects.add(toAdd);
      }
    };

    Cell.prototype.getMostSuitableIdentity = function() {
      var columnTypes, ct, id, identities, node, ranked, suitable, t;
      node = this.model.get('node');
      columnTypes = node.model.getSubclassesOf(node.getType());
      id = this.model.get('entity').get('id');
      identities = (function() {
        var _i, _len, _ref, _results;
        _ref = this.getTypes();
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          t = _ref[_i];
          _results.push({
            'class': String(t),
            id: id
          });
        }
        return _results;
      }).call(this);
      if (this.selectedObjects.isEmpty()) {
        return _.find(identities, function(x) {
          var _ref;
          return _ref = x['class'], __indexOf.call(columnTypes, _ref) >= 0;
        });
      } else {
        ct = node.model.makePath(this.selectedObjects.state.get('commonType'));
        suitable = _.all(identities, function(x) {
          return compatible(ct, x['class']);
        });
        ranked = _.sortBy(identities, function(x) {
          return node.model.getAncestorsOf(x['class']).length;
        });
        return _.last(ranked);
      }
    };

    Cell.prototype.setSelected = function() {
      return this.state.set({
        'selected': this.selectedObjects.get(this.model.get('entity')) != null
      });
    };

    Cell.prototype.setSelectable = function() {
      var commonType, node, selectable, size;
      commonType = this.selectedObjects.state.get('commonType');
      node = this.selectedObjects.state.get('node');
      size = this.selectedObjects.size();
      selectable = ((node == null) || (node === String(this.model.get('node')))) && ((size === 0) || (this.isCompatibleWith(commonType)));
      return this.state.set({
        selectable: selectable
      });
    };

    Cell.prototype.isCompatibleWith = function(commonType) {
      return this.getTypes().some(function(t) {
        return compatible(t, commonType);
      });
    };

    Cell.prototype.setHighlit = function() {
      var highlit, myNode;
      myNode = this.model.get('node');
      highlit = this.tableState.get('highlitNode');
      return this.state.set({
        highlit: (highlit != null) && (String(myNode) === String(highlit))
      });
    };

    Cell.prototype.setMinimised = function() {
      var minimised, myColumn;
      myColumn = this.model.get('column');
      minimised = this.tableState.get('minimisedColumns').contains(myColumn);
      return this.state.set({
        minimised: minimised
      });
    };

    Cell.prototype.onChangeEntity = function() {
      var prev;
      prev = this.model.previous('entity');
      delete this._has_id;
      if (prev != null) {
        this.stopListening(prev);
      }
      return this.listenToEntity();
    };

    Cell.prototype.clickTogglePreview = function() {
      return this.activateChooser() || this.state.toggle('showPreview');
    };

    Cell.prototype.activateChooser = function() {
      var selectable, selecting;
      if (!this.hasID()) {
        return;
      }
      selecting = this.tableState.get('selecting');
      selectable = this.state.get('selectable');
      if (selectable && selecting) {
        return this.toggleSelection();
      }
    };

    Cell.prototype.hasID = function() {
      return this._has_id != null ? this._has_id : this._has_id = this.model.get('entity').get('id') != null;
    };

    Cell.prototype.showPreview = function() {
      return this.state.set({
        showPreview: true
      });
    };

    Cell.prototype.hidePreview = function() {
      return this.state.set({
        showPreview: false
      });
    };

    Cell.prototype._showPreview = function() {
      var opts, show;
      if (this.rendered) {
        if (this.tableState.get('selecting')) {
          return;
        }
        opts = Options.get('TableCell');
        show = (function(_this) {
          return function() {
            var _ref;
            if (_this.state.get('showPreview')) {
              return (_ref = _this.children.popover) != null ? _ref.render() : void 0;
            }
          };
        })(this);
        if (opts.PreviewTrigger === 'hover') {
          return _.delay(show, opts.HoverDelay);
        } else {
          return show();
        }
      }
    };

    Cell.prototype._hidePreview = function() {
      var _ref, _ref1;
      if ((_ref = this.children.popover) != null ? _ref.rendered : void 0) {
        return (_ref1 = this.popoverTarget) != null ? _ref1.popover('hide') : void 0;
      }
    };

    Cell.prototype.onChangeShowPreview = function() {
      if (this.state.get('showPreview')) {
        return this._showPreview();
      } else {
        return this._hidePreview();
      }
    };

    Cell.prototype.onChangeSelectable = function() {
      this.setDisabledCellClass();
      return this.setInputDisabled();
    };

    Cell.prototype.updateValue = function() {
      return _.defer((function(_this) {
        return function() {
          return _this.$('.im-displayed-value').html(_this.getFormattedValue());
        };
      })(this));
    };

    Cell.prototype.setActiveClass = function() {
      var highlit, selected, _ref;
      _ref = this.state.pick('highlit', 'selected'), highlit = _ref.highlit, selected = _ref.selected;
      return this.$el.toggleClass('active', highlit || selected);
    };

    Cell.prototype.setInputChecked = function() {
      return this.$('input').prop({
        checked: this.getInputState().checked
      });
    };

    Cell.prototype.setInputDisplay = function() {
      return this.$('input').css({
        display: this.getInputState().display
      });
    };

    Cell.prototype.setDisabledCellClass = function() {
      return this.$el.toggleClass('im-not-selectable', !this.state.get('selectable'));
    };

    Cell.prototype.setInputDisabled = function() {
      return this.$('input').prop({
        disabled: this.getInputState().disabled
      });
    };

    Cell.prototype.template = Templates.template('table-cell');

    Cell.prototype.getData = function() {
      var data, domain, host, opts, reportUri, url, _ref;
      opts = Options.get('TableCell');
      host = opts.IndicateOffHostLinks ? global.location.host : /.*/;
      data = this.model.toJSON();
      data.formattedValue = this.getFormattedValue();
      data.input = this.getInputState();
      data.icon = null;
      data.url = reportUri = data.entity['report:uri'];
      data.isForeign = reportUri && !reportUri.match(host);
      data.target = data.isForeign ? '_blank' : '';
      data.NULL_VALUE = Templates.null_value;
      _ref = opts.ExternalLinkIcons;
      for (domain in _ref) {
        url = _ref[domain];
        if (reportUri != null ? reportUri.match(domain) : void 0) {
          if (data.icon == null) {
            data.icon = url;
          }
        }
      }
      return _.extend(this.getBaseData(), data);
    };

    Cell.prototype.getFormattedValue = function() {
      var entity, value, _ref;
      _ref = this.model.pick('entity', 'value'), entity = _ref.entity, value = _ref.value;
      if (value != null) {
        return this.formatter.call(null, entity, this.service, value);
      } else {
        return null;
      }
    };

    Cell.prototype.getInputState = function() {
      var checked, disabled, display, selectable, selected, selecting, _ref;
      selecting = this.tableState.get('selecting');
      _ref = this.state.pick('selected', 'selectable'), selected = _ref.selected, selectable = _ref.selectable;
      checked = selected;
      disabled = !selectable;
      display = selecting ? 'inline' : 'none';
      return {
        checked: checked,
        disabled: disabled,
        display: display
      };
    };

    Cell.prototype.canHavePreview = function() {
      return this.hasID() && (!this.state.get('minimised'));
    };

    Cell.prototype.postRender = function() {
      this.setAttrClass();
      this.setActiveClass();
      this.setMinimisedClass();
      this.setDisabledCellClass();
      this.setFormatterClasses();
      return this.initPreview();
    };

    Cell.prototype.setAttrClass = function() {
      var attrType, entType, _i, _len, _ref;
      if (Options.get('TableCell.AddDataClasses')) {
        attrType = this.model.get('column').getType();
        this.$el.addClass('im-type-' + attrType.toLowerCase());
        _ref = this.getTypes();
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          entType = _ref[_i];
          this.$el.addClass(String(entType));
        }
        return this.$el.addClass(this.model.get('column').end.name);
      }
    };

    Cell.prototype.setMinimisedClass = function() {
      return this.$el.toggleClass('im-minimised', this.state.get('minimised'));
    };

    Cell.prototype.setFormatterClasses = function() {
      var cls, target;
      cls = this.formatter.classes;
      if (cls != null) {
        this.$el.addClass(cls);
      }
      target = this.formatter.target;
      if (target != null) {
        return this.$el.addClass(target);
      }
    };

    Cell.prototype.getPreviewContainer = function() {
      var candidates, con;
      con = [];
      candidates = ['.im-query-results', '.im-table-container', '.panel', 'table', 'body'];
      while (candidates.length && (con.length === 0)) {
        con = this.$el.closest(candidates.shift());
      }
      return con;
    };

    Cell.prototype.removeAllChildren = function() {
      this.destroyPreview();
      if (this.children.popover != null) {
        this.stopListeningTo(this.children.popover);
        if (this.children.popover.rendered) {
          this.$el.popover('destroy');
        }
      }
      return Cell.__super__.removeAllChildren.apply(this, arguments);
    };

    Cell.prototype.destroyPreview = function() {
      var _ref;
      if (this.children.popover != null) {
        this.stopListeningTo(this.children.popover);
        if (this.children.popover.rendered) {
          if ((_ref = this.popoverTarget) != null) {
            _ref.popover('destroy');
          }
        }
        delete this.popoverTarget;
        return this.removeChild('popover');
      }
    };

    Cell.prototype.popoverTarget = null;

    Cell.prototype.initPreview = function() {
      var content, getTitle, trigger;
      if ((this.children.popover != null) || (!this.canHavePreview())) {
        return;
      }
      content = this.popovers.get(this.model.get('entity'));
      trigger = Options.get('TableCell.PreviewTrigger');
      this.popoverTarget = trigger === 'hover' ? this.$('.im-cell-link') : this.$el;
      getTitle = Messages.getText.bind(Messages, 'table.cell.PreviewTitle');
      this.listenToOnce(content, 'rendered', (function(_this) {
        return function() {
          var container;
          container = _this.getPreviewContainer();
          return _this.popoverTarget.popover({
            trigger: 'manual',
            template: popoverTemplate,
            placement: 'auto left',
            container: container,
            html: true,
            title: function() {
              return getTitle({
                types: _this.model.get('typeNames')
              });
            },
            content: content.el
          });
        };
      })(this));
      this.listenTo(content, 'rendered', (function(_this) {
        return function() {
          return _this.popoverTarget.popover('show');
        };
      })(this));
      return this.children.popover = content;
    };

    return Cell;

  })(CoreView);

}).call(this);
