(function() {
  var CoreView, ErrorNotice, Messages, Options, Templates, VERSION, getDomain, imjs, indentXML, mailto, withPrettyPrintOne, withResource, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  imjs = require('imjs');

  Options = require('../../options');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Messages = require('../../messages');

  indentXML = require('../../utils/indent-xml');

  mailto = require('../../utils/mailto');

  withResource = require('../../utils/with-cdn-resource');

  VERSION = require('../../version');

  require('../../messages/error');

  withPrettyPrintOne = _.partial(withResource, 'prettify', 'prettyPrintOne');

  getDomain = function(err) {
    if (/(Type|Reference)Error/.test(String(err))) {
      return 'client';
    } else {
      return 'server';
    }
  };

  module.exports = ErrorNotice = (function(_super) {
    __extends(ErrorNotice, _super);

    function ErrorNotice() {
      return ErrorNotice.__super__.constructor.apply(this, arguments);
    }

    ErrorNotice.prototype.className = 'im-error-notice';

    ErrorNotice.prototype.parameters = ['model', 'query'];

    ErrorNotice.prototype.helpers = function() {
      return _.extend(ErrorNotice.__super__.helpers.apply(this, arguments), {
        indent: indentXML
      });
    };

    ErrorNotice.prototype.template = Templates.template('table-error');

    ErrorNotice.prototype.getData = function() {
      var address, domain, err, href, subject, time;
      err = this.model.get('error');
      time = new Date();
      subject = Messages.getText('error.mail.Subject');
      address = this.query.service.help;
      domain = getDomain(err);
      console.error(err);
      href = mailto.href(address, subject, "We encountered an error running a query from an\nembedded result table.\n\npage:       " + global.location + "\nservice:    " + this.query.service.root + "\nerror:      " + err + "\ndate-stamp: " + time + "\n\n-------------------------------\nIMJS:       " + imjs.VERSION + "\n-------------------------------\nIMTABLES:   " + VERSION + "\n-------------------------------\nQUERY:      " + (this.query.toXML()) + "\n-------------------------------\nSTACK:      " + (err != null ? err.stack : void 0));
      return _.extend(ErrorNotice.__super__.getData.apply(this, arguments), {
        domain: domain,
        mailto: href,
        query: this.query.toXML()
      });
    };

    ErrorNotice.prototype.postRender = function() {
      var pre, query;
      query = indentXML(this.query.toXML());
      pre = this.$('.query-xml');
      return withPrettyPrintOne(function(ppo) {
        return pre.html(ppo(_.escape(query)));
      });
    };

    ErrorNotice.prototype.events = function() {
      return {
        'click .im-show-query': function() {
          this.$('.query-xml').slideToggle();
          return this.$('.im-show-query').toggleClass('active');
        },
        'click .im-show-error': function() {
          this.$('.error-message').slideToggle();
          return this.$('.im-show-error').toggleClass('active');
        }
      };
    };

    return ErrorNotice;

  })(CoreView);

}).call(this);
