(function() {
  var ClassSet, Collection, ColumnHeader, CoreView, DropDownColumnSummary, FormattedSorting, HeaderModel, Messages, ModelReader, Options, OuterJoinDropDown, SingleColumnConstraints, Templates, getViewPortHeight, getViewPortWidth, ignore, onChange, sortQueryByPath, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __slice = [].slice;

  _ = require('underscore');

  CoreView = require('../../core-view');

  Collection = require('../../core/collection');

  Templates = require('../../templates');

  Messages = require('../../messages');

  Options = require('../../options');

  ClassSet = require('../../utils/css-class-set');

  sortQueryByPath = require('../../utils/sort-query-by-path');

  onChange = require('../../utils/on-change');

  HeaderModel = require('../../models/header');

  FormattedSorting = require('../formatted-sorting');

  SingleColumnConstraints = require('../constraints/single-column');

  DropDownColumnSummary = require('./column-summary');

  OuterJoinDropDown = require('./outer-join-summary');

  ignore = require('../../utils/events').ignore;

  require('../../messages/table');

  ModelReader = function(model, attr) {
    return function() {
      return model.get(attr);
    };
  };

  getViewPortHeight = function() {
    return Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
  };

  getViewPortWidth = function() {
    return Math.max(document.documentElement.clientWidth, window.innerWidth || 0);
  };

  module.exports = ColumnHeader = (function(_super) {
    __extends(ColumnHeader, _super);

    function ColumnHeader() {
      this.toggleColumnVisibility = __bind(this.toggleColumnVisibility, this);
      this.showFilterSummary = __bind(this.showFilterSummary, this);
      this.showColumnSummary = __bind(this.showColumnSummary, this);
      this.showSummary = __bind(this.showSummary, this);
      return ColumnHeader.__super__.constructor.apply(this, arguments);
    }

    ColumnHeader.prototype.Model = HeaderModel;

    ColumnHeader.prototype.tagName = 'th';

    ColumnHeader.prototype.className = 'im-column-th';

    ColumnHeader.prototype.RERENDER_EVENT = onChange(['composed', 'expanded', 'minimised', 'numOfCons', 'outerJoined', 'parts', 'path', 'sortable', 'sortDirection']);

    ColumnHeader.prototype.template = Templates.template('column-header');

    ColumnHeader.prototype.namePopoverTemplate = Templates.template('column_name_popover');

    ColumnHeader.prototype.parameters = ['query'];

    ColumnHeader.prototype.optionalParameters = ['blacklistedFormatters', 'expandedSubtables'];

    ColumnHeader.prototype.expandedSubtables = new Collection;

    ColumnHeader.prototype.blacklistedFormatters = new Collection;

    ColumnHeader.prototype.initialize = function() {
      ColumnHeader.__super__.initialize.apply(this, arguments);
      this.listenTo(this.query, 'showing:column-summary', this.removeMySummary);
      this.listenTo(Options, 'change:icons', this.reRender);
      return this.createClassSets();
    };

    ColumnHeader.prototype.path = function() {
      return this.model.pathInfo();
    };

    ColumnHeader.prototype.createClassSets = function() {
      this.classSets = {};
      this.classSets.headerClasses = new ClassSet({
        'im-column-header': true,
        'im-minimised-th': ModelReader(this.model, 'minimised'),
        'im-is-composed': ModelReader(this.model, 'composed'),
        'im-has-constraint': ModelReader(this.model, 'numOfCons')
      });
      return this.classSets.colTitleClasses = new ClassSet({
        'im-col-title': true,
        'im-hidden': ModelReader(this.model, 'minimised')
      });
    };

    ColumnHeader.prototype.removeMySummary = function(path) {
      if (!path.equals(this.model.pathInfo())) {
        this.removeChild('summary');
        return this.$('.dropdown.open').removeClass('open');
      }
    };

    ColumnHeader.prototype.onSubtableExpanded = function(node) {
      if (node.toString().match(this.model.getView())) {
        return this.model.set({
          expanded: true
        });
      }
    };

    ColumnHeader.prototype.onSubtableCollapsed = function(node) {
      if (node.toString().match(this.model.getView())) {
        return this.model.set({
          expanded: false
        });
      }
    };

    ColumnHeader.prototype.getData = function() {
      var ancestors, hasAncestors, last, parentType, parts, penult, penultClasses, _i, _ref;
      _ref = parts = this.model.get('parts'), ancestors = 3 <= _ref.length ? __slice.call(_ref, 0, _i = _ref.length - 2) : (_i = 0, []), penult = _ref[_i++], last = _ref[_i++];
      hasAncestors = ancestors.length;
      parentType = hasAncestors ? 'non-root' : 'root';
      penultClasses = new ClassSet({
        'im-title-part im-parent': true,
        'im-root-parent': !hasAncestors,
        'im-non-root-parent': hasAncestors,
        'im-last': !last
      });
      return _.extend({
        penultClasses: penultClasses,
        last: last,
        penult: penult
      }, this.classSets, ColumnHeader.__super__.getData.apply(this, arguments));
    };

    ColumnHeader.prototype.events = function() {
      return {
        'click .im-col-sort': 'setSortOrder',
        'click .im-col-minumaximiser': 'toggleColumnVisibility',
        'click .im-col-filters': 'showFilterSummary',
        'click .im-subtable-expander': this.toggleSubTable,
        'click .im-col-remover': 'removeColumn',
        'hidden.bs.dropdown': function() {
          return this.removeChild('summary');
        },
        'shown.bs.dropdown': 'onDropdownShown',
        'toggle .im-th-button': 'summaryToggled',
        'click .summary-img': this.showColumnSummary,
        'click .im-col-composed': this.addFormatterToBlacklist
      };
    };

    ColumnHeader.prototype.postRender = function() {
      this.setTitlePopover();
      this.announceExpandedState();
      return this.activateTooltips();
    };

    ColumnHeader.prototype.activateTooltips = function() {
      return this.$('.im-th-button').tooltip({
        placement: 'top',
        container: this.el
      });
    };

    ColumnHeader.prototype.onDropdownShown = function(e) {
      return _.defer(function() {
        var ddRect, right;
        delete e.target.style.right;
        ddRect = e.target.getBoundingClientRect();
        if (ddRect.left < 0) {
          right = "" + ddRect.left + "px";
          return e.target.style.right = right;
        }
      });
    };

    ColumnHeader.prototype.addFormatterToBlacklist = function() {
      return this.blacklistedFormatters.add({
        formatter: this.model.get('formatter')
      });
    };

    ColumnHeader.prototype.announceExpandedState = function() {
      if (this.model.get('expanded')) {
        return this.query.trigger('expand:subtables', this.model.get('path'));
      }
    };

    ColumnHeader.prototype.setTitlePopover = function() {
      if (Options.get('TableHeader.FullPathPopoverEnabled')) {
        return this.$('.im-col-title').popover({
          content: (function(_this) {
            return function() {
              return _this.namePopoverTemplate(_this.getData());
            };
          })(this),
          container: this.el,
          placement: 'bottom',
          html: true
        });
      }
    };

    ColumnHeader.prototype.summaryToggled = function(e, isOpen) {
      ignore(e);
      if (e.target !== e.currentTarget) {
        return;
      }
      if (!isOpen) {
        return this.removeChild('summary');
      }
    };

    ColumnHeader.prototype.hideTooltips = function() {
      return this.$('.im-th-button').tooltip('hide');
    };

    ColumnHeader.prototype.removeColumn = function(e) {
      var v, view;
      ignore(e);
      this.hideTooltips();
      view = this.model.getView();
      return this.query.removeFromSelect((function() {
        var _i, _len, _ref, _results;
        _ref = this.query.views;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          v = _ref[_i];
          if (v.match(view)) {
            _results.push(v);
          }
        }
        return _results;
      }).call(this));
    };

    ColumnHeader.prototype.checkHowFarOver = function(el) {
      var bounds;
      bounds = this.$el.closest('.im-table-container');
      if ((el.offset().left + 350) >= (bounds.offset().left + bounds.width())) {
        return this.$el.addClass('too-far-over');
      }
    };

    ColumnHeader.prototype.showSummary = function(selector, View, e) {
      var $menu, $sel, path, summary, _ref;
      ignore(e);
      $sel = this.$(selector);
      path = this.path();
      if ($sel.hasClass('open')) {
        this.query.trigger('hiding:column-summary', path);
        $sel.removeClass('open');
        if ((_ref = this.children.summary) != null) {
          _ref.$el.hide();
        }
        this.removeChild('summary');
        return false;
      } else {
        this.$('.dropdown.open').removeClass('open');
        this.query.trigger('showing:column-summary', path);
        summary = new View({
          query: this.query,
          model: this.model
        });
        $menu = this.$(selector + ' .dropdown-menu');
        if (!$menu.length) {
          throw new Error("" + selector + " not found");
        }
        $menu.empty();
        this.renderChild('summary', summary, $menu);
        $sel.addClass('open');
        this.onDropdownShown({
          target: $menu[0]
        });
        return true;
      }
    };

    ColumnHeader.prototype.ensureDropdownIsWithinTable = function(target, selector, minWidth) {
      var $menu, elRect, h, table, tableRect;
      if (minWidth == null) {
        minWidth = 360;
      }
      elRect = target.getBoundingClientRect();
      table = this.$el.closest('table')[0];
      if (table == null) {
        return;
      }
      h = getViewPortWidth();
      $menu = this.$(selector + ' .dropdown-menu');
      if (minWidth >= h) {
        return $menu.addClass('im-fullwidth-dropdown').css({
          width: h
        });
      } else {
        $menu.css({
          width: null
        });
      }
      if (minWidth >= getViewPortWidth()) {
        return;
      }
      tableRect = table.getBoundingClientRect();
      if (elRect.left + minWidth > tableRect.right) {
        return this.$(selector + ' .dropdown-menu').addClass('dropdown-menu-right');
      }
    };

    ColumnHeader.prototype.showColumnSummary = function(e) {
      var cls, h, shown;
      cls = this.model.get('isReference') ? OuterJoinDropDown : DropDownColumnSummary;
      if (shown = this.showSummary('.im-summary', cls, e)) {
        h = getViewPortHeight();
        this.$('.im-item-table').css({
          'max-height': Math.max(350, h / 2)
        });
        return this.ensureDropdownIsWithinTable(e.target, '.im-summary');
      }
    };

    ColumnHeader.prototype.showFilterSummary = function(e) {
      var shown;
      if (shown = this.showSummary('.im-filter-summary', SingleColumnConstraints, e)) {
        return this.ensureDropdownIsWithinTable(e.target, '.im-filter-summary', 500);
      }
    };

    ColumnHeader.prototype.toggleColumnVisibility = function(e) {
      ignore(e);
      return this.model.toggle('minimised');
    };

    ColumnHeader.prototype.setSortOrder = function(e) {
      ignore(e);
      if (this.model.get('replaces').length) {
        return this.showSummary('.im-col-sort', FormattedSorting, e);
      } else {
        sortQueryByPath(this.query, this.model.getView());
        return this.$('.im-col-sort').removeClass('open');
      }
    };

    ColumnHeader.prototype.toggleSubTable = function(e) {
      ignore(e);
      return this.expandedSubtables.toggle(this.model.pathInfo());
    };

    return ColumnHeader;

  })(CoreView);

}).call(this);
