(function() {
  var CellFactory, ColumnHeaders, CoreView, Formatting, History, PathSet, PopoverFactory, Preview, ResultsTable, SelectedObjects, TableBody, TableHead, TableModel, Templates, Types, flip, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Formatting = require('../../formatting');

  PathSet = require('../../models/path-set');

  ColumnHeaders = require('../../models/column-headers');

  PopoverFactory = require('../../utils/popover-factory');

  History = require('../../models/history');

  TableModel = require('../../models/table');

  SelectedObjects = require('../../models/selected-objects');

  Preview = require('../item-preview');

  Types = require('../../core/type-assertions');

  CellFactory = require('./cell-factory');

  TableBody = require('./body');

  TableHead = require('./head');

  flip = function(f) {
    return function(x, y) {
      return f(y, x);
    };
  };

  module.exports = ResultsTable = (function(_super) {
    __extends(ResultsTable, _super);

    function ResultsTable() {
      this.canUseFormatter = __bind(this.canUseFormatter, this);
      return ResultsTable.__super__.constructor.apply(this, arguments);
    }

    ResultsTable.prototype.className = "im-results-table table table-striped table-bordered";

    ResultsTable.prototype.tagName = 'table';

    ResultsTable.prototype.throbber = Templates.template('table-throbber');

    ResultsTable.prototype.parameters = ['history', 'columnHeaders', 'rows', 'tableState', 'blacklistedFormatters', 'selectedObjects'];

    ResultsTable.prototype.parameterTypes = {
      history: Types.InstanceOf(History, 'History'),
      blacklistedFormatters: Types.Collection,
      rows: Types.Collection,
      tableState: Types.InstanceOf(TableModel, 'TableModel'),
      columnHeaders: Types.InstanceOf(ColumnHeaders, 'ColumnHeaders'),
      selectedObjects: Types.InstanceOf(SelectedObjects, 'SelectedObjects')
    };

    ResultsTable.prototype.initialize = function() {
      var service;
      ResultsTable.__super__.initialize.apply(this, arguments);
      service = (this.query = this.history.getCurrentQuery()).service;
      this.expandedSubtables = new PathSet;
      this.popoverFactory = new PopoverFactory(service, Preview);
      this.cellFactory = CellFactory(service, this);
      this.listenTo(this.blacklistedFormatters, 'reset add remove', this.renderBody);
      return this.listenTo(this.history, 'changed:current', this.setQuery);
    };

    ResultsTable.prototype.setQuery = function() {
      return this.query = this.history.getCurrentQuery();
    };

    ResultsTable.prototype.getFormatter = flip(Formatting.getFormatter);

    ResultsTable.prototype.canUseFormatter = function(formatter) {
      return (formatter != null) && (!this.blacklistedFormatters.findWhere({
        formatter: formatter
      }));
    };

    ResultsTable.prototype.renderChildren = function() {
      this.renderHeaders();
      return this.renderBody();
    };

    ResultsTable.prototype.renderHeaders = function() {
      return this.renderChild('head', new TableHead(_.pick(this, TableHead.prototype.parameters)));
    };

    ResultsTable.prototype.renderBody = function() {
      return this.renderChild('body', new TableBody({
        collection: this.rows,
        history: this.history,
        makeCell: this.cellFactory
      }));
    };

    ResultsTable.prototype.remove = function() {
      this.expandedSubtables.close();
      delete this.expandedSubtables;
      this.popoverFactory.destroy();
      delete this.popoverFactory;
      delete this.cellFactory;
      return ResultsTable.__super__.remove.apply(this, arguments);
    };

    return ResultsTable;

  })(CoreView);

}).call(this);
