(function() {
  var CoreView, Events, ExportDialogue, HasDialogues, LargeTableDisuader, NewFilterDialogue, PageSizer, Paging, Templates, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Paging = require('./paging');

  HasDialogues = require('../has-dialogues');

  CoreView = require('../../core-view');

  Templates = require('../../templates');

  Events = require('../../events');

  NewFilterDialogue = require('../new-filter-dialogue');

  ExportDialogue = require('../export-dialogue');

  LargeTableDisuader = require('./large-table-disuader');

  module.exports = PageSizer = (function(_super) {
    __extends(PageSizer, _super);

    function PageSizer() {
      return PageSizer.__super__.constructor.apply(this, arguments);
    }

    PageSizer.include(Paging);

    PageSizer.include(HasDialogues);

    PageSizer.prototype.tagName = 'form';

    PageSizer.prototype.className = "im-page-sizer im-table-control form-inline";

    PageSizer.prototype.sizes = [[10], [25], [50], [100], [250]];

    PageSizer.prototype.parameters = ['getQuery'];

    PageSizer.prototype.initialize = function() {
      var s, size;
      PageSizer.__super__.initialize.apply(this, arguments);
      size = this.model.get('size');
      if ((size != null) && __indexOf.call((function() {
        var _i, _len, _ref, _results;
        _ref = this.sizes;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          s = _ref[_i][0];
          _results.push(s);
        }
        return _results;
      }).call(this), size) < 0) {
        return this.sizes = [[size, size]].concat(this.sizes);
      }
    };

    PageSizer.prototype.events = function() {
      return {
        'submit': Events.suppress,
        'change select': 'changePageSize'
      };
    };

    PageSizer.prototype.modelEvents = function() {
      return {
        'change:size': function(m, v) {
          return this.$('select').val(v);
        },
        'change:count': this.reRender
      };
    };

    PageSizer.prototype.changePageSize = function(evt) {
      var accept, input, oldSize, pending, size;
      input = this.$(evt.target);
      size = parseInt(input.val(), 10);
      oldSize = this.model.get('size');
      accept = ((function(_this) {
        return function() {
          return _this.model.set({
            size: size
          });
        };
      })(this));
      if (!this.aboveSizeThreshold(size)) {
        return accept();
      }
      pending = this.whenAcceptable(size).then((function(_this) {
        return function(action) {
          if (action === 'accept') {
            return accept();
          }
          input.val(oldSize);
          switch (action) {
            case 'back':
              return _this.goBack(1);
            case 'forward':
              return _this.goForward(1);
            case 'constrain':
              return _this.openDialogue(new NewFilterDialogue({
                query: _this.getQuery()
              }));
            case 'export':
              return _this.openDialogue(ExportDialogue({
                query: _this.getQuery()
              }));
            default:
              return console.debug('dismissed dialogue', action);
          }
        };
      })(this));
      return pending.then(null, function(e) {
        return console.error('Error handling dialogues', e);
      });
    };

    PageSizer.prototype.whenAcceptable = function(size) {
      return this.openDialogue(new LargeTableDisuader({
        model: {
          size: size
        }
      }));
    };

    PageSizer.prototype.template = Templates.template('page_sizer');

    PageSizer.prototype.getData = function() {
      var count, s, size, sizes;
      count = this.model.get('count');
      size = this.model.get('size');
      sizes = (function() {
        var _i, _len, _ref, _results;
        _ref = this.sizes;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          s = _ref[_i];
          if ((count == null) || (s[0] < count)) {
            _results.push(s);
          }
        }
        return _results;
      }).call(this);
      if (sizes.length && __indexOf.call((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = sizes.length; _i < _len; _i++) {
          s = sizes[_i][0];
          _results.push(s);
        }
        return _results;
      })(), size) < 0) {
        sizes.push([size, size]);
      }
      return {
        size: size,
        sizes: sizes
      };
    };

    PageSizer.prototype.pageSizeFeasibilityThreshold = 250;

    PageSizer.prototype.aboveSizeThreshold = function(size) {
      var total;
      if (size && size >= this.pageSizeFeasibilityThreshold) {
        return true;
      }
      if (!size) {
        total = this.model.get('count');
        return total >= this.pageSizeFeasibilityThreshold;
      }
      return false;
    };

    return PageSizer;

  })(CoreView);

}).call(this);
