(function() {
  var Pagination, Paging, SELECT_LIMIT, View, ensureNumber, fs, html, strip, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  SELECT_LIMIT = 200;

  _ = require('underscore');

  

  View = require('../../core-view');

  Paging = require('./paging');

  html = "<ul class=\"pagination\">\n    <li class=\"hidden-xs <%= gotoStart || 'im-goto-start' %>\" title=\"Go to start\">\n        <a class=\"im-pagination-button\">&#x21e4;</a>\n    </li>\n    <li class=\"hidden-xs hidden-sm <%= goFiveBack || 'im-go-back-5' %>\"\n        title=\"Go back five pages\"\n        class=\"visible-desktop\">\n        <a class=\"im-pagination-button\">&#x219e;</a>\n    </li>\n    <li class=\"<%= goOneBack || 'im-go-back-1' %>\"\n        title=\"Go to previous page\">\n        <a class=\"im-pagination-button\">&larr;</a>\n    </li>\n    <li class=\"im-current-page\">\n        <% if (useSelect) { %>\n          <form class=\"im-page-form form form-inline\">\n            <select class=\"form-control\">\n              <% for (i = 0; i < max; i++) { %>\n                <option\n                  <%= selected(i) ? 'selected' : void 0 %>\n                  value=\"<%= i * size %>\">\n                  page <%= i + 1 %>\n                </option>\n              <% } %>\n            </select>\n          </form>\n        <% } else { %>\n          <a href=\"#\">&hellip;</a>\n          <form class=\"im-page-form form form-inline\"\n                style=\"display:none;\">\n            <div class=\"control-group\">\n                <input type=\"number\"\n                       value=\"<%= currentPage %>\"\n                       max=\"<%= max %>\"\n                       min=\"<%= min %>\"\n                       class=\"form-control im-page-number\">\n            </div>\n          </form>\n        <% } %>\n    </li>\n    <li class=\"<%= goOneForward || 'im-go-fwd-1' %>\"\n        title=\"Go to next page\">\n        <a class=\"im-pagination-button\">&rarr;</a>\n    </li>\n    <li class=\"hidden-xs hidden-sm <%= goFiveForward || 'im-go-fwd-5' %>\"\n        title=\"Go forward five pages\"\n        class=\"visible-desktop\">\n        <a class=\"im-pagination-button\">&#x21a0;</a>\n    </li>\n    <li class=\"hidden-xs <%= gotoEnd || 'im-goto-end' %>\"\n        title=\"Go to last page\">\n        <a class=\"im-pagination-button\">&#x21e5;</a>\n    </li>\n</ul>\n";

  strip = function(s) {
    return s.replace(/\s+/g, '');
  };

  ensureNumber = function(raw) {
    if (typeof raw === 'string') {
      return parseInt(strip(raw), 10);
    } else {
      return raw;
    }
  };

  module.exports = Pagination = (function(_super) {
    __extends(Pagination, _super);

    function Pagination() {
      return Pagination.__super__.constructor.apply(this, arguments);
    }

    Pagination.include(Paging);

    Pagination.prototype.tagName = 'nav';

    Pagination.prototype.className = 'im-table-pagination im-table-control';

    Pagination.prototype.RERENDER_EVENT = 'change:start change:count change:size';

    Pagination.prototype.template = _.template(html);

    Pagination.prototype.modelEvents = function() {
      return {
        'change:count': this.setVisible
      };
    };

    Pagination.prototype.getData = function() {
      var count, data, max, size, start, _ref;
      _ref = this.model.toJSON(), start = _ref.start, size = _ref.size, count = _ref.count;
      max = this.getMaxPage();
      return data = {
        max: max,
        min: 1,
        size: size,
        currentPage: this.getCurrentPage(),
        gotoStart: (start === 0 ? 'disabled' : void 0),
        goFiveBack: (start < (5 * size) ? 'disabled' : void 0),
        goOneBack: (start < size ? 'disabled' : void 0),
        gotoEnd: (start >= (count - size) ? 'disabled' : void 0),
        goFiveForward: (start >= (count - 6 * size) ? 'disabled' : void 0),
        goOneForward: (start >= (count - size) ? 'disabled' : void 0),
        selected: function(i) {
          return start === i * size;
        },
        useSelect: max <= SELECT_LIMIT
      };
    };

    Pagination.prototype.postRender = function() {
      this.$('li').tooltip({
        placement: 'top'
      });
      return this.setVisible();
    };

    Pagination.prototype.setVisible = function() {
      var max;
      max = this.getMaxPage();
      return this.$el.toggleClass('im-hidden', max < 2);
    };

    Pagination.prototype.events = function() {
      return {
        'submit .im-page-form': 'pageFormSubmit',
        'click .im-current-page a': 'clickCurrentPage',
        'change .im-page-form select': 'goToChosenPage',
        'blur .im-page-form input': 'pageFormSubmit',
        'click .im-goto-start': (function(_this) {
          return function() {
            return _this.goTo(0);
          };
        })(this),
        'click .im-go-back-5': (function(_this) {
          return function() {
            return _this.goBack(5);
          };
        })(this),
        'click .im-go-back-1': (function(_this) {
          return function() {
            return _this.goBack(1);
          };
        })(this),
        'click .im-go-fwd-5': (function(_this) {
          return function() {
            return _this.goForward(5);
          };
        })(this),
        'click .im-go-fwd-1': (function(_this) {
          return function() {
            return _this.goForward(1);
          };
        })(this),
        'click .im-goto-end': (function(_this) {
          return function() {
            return _this.goTo((_this.getMaxPage() - 1) * _this.model.get('size'));
          };
        })(this)
      };
    };

    Pagination.prototype.goToChosenPage = function(e) {
      var start;
      start = ensureNumber(this.$(e.target).val());
      return this.goTo(start);
    };

    Pagination.prototype.clickCurrentPage = function(e) {
      var size, total;
      size = this.model.get('size');
      total = this.model.get('count');
      if (size >= total) {
        return;
      }
      this.$(e.target).hide();
      return this.$('form').show().find('input').focus();
    };

    Pagination.prototype.pageFormSubmit = function(e) {
      var destination, input, page, pageForm;
      if (e != null) {
        e.stopPropagation();
      }
      if (e != null) {
        e.preventDefault();
      }
      pageForm = this.$('.im-page-form');
      input = this.$('.im-page-form input');
      if (input.size()) {
        destination = ensureNumber(input[0].value);
        if (destination >= 1) {
          page = Math.min(this.getMaxPage(), destination);
          this.goTo((page - 1) * this.model.get('size'));
          this.$('.im-current-page > a').show();
          return pageForm.hide();
        } else {
          pageForm.find('.control-group').addClass('error');
          return inp.val(null);
        }
      }
    };

    return Pagination;

  })(View);

}).call(this);
