(function() {
  var CoreView, ELLIPSIS, Ellipsis, Messages, Options, Step, Templates, UndoHistory, childId, notTrivial, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  CoreView = require('../core-view');

  Templates = require('../templates');

  Messages = require('../messages');

  Options = require('../options');

  Step = require('./undo-history/step');

  childId = function(s) {
    return "state-" + (s.get('revision'));
  };

  ELLIPSIS = -1;

  require('../messages/undo');

  Ellipsis = (function(_super) {
    __extends(Ellipsis, _super);

    function Ellipsis() {
      return Ellipsis.__super__.constructor.apply(this, arguments);
    }

    Ellipsis.prototype.parameters = ['more'];

    Ellipsis.prototype.tagName = 'li';

    Ellipsis.prototype.className = 'im-step im-ellipsis';

    Ellipsis.prototype.template = function() {
      return _.escape("" + (Messages.getText('undo.MoreSteps', {
        more: this.more
      })) + " ...");
    };

    Ellipsis.prototype.attributes = function() {
      return {
        title: Messages.getText('undo.ShowAllStates', {
          n: this.more
        })
      };
    };

    Ellipsis.prototype.postRender = function() {
      return this.$el.tooltip({
        placement: 'right'
      });
    };

    Ellipsis.prototype.events = function() {
      return {
        click: function(e) {
          e.preventDefault();
          e.stopPropagation();
          return this.state.toggle('showAll');
        }
      };
    };

    return Ellipsis;

  })(CoreView);

  notTrivial = function(m, i, ms) {
    var prev;
    prev = ms[i - 1];
    return (i === 0) || (i === ms.length - 1) || (m.get('count') !== prev.get('count'));
  };

  module.exports = UndoHistory = (function(_super) {
    __extends(UndoHistory, _super);

    function UndoHistory() {
      return UndoHistory.__super__.constructor.apply(this, arguments);
    }

    UndoHistory.prototype.parameters = ['collection'];

    UndoHistory.prototype.className = 'btn-group im-undo-history';

    UndoHistory.prototype.template = Templates.template('undo-history');

    UndoHistory.prototype.events = function() {
      return {
        'click .btn.im-undo': 'revertToPreviousState',
        'click .im-toggle-trivial': 'toggleTrivial'
      };
    };

    UndoHistory.prototype.initState = function() {
      return this.state.set({
        showAll: false,
        hideTrivial: false
      });
    };

    UndoHistory.prototype.stateEvents = function() {
      return {
        'change:showAll': this.reRender,
        'change:hideTrivial': this.reRender
      };
    };

    UndoHistory.prototype.collectionEvents = function() {
      return {
        remove: this.removeStep,
        'add remove': this.reRender,
        'change:count': this.reRenderIfHidingTrivial
      };
    };

    UndoHistory.prototype.toggleTrivial = function(e) {
      e.preventDefault();
      e.stopPropagation();
      return this.state.toggle('hideTrivial');
    };

    UndoHistory.prototype.revertToPreviousState = function() {
      return this.collection.popState();
    };

    UndoHistory.prototype.reRenderIfHidingTrivial = function() {
      if (this.state.get('hideTrivial')) {
        return this.reRender();
      }
    };

    UndoHistory.prototype.postRender = function() {
      this.$('.im-toggle-trivial').tooltip({
        placement: 'right'
      });
      this.$list = this.$('.im-state-list');
      this.renderStates();
      this.$el.toggleClass('im-has-history', this.collection.size() > 1);
      return this.$el.toggleClass('im-hidden', this.collection.size() <= 1);
    };

    UndoHistory.prototype.renderStates = function() {
      var coll, cutoff, hideTrivial, i, models, range, showAll, states, _i, _j, _k, _len, _ref, _ref1, _ref2, _ref3, _results, _results1, _results2;
      _ref = this.state.toJSON(), showAll = _ref.showAll, hideTrivial = _ref.hideTrivial;
      coll = this.collection;
      models = hideTrivial ? coll.filter(notTrivial) : coll.models.slice();
      states = models.length;
      cutoff = Options.get('UndoHistory.ShowAllStatesCutOff');
      range = showAll || (states <= cutoff) ? (function() {
        _results = [];
        for (var _i = _ref1 = states - 1; _ref1 <= 0 ? _i <= 0 : _i >= 0; _ref1 <= 0 ? _i++ : _i--){ _results.push(_i); }
        return _results;
      }).apply(this) : (function() {
        _results1 = [];
        for (var _j = _ref2 = states - 1, _ref3 = states - (cutoff - 1); _ref2 <= _ref3 ? _j <= _ref3 : _j >= _ref3; _ref2 <= _ref3 ? _j++ : _j--){ _results1.push(_j); }
        return _results1;
      }).apply(this).concat([ELLIPSIS, 0]);
      _results2 = [];
      for (_k = 0, _len = range.length; _k < _len; _k++) {
        i = range[_k];
        if (i === ELLIPSIS) {
          _results2.push(this.renderEllipsis(states - cutoff));
        } else {
          _results2.push(this.renderState(models[i]));
        }
      }
      return _results2;
    };

    UndoHistory.prototype.renderEllipsis = function(more) {
      return this.renderChild('...', new Ellipsis({
        more: more,
        state: this.state
      }), this.$list);
    };

    UndoHistory.prototype.renderState = function(s) {
      return this.renderChild(childId(s), new Step({
        model: s
      }), this.$list);
    };

    UndoHistory.prototype.removeStep = function(s) {
      return this.removeChild(childId(s));
    };

    return UndoHistory;

  })(CoreView);

}).call(this);
