(function() {
  var Collection, ConstraintList, ConstraintModel, ConstraintsView, CoreView, M, ObjConstructorPM, OrderElementModel, PathModel, QueryProperty, SelectListView, SortOrder, SortOrderView, Templates, UndoStep, ViewList, idsLength, liftPathAndType, valuesLength, withAdded, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  CoreView = require('../../core-view');

  Collection = require('../../core/collection');

  Templates = require('../../templates');

  M = require('../../messages');

  PathModel = require('../../models/path');

  ConstraintModel = require('../../models/constraint');

  OrderElementModel = require('../../models/order-element');

  QueryProperty = require('./query-property-section');

  require('../../messages/undo');

  require('../../messages/constraints');

  withAdded = function(Base) {
    var WithAddedAndRemoved;
    return WithAddedAndRemoved = (function(_super) {
      __extends(WithAddedAndRemoved, _super);

      function WithAddedAndRemoved() {
        return WithAddedAndRemoved.__super__.constructor.apply(this, arguments);
      }

      WithAddedAndRemoved.prototype.defaults = function() {
        return _.extend(WithAddedAndRemoved.__super__.defaults.apply(this, arguments), {
          added: false,
          removed: false
        });
      };

      return WithAddedAndRemoved;

    })(Base);
  };

  ObjConstructorPM = (function(_super) {
    __extends(ObjConstructorPM, _super);

    function ObjConstructorPM(_arg) {
      var path;
      path = _arg.path;
      ObjConstructorPM.__super__.constructor.call(this, path);
    }

    return ObjConstructorPM;

  })(PathModel);

  ViewList = (function(_super) {
    __extends(ViewList, _super);

    function ViewList() {
      return ViewList.__super__.constructor.apply(this, arguments);
    }

    ViewList.prototype.model = withAdded(ObjConstructorPM);

    return ViewList;

  })(Collection);

  ConstraintList = (function(_super) {
    __extends(ConstraintList, _super);

    function ConstraintList() {
      return ConstraintList.__super__.constructor.apply(this, arguments);
    }

    ConstraintList.prototype.model = withAdded(ConstraintModel);

    return ConstraintList;

  })(Collection);

  SortOrder = (function(_super) {
    __extends(SortOrder, _super);

    function SortOrder() {
      return SortOrder.__super__.constructor.apply(this, arguments);
    }

    SortOrder.prototype.model = withAdded(OrderElementModel);

    return SortOrder;

  })(Collection);

  liftPathAndType = function(query) {
    return function(obj) {
      var attrs;
      obj = _.isString(obj) ? {
        path: obj
      } : obj;
      attrs = {
        path: query.makePath(obj.path)
      };
      if (obj.type) {
        attrs.type = query.model.makePath(obj.type);
      }
      return _.extend(attrs, _.omit(obj, 'path', 'type'));
    };
  };

  module.exports = UndoStep = (function(_super) {
    __extends(UndoStep, _super);

    function UndoStep() {
      return UndoStep.__super__.constructor.apply(this, arguments);
    }

    UndoStep.prototype.className = 'im-step';

    UndoStep.prototype.tagName = 'li';

    UndoStep.prototype.template = Templates.template('undo-history-step');

    UndoStep.prototype.getData = function() {
      return _.extend(UndoStep.__super__.getData.apply(this, arguments), {
        diff: this.getCountDiff()
      });
    };

    UndoStep.prototype.getCountDiff = function() {
      if (this.state.has('prevCount')) {
        return this.model.get('count') - this.state.get('prevCount');
      } else {
        return null;
      }
    };

    UndoStep.prototype.initialize = function() {
      var lifter, q, title, _ref;
      UndoStep.__super__.initialize.apply(this, arguments);
      q = this.model.get('query');
      this.initPrevCount();
      lifter = liftPathAndType(q);
      this.listenTo(this.model.collection, 'add remove', this.setCurrent);
      this.views = new ViewList(q.views.map(lifter));
      this.constraints = new ConstraintList(q.constraints.map(lifter));
      this.sortOrder = new SortOrder(q.sortOrder.map(lifter));
      title = this.model.get('title');
      if ((_ref = title.verb) === 'Added' || _ref === 'Removed' || _ref === 'Changed') {
        switch (title.label) {
          case 'column':
            return this.diffView();
          case 'sort order element':
            return this.diffSortOrder();
          case 'filter':
            return this.diffConstraints();
          case 'Initial':
            return null;
          default:
            return console.error('Cannot diff', title.label);
        }
      }
    };

    UndoStep.prototype.initPrevCount = function() {
      var prev;
      if (prev = this.getPrevModel()) {
        this.state.set({
          prevCount: prev.get('count')
        });
        return this.listenTo(prev, 'change:count', (function(_this) {
          return function() {
            return _this.state.set({
              prevCount: prev.get('count')
            });
          };
        })(this));
      }
    };

    UndoStep.prototype.getPrevModel = function() {
      var index;
      index = this.model.collection.indexOf(this.model);
      if (index === 0) {
        return null;
      }
      return this.model.collection.at(index - 1);
    };

    UndoStep.prototype.diff = function(prop, coll, test) {
      var currQuery, e, i, lifter, prev, prevQuery, _i, _j, _len, _len1, _ref, _ref1, _results;
      prev = this.getPrevModel();
      currQuery = this.model.get('query');
      prevQuery = prev.get('query');
      _ref = currQuery[prop];
      for (i = _i = 0, _len = _ref.length; _i < _len; i = ++_i) {
        e = _ref[i];
        if (test(e, prevQuery)) {
          coll.at(i).set({
            added: true
          });
        }
      }
      lifter = liftPathAndType(prevQuery);
      _ref1 = prevQuery[prop];
      _results = [];
      for (i = _j = 0, _len1 = _ref1.length; _j < _len1; i = ++_j) {
        e = _ref1[i];
        if (test(e, currQuery)) {
          _results.push(coll.add(lifter(e), {
            at: i
          }).set({
            removed: true
          }));
        }
      }
      return _results;
    };

    UndoStep.prototype.diffView = function() {
      return this.diff('views', this.views, function(v, _arg) {
        var views;
        views = _arg.views;
        return __indexOf.call(views, v) < 0;
      });
    };

    UndoStep.prototype.diffSortOrder = function() {
      return this.diff('sortOrder', this.sortOrder, function(oe, q) {
        return !_.findWhere(q.sortOrder, oe);
      });
    };

    UndoStep.prototype.diffConstraints = function() {
      return this.diff('constraints', this.constraints, function(c, q) {
        return !_.findWhere(q.constraints, c);
      });
    };

    UndoStep.prototype.initState = function() {
      return this.setCurrent();
    };

    UndoStep.prototype.setCurrent = function() {
      var index, size, _ref;
      if (!((_ref = this.model) != null ? _ref.collection : void 0)) {
        return;
      }
      index = this.model.collection.indexOf(this.model);
      size = this.model.collection.size();
      return this.state.set({
        current: index === size - 1
      });
    };

    UndoStep.prototype.stateEvents = function() {
      return {
        'change': this.reRender
      };
    };

    UndoStep.prototype.modelEvents = function() {
      return {
        'change': this.reRender
      };
    };

    UndoStep.prototype.events = function() {
      return {
        'click .btn.im-state-revert': 'revertToState',
        click: function(e) {
          e.preventDefault();
          return e.stopPropagation();
        }
      };
    };

    UndoStep.prototype.revertToState = function() {
      return this.model.collection.revertTo(this.model);
    };

    UndoStep.prototype.postRender = function() {
      var title;
      this.$details = this.$('.im-step-details');
      this.$el.toggleClass('im-current-state', this.state.get('current'));
      this.$('.btn[title]').tooltip({
        placement: 'right'
      });
      if (this.state.get('current')) {
        return this.renderAllSections();
      } else {
        title = this.model.get('title');
        switch (title.label) {
          case 'column':
            return this.renderViews();
          case 'filter':
            return this.renderConstraints();
          case 'sort order element':
            return this.renderSortOrder();
          case 'Initial':
            return null;
          default:
            return console.error('Cannot render', title.label);
        }
      }
    };

    UndoStep.prototype.renderAllSections = function() {
      this.renderViews();
      this.renderConstraints();
      return this.renderSortOrder();
    };

    UndoStep.prototype.renderViews = function() {
      return this.renderChild('views', new SelectListView({
        collection: this.views
      }), this.$details);
    };

    UndoStep.prototype.renderSortOrder = function() {
      return this.renderChild('so', new SortOrderView({
        collection: this.sortOrder
      }), this.$details);
    };

    UndoStep.prototype.renderConstraints = function() {
      return this.renderChild('cons', new ConstraintsView({
        collection: this.constraints
      }), this.$details);
    };

    return UndoStep;

  })(CoreView);

  SelectListView = (function(_super) {
    __extends(SelectListView, _super);

    function SelectListView() {
      return SelectListView.__super__.constructor.apply(this, arguments);
    }

    SelectListView.prototype.summaryLabel = 'undo.ViewCount';

    SelectListView.prototype.labelContent = function(view) {
      return view.displayName;
    };

    return SelectListView;

  })(QueryProperty);

  SortOrderView = (function(_super) {
    __extends(SortOrderView, _super);

    function SortOrderView() {
      return SortOrderView.__super__.constructor.apply(this, arguments);
    }

    SortOrderView.prototype.summaryLabel = 'undo.OrderElemCount';

    SortOrderView.prototype.labelContent = function(oe) {
      return "" + oe.displayName + " " + oe.direction;
    };

    return SortOrderView;

  })(QueryProperty);

  valuesLength = function(con) {
    return M.getText('constraints.NoOfValues', {
      n: con.values.length
    });
  };

  idsLength = function(con) {
    return M.getText('constraints.NoOfIds', {
      n: con.ids.length
    });
  };

  ConstraintsView = (function(_super) {
    __extends(ConstraintsView, _super);

    function ConstraintsView() {
      return ConstraintsView.__super__.constructor.apply(this, arguments);
    }

    ConstraintsView.prototype.summaryLabel = 'undo.ConstraintCount';

    ConstraintsView.prototype.labelContent = function(con) {
      var parts;
      parts = (function() {
        switch (con.CONSTRAINT_TYPE) {
          case 'MULTI_VALUE':
            return [con.displayName, con.op, con.values.length, valuesLength(con)];
          case 'TYPE':
            return [con.displayName, M.getText('constraints.ISA'), con.typeName];
          case 'IDS':
            return [con.displayName, con.op, idsLength(con)];
          case 'LOOKUP':
            return [con.displayName, con.op, con.value, M.getText('constraints.LookupIn'), con.extraValue];
          default:
            return [con.displayName, con.op, con.value];
        }
      })();
      return parts.join(' ');
    };

    return ConstraintsView;

  })(QueryProperty);

}).call(this);
